Package: MGBT
Type: Package
Title: Multiple Grubbs-Beck Low-Outlier Test
Version: 1.1.6
Depends: R (>= 4.0.0)
Suggests: dataRetrieval, lmomco
Date: 2026-05-21
Authors@R: c(person(c("William", "H."), "Asquith",
                    role = c("aut", "cre"),
                    email = "wasquith@usgs.gov",
		                  comment = c(ORCID = "0000-0002-7400-1861")),
             person(c("John", "F."), "England",
                    role = c("aut", "ctb"),
                    comment = c(ORCID = "0000-0001-5563-6274")),
             person(c("George", "R."),"Herrmann",
                    role = "ctb",
                    comment = c(ORCID = "0000-0003-3829-5580")))
Description: Compute the multiple Grubbs-Beck low-outlier test on positively distributed
 data and utilities for noninterpretive U.S. Geological Survey annual peak-streamflow
 data processing discussed in Cohn et al. (2013) and
 England et al. (2017) <doi:10.3133/tm4B5>. Other utilities for working with peak streamflow
 are provided.
Maintainer: William H. Asquith <wasquith@usgs.gov>
License: CC0
Copyright: This software is in the public domain because it contains
        materials that originally came from the United States
        Geological Survey, an agency of the United States Department of
        Interior. For more information, see the official USGS copyright
        policy at
        https://www.usgs.gov/information-policies-and-instructions/copyrights-and-credits
NeedsCompilation: no
URL: https://doi.org/10.5066/P9CW9EF0
Packaged: 2026-05-26 21:54:54 UTC; wasquith
Author: William H. Asquith [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7400-1861>),
  John F. England [aut, ctb] (ORCID:
    <https://orcid.org/0000-0001-5563-6274>),
  George R. Herrmann [ctb] (ORCID:
    <https://orcid.org/0000-0003-3829-5580>)
Repository: CRAN
Date/Publication: 2026-05-27 05:10:10 UTC
Built: R 4.5.2; ; 2026-05-27 07:33:39 UTC; unix

Package: MGBT
Type: Package
Title: Multiple Grubbs-Beck Low-Outlier Test
Version: 1.1.6
Depends: R (>= 4.0.0)
Suggests: dataRetrieval, lmomco
Date: 2026-05-21
Authors@R: c(person(c("William", "H."), "Asquith",
                    role = c("aut", "cre"),
                    email = "wasquith@usgs.gov",
		                  comment = c(ORCID = "0000-0002-7400-1861")),
             person(c("John", "F."), "England",
                    role = c("aut", "ctb"),
                    comment = c(ORCID = "0000-0001-5563-6274")),
             person(c("George", "R."),"Herrmann",
                    role = "ctb",
                    comment = c(ORCID = "0000-0003-3829-5580")))
Description: Compute the multiple Grubbs-Beck low-outlier test on positively distributed
 data and utilities for noninterpretive U.S. Geological Survey annual peak-streamflow
 data processing discussed in Cohn et al. (2013) and
 England et al. (2017) <doi:10.3133/tm4B5>. Other utilities for working with peak streamflow
 are provided.
Maintainer: William H. Asquith <wasquith@usgs.gov>
License: CC0
Copyright: This software is in the public domain because it contains
        materials that originally came from the United States
        Geological Survey, an agency of the United States Department of
        Interior. For more information, see the official USGS copyright
        policy at
        https://www.usgs.gov/information-policies-and-instructions/copyrights-and-credits
NeedsCompilation: no
URL: https://doi.org/10.5066/P9CW9EF0
Packaged: 2026-05-26 21:54:54 UTC; wasquith
Author: William H. Asquith [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7400-1861>),
  John F. England [aut, ctb] (ORCID:
    <https://orcid.org/0000-0001-5563-6274>),
  George R. Herrmann [ctb] (ORCID:
    <https://orcid.org/0000-0003-3829-5580>)
Repository: CRAN
Date/Publication: 2026-05-27 05:10:10 UTC
Built: R 4.6.0; ; 2026-05-27 05:22:15 UTC; unix

importFrom("stats", "aggregate", "dnorm", "integrate", "pnorm", "pt",
                    "qbeta", "qnorm", "runif", "uniroot", "var")
importFrom("stats", "complete.cases")
importFrom("graphics", "plot", "points", "lines", "rug", "text")
importFrom("graphics", "axTicks", "axis", "legend", "mtext", "par", "polygon")
importFrom("grDevices", "grey", "dev.off", "pdf")
importFrom("utils", "write.table")
#-----------------------------------------------
# Following is not explicitly needed because Suggests: lmomco is used
# and not Depends:
#importFrom("lmomco", "add.log.axis", "f2flo", "lmoms", "pp",
#                     "parpe3", "quape3", "T2prob", "x2xlo")
#-----------------------------------------------
export(ASlo)
export(BLlo)
export(critK10)
export(CondMomsZ)
export(CondMomsChi2)
export(CritK)
export(EMS)
export(gtmoms)
export(GGBK)
export(jointPeaks)
export(RthOrderPValueOrthoT)
export(MGBT)
export(MGBT17c)
export(MGBT17c.verb)
export(MGBTcohn2016)
export(MGBTcohn2013)
export(MGBTnb)
export(MGBTcohn2011)
export(peta)
export(RSlo)
export(V)
export(VMS)
export(makeWaterYear)
export(peakTMtoHRS)
export(plotFFQevol)
export(plotPeaks)
export(plotPeaks_batch)
export(ratioPeakMax1Day)
export(readNWISwatstore)
export(splitPeakCodes)

# README (./MGBT/inst/README.md)

#### Author:           William H. Asquith
#### Point of contact: William H. Asquith (wasquith@usgs.gov)

***
***

This README file provides original source context to the **MGBT** package and other package design notes. These are partial notes and the `man/MGBT-package.Rd` is contains much more information. The `./inst/` subdirectory is standard for _R_ packages. The `CITATION` file is standard for the _R_ language. The documentation below provides a manifest of sorts for the contents of this directory.

# README (`./MGBT/inst/README.md)

***
***

# DIRECTORY `./MGBT/inst/`

**FILE:** `AsquithLOT(1995).pdf` — A hard to find reference concerning low-outlier thresholds in Texas by the author.

**FILE:** `tim-low-outliers-2010-notes.pdf` — John F. England (JFE) visited Reston, Virginia (USGS Headquarters) in Spring 2017 after the passing of Timothy A. Cohn (TAC). He scanned Cohn's personal note books. One is titled "Tim Cohn Low Outliers" and contains 33 pages of mathematical notes. TAC, within `COHN_MGBT_LowOutliers(R).txt`, describes "Orthogonal evaluation of p-value" using "mathematical notes dated December 16, 2009" as spanning 39 pages and that these are "not for the faint of heart ;-)" This note is not found in the file `./sources/P3_089(R).txt`. It does not appear that the 33 pages scanned are the 39 pages being referenced.

**FILE:** `USGSapproval20190924.pdf` — This is a copy of the approval email from USGS internal publication tracking authorizing dissemination (release) of the product.

## Subdirectory `./MGBT/inst/helpers/`

Convenient research scripts related to experimental efforts by William H. Asquith (WHA). These will not necessarily do anything for end users.

## Subdirectory `./MGBT/inst/legend/`

This directory contains reference information and portable document format (PDF) files related to explanation of the `MGBT::plotPeaks()` function.

## Subdirectory `./MGBT/inst/sources/`

This directory holds original source code acquired from TAC at various times by colleagues. It is advised to see also the documentation `./MGBT/man/MGBT-package.Rd`.

### Source Code File `./sources/FigureMacros_jfe(R).txt`

Source code exchanged between TAC and JFE focused on low-outlier thresholds (potentially influential low floods).

### Source Code File `./sources/LowOutliers_jfe(R).txt`

Source code exchanged between TAC and JFE focused on low-outlier thresholds (potentially influential low floods).


### Source Code File `./sources/LowOutliers_wha(R).txt`

On or about August 21, 2013, TAC sent a file in an email attachment titled `COHN_MGBT_LowOutliers.R` to WHA. At the time, WHA was needing an MGBT implementation for a study of "modern periods of statitic flood flow regulation" through flood frequency computations on annual peak streamflow data for USACE Fort Worth. Asquith was using the method of L-moments, but wanted the MGBT as protection against some mixed population effects. This communication to WHA appears to be 100 percent independent of from TAC's personal _R_ code base related to the Expected Moments Algorithm (EMA) and Bulletin 17C (log-Pearson type III distribution) communicated with JFE in the years prior. This file has been retitled to `LowOutliers_wha(R).txt`.

The source files have a `*(R).txt` in their name. This is deliberate. They are _R_ scripts, but _R_ will complain on an `R CMD check MGBT` when any `.R` file exits in the `./inst/` subdirectory of a package. Cohn has many functions related to low-outliers in his code bases, but many are not directly related to MGBT for purposes of implementation.  All of these are ported into the MGBT package and documented. It is clear that Cohn shared functions with Asquith in `./LowOutliers_wha(R).txt` that are not available in the file `./sources/P3_089(R).txt`.  The typical design of _R_ packages by WHA to break as much code into separate `.R` files as possibly (logically) and generally have about a one-to-one pairing of `.Rd` files.

### Source Code File `./sources/P3_075_jfe(R).txt`

Source code exchanged between TAC and JFE at some point in time.

### Source Code File `./sources/P3_085_wha(R).txt`

Source code exchanged between TAC and WHA in November 2015.

### Source Code File `./sources/P3_089(R).txt`

On May 16, 2017, Julie E. Kiang sent was she believes to be TAC's master `.R` file of the EMA-17C code. Within which separate "MGBT" functions from the WHA communication appear present. The short comments by TAC indicate changes as late as June 28, 2016. These are left intact. The **MGBT** package will first pass into maturity as CRAN-compliant through communication between TAC and WHA and then port evidently TAC's later developments. Asquith suggests appending "2013" to the original functions (e.g. MGBT.2013) given to Asquith and eventually use `MGBT()` as the canonical version as embedded into 17C.

## Subdirectory `./MGBT/www/`

This directory is a holding ground for files to enrich the Internet presentation of the package.

# SOME WORKING NOTES ON THE MGBT PACKAGE

Nancy A. Barth has encountered a situation where USGS PeakFQ software and the **MGBT** package at first glance appear divergent from each other in regards to the threshold defined by the MGBT. For now (summer 2019), this paragraph just lists USGS streamflow-gaging station number(s) of such sites; further study is needed.

* 06430898 for the period of record of 1989–2013

# README (path ./MGBT/inst/www/README.md)

#### Author:           William H. Asquith
#### Point of contact: William H. Asquith (wasquith@usgs.gov)

***
***

This README file describes the contents of the above path. This directory is to contain
various files to augment the Internet viewing of the code base as marked up by GitHub,
GitLab, or other markup resources. The avatar for the project itselt is contained herein
along with example graphics as needed for the README file in the home directory.

README (./MGBT/inst/sources/README.md)

Author:           William H. Asquith
Point of contact: William H. Asquith (wasquith@usgs.gov)

------------------------------------------------------------------------------------------

The sources files herein were acquired over a period of time from the late Tim A. Cohn
through written communication between William H. Asquith and John F. England. Several
sources are clearly related to low outliers and scripts for article generation.
Other sources `P3_***_***(R).txt` contain low outlier code but the bulk of which
is used for log-Pearson type III (LPIII), expected moments algorithm (EMA) computation
of peak-streamflow frequency. This extra code is not needed for the MGBT package
and IS NOT TO BE CONSIDERED CANONICAL OF LPIII-EMA-like computations! Please
feel invited to contact John F. England or William H. Asquith for further details about
LPIII-EMA computations.

The sources herein contained in the `*(R).txt` will never be eligible for updates.
They are locked down forever. Finally, the `(R).txt` in the file names is deliberate to
prevent _R_ during `R CMD check check MGBT` from throwing exceptions to _R_ code being
contained within the `./inst` subdirectory of the package.

README (path ./MGBT/inst/helpers/README.md)

Author:           William H. Asquith
Point of contact: William H. Asquith (wasquith@usgs.gov)

------------------------------------------------------------------------------------------

The files herein are used to provide representative working notes of the author in the development of some tools for data-mining study of the U.S. Geological Survey (USGS) peak-streamflow database. The file manifest follows:

1. `testsites.pdf` --- This file is a portable document format (PDF) graphic from the `MGBT::plotPeaks()` function call made through the _R_ script `vispks(R).txt` where the `.txt` extension is used to keep `R CMD check MGBT` from complaining about _R_ sources in the package that are not in the `R/MGBT` directory.

2. `testsites.txt` --- This file is a short list of various USGS streamflow-gaging stations for which the peaks will be retrieved using the **dataRetrieval** package and plotted by the **MGBT** package in file `testsites.pdf`. The code comments therein provide further details. These sites are somewhat randomly chosen and mostly in Texas, however, some thought in the choice is made to stress the `MGBT::plotPeaks()` alogrithms and the `MGBT()` itself.

3. `vispks(R).txt` --- This is an _R_ script. (See no. 1.)

README (path ./MGBT/inst/legend/README.md)

Author:           William H. Asquith
Point of contact: William H. Asquith (wasquith@usgs.gov)

------------------------------------------------------------------------------------------

This directory contains reference information and portable document format (PDF) files related to the legend or explanation of the `MGBT::plotPeaks()` function. The file manifest follows:

1. `legend_camera.pdf` --- This file is PDF and is the "camera ready" version of the raw output (`peaks_for_legend.pdf`) from `makePeakLegend(R).txt` that is an _R_ script but ends in a `.txt` convention to keep the `R CMD check MGBT` on the package from complaining that _R_ source code exists outside of the `./MGBT/R/` directory.

2. `makePeakLegend(R).txt` --- This file is an _R_ script. (See no. 1.)

3. `peaks_for_legend.pdf` --- The output from `makePeakLegend(R).txt`.

4. `peaks_fot_legend.txt` --- The input for `makePeakLegend(R).txt` to make `makePeakLegend(R).txt`. And the peaks therein and the attendant peak discharge qualification codes have been manually adjusted by the author from a real U.S. Geological Streamflow (USGS) gaging station in order to comprehensively demonstrate most the the possible permutations that could be seen across the entire USGS peak-streamflow database. The sole purpose of this file is to have a source basis on which the `peaks_for_legend.pdf` could be created and then manually edited to create `legend_camera.pdf`.

"ASlo" <- function(mu, sigma, gamma) {
   isInside <- TRUE
   if(mu    <  1.9   | mu    > 4.842) isInside <- FALSE
   if(sigma <  0.125 | sigma > 1.814) isInside <- FALSE
   if(gamma < -2.714 | gamma > 0.698) isInside <- FALSE
   if(! isInside) {
      warning("input data outside of the potential applicable range of the regression")
   }
   10^(1.09*mu - 0.584*sigma + 0.14*gamma - 0.799)
}

.onAttach <- function(libname, pkgname) {
  packageStartupMessage(paste(
  "USGS Research Package:
https://owi.usgs.gov/R/packages.html#research
This package is an approved software release, and
is shown to be consistent with USGS-PeakFQ software.",
  collapse="\n"))
}

#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN   Conditional moments
#COHN     N.B. Moments employ only obs. above xsi
#COHN
#COHN "CondMomsZ" <- function(n,r,xsi){
#COHN   cbind( gtmoms(xsi,1) , (gtmoms(xsi,2)-gtmoms(xsi,1)^2)/(n-r) )
#COHN }
#COHN "CondMomsChi2" <- function(n,r,xsi){
#COHN   mu <- gtmoms(xsi,1);
#COHN   cbind(gtmoms(xsi,2) - mu^2 ,
#COHN   V(n,r,pnorm(xsi))[2,2])
#COHN }

"CondMomsZ"    <- function(n, r, xsi) {
   cbind( gtmoms(xsi,1), ( gtmoms(xsi,2) - gtmoms(xsi,1)^2) / (n-r) )
}

"CondMomsChi2" <- function(n, r, xsi) {
   cbind( gtmoms(xsi,2) - gtmoms(xsi,1)^2, V(n,r,pnorm(xsi))[2,2] )
}

# CondMomsZ does not appear used in COHN_MGBT_LowOutliers(R).txt
# CondMomsChi2 is used in COHN_MGBT_LowOutliers(R).txt
# Does N.B. stand for nonbiased? (WHA 05/23/2017)
# Does N.B. stand for NoBackup procedure, which seems some type of note for MGBTnb().
#   (WHA 06/06/2017)

"makeWaterYear" <- function(x, datestr="peak_dt") {
   if(is.character(x)) {
     zz <- data.frame(date=x)
     zz$year_va  <- as.numeric(sapply(x, function(t) strsplit(t, split="-")[[1]][1]))
     zz$month_va <- as.numeric(sapply(x, function(t) strsplit(t, split="-")[[1]][2]))
     zz$day_va   <- as.numeric(sapply(x, function(t) strsplit(t, split="-")[[1]][3]))
     zz$water_yr <- zz$year_va
     zz$water_yr[! is.na(zz$month_va) & zz$month_va >= 10] <-
     zz$water_yr[! is.na(zz$month_va) & zz$month_va >= 10] + 1
     zz$month_va[zz$month_va == 0] <- NA
     zz$day_va[  zz$day_va   == 0] <- NA
     hh <- aggregate(zz[, "water_yr"], by=list(zz$water_yr), length)
     hh <- hh[hh$x > 1,]
     if(nrow(hh) > 0) {
       for(i in 1:nrow(hh)) {
         warning("water year ", hh$Group.1[i], " occurs more than once (x", hh$x[i], ")")
       }
     }
     return(zz)
   }
   if(is.data.frame(x)) {
     if(exists(datestr, x)) {
       d <- x[ , datestr]
       x$year_va  <- as.numeric(sapply(d, function(t) strsplit(t, split="-")[[1]][1]))
       x$month_va <- as.numeric(sapply(d, function(t) strsplit(t, split="-")[[1]][2]))
       x$day_va   <- as.numeric(sapply(d, function(t) strsplit(t, split="-")[[1]][3]))
       x$water_yr <- x$year_va
       x$water_yr[! is.na(x$month_va) & x$month_va >= 10] <-
       x$water_yr[! is.na(x$month_va) & x$month_va >= 10] + 1
       x$month_va[x$month_va == 0] <- NA
       x$day_va[  x$day_va   == 0] <- NA
       hh <- aggregate(x[, "water_yr"], by=list(x$water_yr), length)
       hh <- hh[hh$x > 1,]
       if(nrow(hh) > 0) {
         for(i in 1:nrow(hh)) {
           warning("water year ", hh$Group.1[i], " occurs more than once (x", hh$x[i], ")")
         }
       }
       return(x)
     } else {
       warning("'datestr' does not exist in the data.frame 'x', returning unmodified 'x'")
       return(x)
     }
   } else {
     warning("a data.frame with 'peak_dt' as in ",
             "'dataRetrieval::readNWISpeak(site_no, convertType=FALSE)' required")
     return(x)
   }
}

#peta <- function(pzr,n,r,eta){
#                   zr       <- qnorm(qbeta(pzr,shape1=r,shape2=n+1-r))
#                   CV       <- VMS(n,r,qmin=pnorm(zr))
#                   lambda   <- CV[1,2]/CV[2,2]
#                   etap     <- eta + lambda
#                   EMp      <- EMS(n,r,qmin=pnorm(zr))
#                   muMp     <- EMp[1]-lambda*EMp[2]
#                   SigmaMp  <- sqrt(CV[1,1] - CV[1,2]^2/CV[2,2]);
#                   momS2    <- CondMomsChi2(n,r,zr)
#                   shape    <- momS2[1]^2/momS2[2]
#                   scale    <- momS2[2]/momS2[1]
#                   sqrtS2   <- sqrt(momS2[1])
#                   df       <- 2*shape
#                   ncp      <- (muMp-zr)/SigmaMp
#                   q        <- -(sqrtS2/SigmaMp)*etap
#               (1 - pt(q,df=df, ncp=ncp, lower.tail = TRUE))
#     }

"peta" <- function(pzr, n, r, eta) {
   sapply(pzr, function(the.pzr) {
      br     <- qbeta(the.pzr, shape1=r, shape2=n+1-r) # isolated by WHA
      zr     <- qnorm(br) # so that br can be used twice without need for pnorm().
      COV    <- VMS(n, r, qmin=br)
      EMp    <- EMS(n, r, qmin=br)
      lambda <- COV[1,2] / COV[2,2] # renamed to be more COVariance like
      etap   <- eta    + lambda
      muMp   <- EMp[1] - lambda*EMp[2]
      suppressWarnings(SigmaMp  <- sqrt(COV[1,1] - COV[1,2]^2/COV[2,2]))  # WHA error trapping
      if(! is.finite(SigmaMp)) return(1) # TODO, Is unity okay? Seems so  # WHA error trapping
      momS2    <- CondMomsChi2(n,r,zr)
                       # shape <- momS2[1]^2/momS2[2]  # WHA: only needed for df
                       # scale <- momS2[2]  / momS2[1] # WHA: not used by TAC
      q   <- -(sqrt(momS2[1])  / SigmaMp)*etap # a quantile for pt()
      df  <-     2*(momS2[1]^2 / momS2[2])     # degrees of freedom
      ncp <-         (muMp-zr) / SigmaMp       # non-centrality parameter
      #if(abs(ncp) > 37.62) { # WHA limit exploring of pt(), 37.62 from R documentation
      #   message("Exploring limits of peta:",q, ", df=",df," and ncp"=ncp)
      #}
      suppressWarnings(the.pt <- pt(q, df=df, ncp=ncp, lower.tail=TRUE))
      return(1 - the.pt)
   })
}

# Peaks <- c(53, 88, 55, 45, 40); MGBT(Peaks)
# if(! is.finite(SigmaMp)) return(1)
#$pvalues for 40 and 45
#[1] 0.8139058 0.8011052

#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   Expected values of M and S
#COHN
"EMS" <- function(n, r, qmin) {
         if(length(qmin) != 1) {
            warning("not expecting a vector for qmin, using only the first value")
            qmin <- qmin[1]
         }
         zr       <- qnorm(qmin)
         Em       <- gtmoms(zr,1)
         momS2    <- CondMomsChi2(n, r, zr)
           alpha  <- momS2[1]^2 / momS2[2]
           beta   <- momS2[2]   / momS2[1]
         suppressWarnings(Es <- sqrt(beta)*exp( lgamma(alpha+0.5) - lgamma(alpha)))
         c(Em[1], Es)
}


#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   Covariance matrix of M and S
#COHN
#COHN "VMS" <- function(n,r,qmin){
#COHN          E    <- sapply(1:2,function(i){gtmoms(qnorm(qmin),i)});
#COHN          Es   <- EMS(n,r,qmin)[2]
#COHN          Es2  <- E[2]-E[1]^2
#COHN          V2   <- V(n,r,qmin)
#COHN          array(c(
#COHN                  V2[1,1],
#COHN                  rep(V2[1,2]/(2*Es),2),
#COHN                  Es2-(Es)^2
#COHN                 ),dim=c(2,2))
#COHN }

"VMS" <-
function(n, r, qmin) {
   if(length(qmin) != 1) {
      warning("not expecting a vector for qmin, using only the first value")
      qmin <- qmin[1]
   }
   E   <- sapply(1:2, function(i) gtmoms(qnorm(qmin), i))
   Es  <- EMS(n, r, qmin)[2]
   Es2 <- E[2] - E[1]^2
   V2  <- V(n,r,qmin)
   array(c(V2[1,1], rep(V2[1,2]/(2*Es),2), Es2-(Es)^2),
         dim=c(2,2))
}

#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   Covariance matrix of M and S^2
#COHN     Note that E is vector of expected values of X^k given X>qmin
#COHN
#COHN "V" <- function(n,r,qmin){
#COHN         n2 <- n-r
#COHN       E<-sapply(1:4,function(i){gtmoms(qnorm(qmin),i)});
#COHN 	   cm      <- c(E[1],
#COHN 					E[2]-E[1]^2,
#COHN 					E[3]-3*E[2]*E[1]+2*E[1]^3,
#COHN 					E[4]-4*E[3]*E[1]+6*E[2]*E[1]^2-3*E[1]^4
#COHN 				   )
#COHN       array(
#COHN         c((E[2] - E[1]^2)/n2,
#COHN         rep(
#COHN           (E[3] - 3*E[1]*E[2] + 2*E[1]^3)/sqrt(n2*(n2-1))
#COHN           ,2),
#COHN           (cm[4]-cm[2]^2)/n2 + 2/((n2-1)*n2)*cm[2]^2),
#COHN        dim=c(2,2))
#COHN }

"V" <- function(n, r, qmin) {
   if(length(qmin) != 1) {
      warning("not expecting a vector for qmin, using only the first value")
      qmin <- qmin[1]
   }
   k  <- n-r
   E  <- sapply(1:4, function(i) gtmoms(qnorm(qmin),i) )
	  cm <- c(E[1],
					      E[2] -   E[1]^2,
					      E[3] - 3*E[2]*E[1] + 2*E[1]^3,
					      E[4] - 4*E[3]*E[1] + 6*E[2]*E[1]^2 - 3*E[1]^4)
   array(  c( cm[2]/k, # streamline because TAC has this without the k already computed
         rep( cm[3]/sqrt(k*(k-1)), 2),  # streamline because TAC has this without the k already computed
             (cm[4] -  cm[2]^2)/k + 2/((k-1)*k)*cm[2]^2),
         dim=c(2,2))
}

readNWISwatstore <-
function(siteNumbers, path=".", dirend="d", return.progress=FALSE,
                      tabpk=TRUE, vispk=TRUE, vispdf=TRUE,
                      unlinkpath=FALSE,  citeNWISdoi=TRUE,
                      all_peaks_na_okay=TRUE, ...) {
  #https://nwis.waterdata.usgs.gov/nwis/peak?site_no=09416000
  #                                          &agency_cd=USGS&format=hn2

  headit <- "https://nwis.waterdata.usgs.gov/nwis/peak?site_no="
  tailit <- "&agency_cd=USGS&format=hn2"

  gotThePeaks <- vector(mode="logical", length(siteNumbers))
  i <- 0
  for(site in siteNumbers) {
    i <- i + 1
    fullpath <- paste0(path, "/", site, dirend)
    if(unlinkpath) unlink(fullpath, recursive=TRUE)
    if(! dir.exists(fullpath)) dir.create(fullpath)

    FH <- file(paste0(headit, site, tailit))
    pk <- readLines(FH); close(FH)

    cat(pk, file=paste0(fullpath, "/", site, ".pkf"), sep="\n")

    if(vispk | tabpk) {
      pkall <- dataRetrieval::readNWISpeak(site, convert=FALSE)
      if(nrow(pkall) == 0) {
        message("  for streamgage '", site, "'")
        gotThePeaks[i] <- FALSE
        next
      }
      pkall <- splitPeakCodes(pkall, all_peaks_na_okay=all_peaks_na_okay)
      sname <- dataRetrieval::readNWISsite(site)
      sname <- toupper(sname$station_nm[sname$agency_cd == "USGS"])
    }

    if(vispk) {
        file <- paste0(fullpath, "/", site, ".pdf")
      if(vispdf) pdf(file, useDingbats=FALSE)
        plotPeaks(pkall, codes=TRUE, showDubNA=TRUE, site=site, mtext.site=FALSE,
                  xlab="Water year", ylab="Peak streamflow, in cubic feet per second",
                  ...)
        mtext(paste0(site, " ", sname), line=0.5)
      if(vispdf) dev.off()
    }

    if(tabpk) {
      file <- paste0(fullpath, "/", site, ".txt")
      write.table(pkall, file=file, sep="\t", row.names=FALSE, quote=FALSE)
    }

    if(citeNWISdoi) {

      tx <- date() # going to workout the ability to make a contemporaneous citation to NWIS!
      yr <- strsplit(tx, "\\s+")[[1]][5]
      mn <- strsplit(tx, "\\s+")[[1]][2]
      dd <- strsplit(tx, "\\s+")[[1]][3]
      tx <- paste0(mn, " ", dd, ", ", yr)

      txt <-
       paste0("# CITATION.md---U.S. Geological Survey Peak Streamflow Data\n",
              "\n",
              "***\n",
              "***\n",
              "\n",
              "The peak streamflow data contained in this directory were acquired from\n",
              "U.S. Geological Survey (USGS) (", yr, ") using a combination of the **dataRetrieval**\n",
              "and **MGBT** _R_ packages. These packages are referenced below and deliberately\n",
              "left undated because basic **dataRetrieval** operations for reading peaks are used\n",
              "and technically, no features in **MGBT** itself were used for data acquisition,\n",
              "just plotting. However, if a user's system was reasonably up to date, then year\n",
              "and version numbers of the respective packages can be inferred by the access date\n",
              "emphasized here for the USGS National Water Information System database retrieval.\n",
              "\n",
              "# REFERENCES\n\n",
              "Asquith, W.H., England, J.F., and Herrmann, G.R., MGBT---Multiple Grubbs--Beck\n",
              "   low-outlier test: https://cran.r-project.org/package=MGBT.\n\n",
              "DeCicco, L., Hirsch, R., Lorenz, D., Read, J., Walker, J., Carr, L., and Watkins, D.,\n",
              "   dataRetrieval---Retrieval functions for USGS and EPA hydrologic and water quality data:\n",
              "   https://cran.r-project.org/package=dataRetrieval.\n\n",
              "U.S. Geological Survey, ",yr,", USGS water data for the Nation:\n",
              "   U.S. Geological Survey National Water Information System database,\n",
              "   accessed ",tx,", at https://doi.org/10.5066/F7P55KJN.\n",
              "   [ peak streamflows in WATSTORE format by direct URL at\n",
              "     ", headit, site, tailit, " ]")
      cat(txt, file=paste0(fullpath, "/CITATION.md"), sep="\n")
    }
    gotThePeaks[i] <- TRUE
  }
  if(return.progress) return(data.frame(site=siteNumbers, gotThePeaks=gotThePeaks))
}

"plotFFQevol" <-
function(pkenv, lot=NULL, finalquas=NULL, log10offsets=NULL,
         minyrs=10, byr=1940, edgeyrs=c(NA, NA), logyaxs=TRUE,
         lego=NULL, maxs=NULL, mins=NULL, names=NULL, auxeyr=NA,
         xlab="Water Year", ylab="Peak streamflow, in cubic feet per second",
         title="Time Evolution of Peak-Frequency Streamflow Estimates\n",
         data_ext="_data.txt", ffq_ext="_ffq.txt", path=tempdir(),
         showfinalyr=TRUE, usewyall=FALSE, silent=TRUE, ...) {

  if(! is.environment(pkenv)) {
    warning("require an environment holding the peaks along the lines of:\n",
            "pks <- dataRetrieval::readNWISpeak(station, convertType=FALSE);\n",
            "pks <- MGBT::splitPeakCodes(MGBT::makeWaterYear(data))")
    return(NULL)
  }
  if(! is.data.frame(finalquas)) {
    warning("require a data.frame holding the final quantiles and the final water year\n")
    return(NULL)
  }

  plotem <- ifelse(is.null(log10offsets), FALSE, TRUE)
  stations <- ls(pkenv)
  tyrs <- c(2, 5, 10, 25, 50, 100, 200, 500)
  FF <- lmomco::T2prob(tyrs)

  Maroon <- grDevices::rgb(159/256,  59/256,  56/256)
  Purple <- grDevices::rgb(112/256,  48/256, 160/256)
  Green  <- grDevices::rgb(175/256, 201/256, 122/256)
  Blue   <- grDevices::rgb(114/256, 154/256, 202/256)

  n <- length(stations)
  zz <- data.frame(site_no=rep(NA,n), Q002=rep(NA,n), Q005=rep(NA,n),
                      Q010=rep(NA,n), Q025=rep(NA,n), Q050=rep(NA,n),
                      Q100=rep(NA,n), Q200=rep(NA,n), Q500=rep(NA,n))
  zz <- zz[0,] # will become the offset values

  for(i in 1:length(stations)) {
     station <- stations[i]; lothres <- lot[i]
     if(! silent) message("Working on ",station, " ", appendLF=FALSE)
     fv  <- finalquas[  finalquas$site_no == station, ]
     if(length(fv$site_no) == 0) {
       warning("could not find ",station," on the finalquas argument data.frame")
       next
     }
     fvo <- log10offsets[log10offsets$site_no == station, ]
     if(is.null(fvo)) {
       fvo <- data.frame(site_no=station,
                         Q002=0, Q005=0, Q010=0, Q025=0,
                         Q050=0, Q100=0, Q200=0, Q500=0)
     } else if(length(fvo$site_no) == 0) {
       warning("could not find ",station," on the logoffsets argument data.frame")
       next
     }
     minyears <- range(c(byr,fv$final_water_yr, auxeyr), na.rm=TRUE)
     minyears <- minyears[1]:minyears[2]
     ffq050 <- ffq025 <- ffq010 <- ffq005 <- ffq002 <-
        data.frame(water_yr=minyears, quantile=rep(NA, length(minyears)))
     ffq500 <- ffq200 <- ffq100 <- ffq050
     pks <- get(station, pkenv); pks <- pks[! is.na(pks$peak_va), ]
     Qall <- pks$peak_va; Wall <- pks$water_yr; appearsSystematic.all <- pks$appearsSystematic
     if(plotem) {
       xlim <- range(c(edgeyrs, Wall[pks$appearsSystematic]), na.rm=TRUE)
       if(usewyall) xlim <- range(c(edgeyrs, Wall[! pks$appearsSystematic]), na.rm=TRUE)
       ylim <- c(mins[i],maxs[i])
     }
     if(plotem & logyaxs) {
        tmp <- c(mins[i],maxs[i]); tmp <- tmp[tmp > 0]
        plotPeaks(pks, lot=lothres, xlab=xlab, ylab=ylab, xlim=xlim, xlim.inflate=FALSE, aux.y=tmp)
     } else if(plotem) {
        plot(Wall[appearsSystematic.all],Qall[appearsSystematic.all], xaxs="i", yaxs="i", xlim=xlim,
             ylim=ylim, pch=16, tcl=0.5,
             xlab=xlab, ylab=ylab)
        points(Wall[! appearsSystematic.all], Qall[! appearsSystematic.all], pch=16, cex=1, col=6)
    }
    if(plotem) {
      if(showfinalyr) lines(rep(fv$final_water_yr,2), par()$usr[3:4], lty=4, lwd=0.8)
      lines(xlim, rep(lothres, 2), lty=2, lwd=0.8)
      if(! is.null(title)) mtext(paste0(title, names[i]), cex=0.90)
    }
    for(j in 1:length(minyears)) {
       minyear <- minyears[j]
       pks <- pks[pks$appearsSystematic,]
       W <- pks$water_yr[pks$water_yr <= minyear]
       Q <- pks$peak_va[ pks$water_yr <= minyear]
       if(length(Q) < minyrs) {
         if(! silent) message(" skipping ", minyear)
         next;
       }
       xlo <- lmomco::x2xlo(Q, leftout=lothres, a=0); #print(xlo); stop()
       lmr <- lmomco::lmoms(log10(xlo$xin), nmom=3) # we don't need more than
       pe3lmr <- lmomco::parpe3(lmr) # three to fit the Pearson.
       ffq <- 10^lmomco::quape3(lmomco::f2flo(FF, pp=xlo$pp), pe3lmr)
       ffq002[j, 2] <- ffq[1]; ffq005[j, 2] <- ffq[2]; ffq010[j, 2] <- ffq[3]
       ffq025[j, 2] <- ffq[4]; ffq050[j, 2] <- ffq[5]; ffq100[j, 2] <- ffq[6]
       ffq200[j, 2] <- ffq[7]; ffq500[j, 2] <- ffq[8]

       if(minyear == fv$final_water_yr) {
         if(! silent) message("intercepting final water year=", minyear)
         offQ002 <- log10(fv$Q002/ffq002[j,2]); offQ005 <- log10(fv$Q005/ffq005[j,2])
         offQ010 <- log10(fv$Q010/ffq010[j,2]); offQ025 <- log10(fv$Q025/ffq025[j,2])
         offQ050 <- log10(fv$Q050/ffq050[j,2]); offQ100 <- log10(fv$Q100/ffq100[j,2])
         offQ200 <- log10(fv$Q200/ffq200[j,2]); offQ500 <- log10(fv$Q500/ffq500[j,2])
         zz <- rbind(zz, data.frame(site_no=station,
                          Q002=offQ002, Q005=offQ005, Q010=offQ010, Q025=offQ025,
                          Q050=offQ050, Q100=offQ100, Q200=offQ200, Q500=offQ500))
       }
       ffq002[j, 2] <- 10^(log10(ffq002[j, 2]) + fvo$Q002)
       ffq005[j, 2] <- 10^(log10(ffq005[j, 2]) + fvo$Q005)
       ffq010[j, 2] <- 10^(log10(ffq010[j, 2]) + fvo$Q010)
       ffq025[j, 2] <- 10^(log10(ffq025[j, 2]) + fvo$Q025)
       ffq050[j, 2] <- 10^(log10(ffq050[j, 2]) + fvo$Q050)
       ffq100[j, 2] <- 10^(log10(ffq100[j, 2]) + fvo$Q100)
       ffq200[j, 2] <- 10^(log10(ffq200[j, 2]) + fvo$Q200)
       ffq500[j, 2] <- 10^(log10(ffq500[j, 2]) + fvo$Q500)
    }
    if(plotem) {
      lines(minyears, ffq002$quantile, lwd=1.25, col=Blue  )
      lines(minyears, ffq010$quantile, lwd=1.25, col=Green )
      lines(minyears, ffq100$quantile, lwd=1.25, col=Purple)
      lines(minyears, ffq500$quantile, lwd=1.25, col=Maroon)
      if(! is.null(lego)) {
        legend(lego[i], maxs[i],
             c("Annual peak streamflow value (filled grey to avoid confusion)",
               "2-year return period estimate of streamflow",
               "10-year return period estimate of streamflow",
               "100-year return period estimate of streamflow",
               "500-year return period estimate of streamflow",
               "Low-outlier threshold used (if line is present)"),
              bty="n", lwd=c(NA,rep(1.25, 4),0.8), lty=c(NA,1,1,1,1,2),
              pch=c(21,NA,NA,NA,NA,NA), pt.bg=c(8,NA,NA,NA,NA,NA),cex=0.85,
              col=c(1, Blue, Green, Purple, Maroon))
      }
    }
    datafile <- paste0(path,"/",stations[i],data_ext)
    if(! is.na(data_ext)) {
       if(! silent) message("writing external datafile=",datafile)
       write.table(data.frame(water_yr=Wall, peak_va=Qall, appearsSystematic=appearsSystematic.all),
                   file=datafile, sep=",", quote=FALSE, row.names=FALSE)
    }
    tmp <- merge(ffq002, ffq010, all=TRUE, by="water_yr")
    names(tmp) <- c("water_yr", "ffq002", "ffq010")           # see Warning below
    tmp <- merge(tmp,    ffq100, all=TRUE, by="water_yr")
    names(tmp) <- c("water_yr", "ffq002", "ffq010", "ffq100") # see Warning below
    tmp <- merge(tmp,    ffq500, all=TRUE, by="water_yr")     # see Warning below
    names(tmp) <- c("water_yr", "ffq002", "ffq010", "ffq100", "ffq500")
    #Warning in merge.data.frame(tmp, ffq500, all = TRUE, by = "water_yr") :
    #column names 'quantile.x', 'quantile.y' are duplicated in the result
    # Technically, the third merge triggers a warning. Instead of suppressWarnings()
    # wrapper, it is felt better to just continuous rename the columns for better documentation.

    ffqfile <- paste0(path,"/",stations[i],ffq_ext)
    if(! is.na(ffq_ext)) {
      if(! silent) message("writing external ffqfile=",ffqfile)
      write.table(tmp, file=ffqfile, sep=",", quote=FALSE, row.names=FALSE)
    }
  }
  if(! plotem) return(zz)
}

#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   New Generalized Grubbs-Beck critical values for 10% test; TAC Approximation
#COHN
#COHN  fw  <- function(n,w){n*w/((n-1)*sqrt((1/(n-2))*(n-1-(n/(n-1))*w^2)))}
#COHN  fw1 <- function(n,w1){-sqrt(((n-1)^3*w1^2)/(n^2*(n-2)+w1^2*n*(n-1)))}
#COHN GGBK  <-function(n) fw1(n,CritK(n,1,.1,-fw(n,critK10(n))))

"GGBK" <- function(n)   {
   fw  <- function(n, w ) {
             n*w / ((n-1) * sqrt((1/(n-2)) * (n-1 - (n/(n-1))*w^2) )) }
   fw1 <- function(n, w1) {
             -sqrt( ((n-1)^3*w1^2) / (n^2*(n-2) + w1^2*n*(n-1)) ) }
   #return( fw1(n, CritK(n, 1, 0.1, -fw(n, critK10(n)) )) )
   stop("GGBK function is incomplete, its CritK() call is the issue, and ",
        "that function requires three arguments, but this code in ",
        "Timothy A. Cohn's sources passes four!")
}

# GGBK is the consumer of fw() and fw1(). These should be defined within it
# to keep the scope local.

"peakTMtoHRS" <- function(x, type=c("asis", "musd"), parseHHMM=TRUE) {
  type <- match.arg(type)

  if(is.data.frame(x)) {
    if(exists("peak_tm", x)) tm <- x$peak_tm
  } else{
    tm <- x
  }
  # tm <- paste0(floor(tm), ":", as.integer(60*(tm - floor(tm))))
  if(parseHHMM) {
    tm <- strsplit(tm, ":")
    fd <- as.numeric(sapply(seq_len(length(tm)), function(i) tm[[i]][1])) +
          as.numeric(sapply(seq_len(length(tm)), function(i) tm[[i]][2])) / 60
  } else {
    fd <- tm
  }
  if(type == "asis") return(fd)

  txt <- c("circular.mean", "circular.sd")
  na <- length( fd[is.na(fd)] )
  fd <- fd[! is.na(fd)]
  if(length(fd) == 0) {
    zz <- c(NA, NA, 0, na)
    names(zz) <- c("circular.mean", "circular.sd", "num_peak_tm", "num_NAs")
    return( zz )
  }

  "circ_musd" <- function(hours) {
    # convert hours to radians, to convert from hours to radians
    rads <- (hours / 24 ) * (2 * pi)

    # Refer also to circular:::RhoCircularRad
    # Refer also to circular::sd.circular
    sd <- sqrt( -2*log( sqrt(sum(sin(rads))^2 + sum(cos(rads))^2) / length(rads) ) )

    # Refer also to circular::mean.circular
    sin_mu <- mean( sin(rads) ); cos_mu <- mean( cos(rads) )
    rad_mu <- atan2(sin_mu, cos_mu)
    # print(c(sin_mu, cos_mu, rad_mu))
    if(sin_mu < 0 & cos_mu > 0) {
      rad_mu <- rad_mu + 2*pi
    } else if(sin_mu < 0 & cos_mu <= 0) {
      rad_mu <- rad_mu + 2*pi
    }
    mu <- rad_mu / (2*pi) * 24
    zz <- c(mu, sd)
    names(zz) <- c("circular.mean", "circular.sd")
    return( zz )
  }

  if(type == "musd") {
    zz <- circ_musd(fd)
    zz[3:4] <- c(length(fd), na)
    names(zz) <- c("circular.mean", "circular.sd", "num_peak_tm", "num_NAs")
    return( zz )
  }
}

"plotPeaks" <-
function(x, codes=TRUE, lot=NULL, site="",
            xlab="", ylab="", xlim=NULL, ylim=NULL,
            xlim.inflate=TRUE, ylim.inflate=TRUE, aux.y=NULL,
            log.ticks=c(1, 2, 3, 5, 8), mtext.site=TRUE,
            show48=FALSE, showDubNA=FALSE, showGHyrs=TRUE,
            make_text_for_all_na_peaks=TRUE, ...) {

  if(! exists("anyCodes", as.list(x))) x <- splitPeakCodes(x)
  x$peak_va <- as.numeric(x$peak_va) # just in case--expectation is that
  # readNWISpeak is called with convertType=FALSE to more seemingless deal with dates
  x$gage_ht <- as.numeric(x$gage_ht)
  x$gage_ht[is.na(x$gage_ht)] <- as.numeric(x$ag_gage_ht[is.na(x$gage_ht)])
  # the above line is a last ditch effort figure out gage height record

  nwis_double_nullyrs <- x$water_yr[is.na(x$peak_va) &   is.na(x$gage_ht)]
  na_peak_gage_avail  <- x$water_yr[is.na(x$peak_va) & ! is.na(x$gage_ht)]

  if(! exists("water_yr", as.list(x))) x <- makeWaterYear(x)

  have_at_least_one_peak <- TRUE
  x <- x[! is.na(x$peak_va),]
  if(length(x[,1]) == 0) have_at_least_one_peak <- FALSE

  if(have_at_least_one_peak) {
    xrng <- suppressWarnings(range(x$water_yr)) # note x-range is not potential xlim until later
    if(is.null(lot)) lot <- MGBT(x$peak_va)$LOThresh
  } else {
    xrng <- suppressWarnings(range(na_peak_gage_avail))
  }

  gap <- xrng[1]:xrng[2]; #print(gap)
  gap <- sapply(gap, function(i) ifelse(length(x$water_yr[x$water_yr == i]) == 0,i,NA)); #print(gap)
  gap <- gap[! is.na(gap)]; #print(gap)

  if(is.null(xlim)) xlim <- xrng
  if(  showGHyrs  ) xlim <- range(c(xlim, na_peak_gage_avail))
  if(is.null(ylim)) ylim <- suppressWarnings(range(x$peak_va[x$peak_va > 0]))
  if(! is.finite(ylim[1])) {
    ylim <- c(0.01, 0.1) # should only trigger have_at_least_one_peak == FALSE
  }
  # This is a little nudger. What could happen without this is say a maximum
  # being exactly 50,000 cfs, which would place that data point on the top axis.
  # The ylim.inflate does not quite catch this otherwise. So the following 0.01
  # log10 cycle seems to work okay.
  if(ylim.inflate)     ylim <- 10^c(log10(ylim)+c(-0.01,0.01))
  if(! is.null(aux.y)) ylim <- range(c(ylim, aux.y), na.rm=TRUE)

  if(xlim.inflate) {
    xlim <- c(as.integer(xlim[1]/10-1)*10, as.integer(xlim[2]/10+1)*10)
  }
  if(ylim.inflate) {#print(ylim)
    ylim <- 10^(log10(ylim)+6) # complicating sign reversals at log10(1)
    # so let us offset six orders of magnitude and reverse the offset later
    needs <- log10(1:10)
    wants <- log10(ylim) - as.integer(log10(ylim))
    left  <- ifelse(wants[1] == 0, 0, rev(needs[needs <= wants[1]])[1])
    right <- ifelse(wants[2] == 0, 0,     needs[needs >= wants[2]][1])
    ylim <- 10^(as.integer(log10(ylim)) + c(left, right))
    ylim <- 10^(log10(ylim)-6); #print(ylim)
  }
  xx <- xlim; xx <- c(xx, xx[2:1], xx[1])
  yy <- ylim[1]; yy <- c(rep(yy,2), rep(lot,2), yy[1])
  core <- seq(xlim[1], xlim[2], by=10)
  core <- core[core <= xx[2] & core >= xx[1]]

  opar <- par(no.readonly=TRUE)
  par(mgp=c(3,0.5,0), las=0)

  plot(x$water_yr[x$peak_va > 0], x$peak_va[x$peak_va > 0],
       type="n", log="y", xlim=xlim, ylim=ylim, xlab=xlab, ylab="",
       xaxs="i", yaxs="i", axes=FALSE, tcl=0.5, ...)

  if(! have_at_least_one_peak) {
    if(make_text_for_all_na_peaks) {
      txt <- "There are no non-missing peaks for this streamgage."
      text(mean(par()$usr[1:2]), 10^mean(par()$usr[3:4]), txt, font=3)
    }
  }

  if(showDubNA) {
    usry <- par()$usr[3:4]
    for(wy in nwis_double_nullyrs) {
      polygon(   c(   -0.5,    +0.5,    +0.5,    -0.5,    -0.5)+wy,
              10^c(usry[1], usry[1], usry[2], usry[2], usry[1]),
              col="#EDF8B1", lty=0)
    }
  }

  if(length(xx) == length(yy)) { # should only trigger have_at_least_one_peak == FALSE
    polygon(xx,yy, col=grey(0.95), border=NA)
  }

  lines(xx[1:2], rep(lot,2), lwd=0.5, lty=2)
  lmomco::add.log.axis(side=2,    tcl=+0.8*abs(par()$tcl), two.sided=TRUE)
  lmomco::add.log.axis(logs=c(1), tcl=+1.3*abs(par()$tcl), two.sided=TRUE, side=2)
  axis(1, c(xx[1],core,xx[2]), at=c(xx[1],core,xx[2]), labels=c("",core,""), tcl=0.5)
  htix <- axTicks(1)
  axis(3, c(xx[1],core,xx[2]), labels=FALSE, tcl=0.5)
  axis(1, xlim[1], tcl=0, lwd=0)
  axis(1, xlim[2], tcl=0, lwd=0)
  lmomco::add.log.axis(logs=log.ticks, side=2, make.labs=TRUE, las=1, label="")
  axis(2, ylim, labels=FALSE, tcl=0) # very late additions, did lmomco change?
  axis(4, ylim, labels=FALSE, tcl=0) # to require these to be manually added (WHA 11/27/2018)
  mtext(ylab, side=2, line=3)

  gap.lwd <- 0.76
  for(wy in gap) {
     lines(rep(wy,2), c(ylim[1],ylim[2]), col="#FFAA00", lwd=gap.lwd, lty=5)
  }
  with(x[x$peak_va == 0,], rug(x$water_yr[x$peak_va == 0],
                               side=1, tcl=-0.5, col="#006F41", lwd=2.1))

  if(showGHyrs) {
    # A with() operation does not work here if the na_peak_gage_avail is empty it seems
    if(length(na_peak_gage_avail) >0) rug(na_peak_gage_avail, side=1,
                                          tcl=-0.4, col="#42B3F4", lwd=2.1)
  }

  with(x[x$appearsSystematic & x$isCode7,],
           points(water_yr, peak_va, pch= 1, col=grey(0.1), bg=NULL, cex=0.7, lwd=0.8))
  with(x[! x$anyCodes,],
           points(water_yr, peak_va, pch=21, col=grey(0.1), bg=grey(0.8), cex=0.7, lwd=0.8))
  with(x[x$isCode1,], # circle with + in it, maximum daily average
           points(water_yr, peak_va, pch=10, col=grey(0.1), cex=1.1, lwd=0.8))
  with(x[x$isCode9,], # square with x in it affected by snowmelt, hurricane, ice-jam, or debris-dam breakup
           points(water_yr, peak_va, pch=7,  col=grey(0.1), cex=1.9, lwd=0.8))
  with(x[(x$isCode2 | x$isCodeA | x$isCodeB  | x$isCodeD | x$isCodeE) & # no distinction of an estimated flow
       ! x$isCode1 & ! x$isCode3 & ! x$isCode4 & ! x$isCode5 & # is made or if a date is ambiguous
       ! x$isCode6 & ! x$isCode7 & ! x$isCode8 & ! x$isCode9 & ! x$isCodeC, ],
           points(water_yr, peak_va, pch=21, col=grey(0.1), bg=grey(0.8), cex=0.7, lwd=0.8))

  for(wy in x$water_yr[x$isCode4 == TRUE]) {
    my.pk <- x$peak_va[x$water_yr == wy]
    if(length(my.pk) == 1) {
      lines(rep(wy,2), c(ylim[1], x$peak_va[x$water_yr == wy]), col="#FF3C00", lwd=0.81)
    } else {
      warning("site '", site, "', water year ", wy, " occurs more than once (x",length(my.pk), ") (drawing them all)")
      for(pk in my.pk) {
        lines(rep(wy,2), c(ylim[1], pk), col="#FF3C00", lwd=0.81)
      }
    }
  }

  for(wy in x$water_yr[x$isCode8 == TRUE]) {
    my.pk <- x$peak_va[x$water_yr == wy]
    if(length(my.pk) == 1) {
      lines(rep(wy,2), c(ylim[2], x$peak_va[x$water_yr == wy]), col="#FF3C00", lwd=0.81)
    } else {
      warning("site '", site, "' has water year ", wy, " more than once (x",length(my.pk), ") (drawing them all)")
      for(pk in my.pk) {
        lines(rep(wy,2), c(ylim[1], pk), col="#FF3C00", lwd=0.81)
      }
    }
  }

  if(codes) {
    with(x[x$isCodeC == TRUE,], points(water_yr, peak_va, pch="C", cex=1.3, col="blue"))
    with(x[x$isCodeO == TRUE,], points(water_yr, peak_va, pch="O", cex=1.9, col="#FF8C00"))
    with(x[x$isCode3,], # and big letter D for dam
           points(water_yr, peak_va, pch="D", col="#377EB8", cex=2.5, lwd=0.8))
    if(show48) {
      with(x[x$isCode4 == TRUE,], points(water_yr, peak_va, pch="4", cex=0.8, col="red"))
      with(x[x$isCode8 == TRUE,], points(water_yr, peak_va, pch="8", cex=0.8, col="red"))
    } else {
      with(x[x$isCode4 == TRUE,], points(water_yr, peak_va, pch=16,  cex=0.7, lwd=0.8, col="#FF3C00"))
      with(x[x$isCode8 == TRUE,], points(water_yr, peak_va, pch=16,  cex=0.7, lwd=0.8, col="#FF3C00"))
    }
    with(x[x$isCode5 == TRUE,], points(water_yr, peak_va, pch="5", cex=0.8, col="#4B284B"))
    with(x[x$isCode6 == TRUE,], points(water_yr, peak_va, pch="6", cex=0.8, col="#7D287D"))
    with(x[x$isCode7 == TRUE,], points(water_yr, peak_va, pch="7", cex=1.0, col=6))
  } else {
    with(x[x$isCodeC == TRUE,], points(water_yr, peak_va, pch=3,   cex=1.1, lwd=0.8, col="blue"))
    with(x[x$isCode4 == TRUE,], points(water_yr, peak_va, pch=16,  cex=0.7, lwd=0.8, col="#FF3C00"))
    with(x[x$isCode5 == TRUE,], points(water_yr, peak_va, pch=2,   cex=0.7, lwd=0.8, col="#4B284B"))
    with(x[x$isCode6 == TRUE,], points(water_yr, peak_va, pch=1,   cex=0.7, lwd=0.8, col="#7D287D"))
    with(x[x$isCode7 == TRUE,], points(water_yr, peak_va, pch=6,   cex=0.7, lwd=0.8, col=6))
    with(x[x$isCode8 == TRUE,], points(water_yr, peak_va, pch=16,  cex=0.7, lwd=0.8, col="#FF3C00"))
  }
  if(mtext.site) mtext(site, line=0.5)
  on.exit(par(opar))
}

"splitPeakCodes" <- function(x, all_peaks_na_okay=FALSE) {
   if(! is.data.frame(x)) {
      warning("a data.frame as in 'dataRetrieval::readNWISpeak(station, convertType=FALSE)' required")
      return(NULL)
   }
   nu <- length(unique(x$site_no))
   if(nu > 1) {
      message("more than one site_no encountered, please just use one station ",
              "at a time and then rbind() the successive results on your own, ",
              "testing indicates some difficulty in setting appearsSystematic ",
              "as the algorithms rolls onto or off of a data frame with multiple ",
              "streamgages, it is safer and easier to check if this algorithm ",
              "works for all situations of gaps and code 7 this way")
      return(NULL)
   } else if(nu == 0) {
      message("no rows of peaks to process")
      return(NULL)
   }
   site_for_messaging <- x$site_no[1]

   if(length(x$water_yr) == 0) x <- makeWaterYear(x)

   x$appearsSystematic <- TRUE
   x$appearsSystematic[is.na(x$peak_va)] <- FALSE
   if(all_peaks_na_okay) {
     tmp <- x
   } else {
     tmp <- x[! is.na(x$peak_va),]
     if(length(tmp$peak_va) == 0) {
       message("no non NA peaks available for ", site_for_messaging, ", returning NULL\n",
               "user might be interested instead of using all_peaks_na_okay=TRUE argument")
       return(NULL)
     }
   }
   ix <- range(tmp$water_yr)[1]:range(tmp$water_yr)[2]
   gap <- sapply(ix, function(i) ifelse(length(tmp$water_yr[tmp$water_yr == i]) == 0,i,NA))
   gap <- gap[! is.na(gap)]

   x$peak_va <- as.numeric(x$peak_va) # presuming dataRetrieval::readNWISpeak(station, convertType=FALSE)
   x$gage_ht <- as.numeric(x$gage_ht) # ..ditto..

   x$anyCodes <- NA
   x$isCode9 <- x$isCode8 <- x$isCode7 <-
   x$isCode6 <- x$isCode5 <- x$isCode4 <-
   x$isCode3 <- x$isCode2 <- x$isCode1 <- FALSE
   x$isCodeB <- x$isCodeA <- FALSE
   x$isCodeE <- x$isCodeD <- x$isCodeC <- FALSE
   x$isCodeR <- x$isCodeO <- x$isCodeF <- FALSE
   x$isCodeA[grep("A", x$peak_cd)] <- TRUE
   x$isCodeB[grep("B", x$peak_cd)] <- TRUE
   x$isCodeC[grep("C", x$peak_cd)] <- TRUE
   x$isCodeD[grep("D", x$peak_cd)] <- TRUE
   x$isCodeE[grep("E", x$peak_cd)] <- TRUE
   x$isCodeF[grep("F", x$peak_cd)] <- TRUE
   x$isCodeO[grep("O", x$peak_cd)] <- TRUE
   x$isCodeR[grep("R", x$peak_cd)] <- TRUE
   x$isCode1[grep("1", x$peak_cd)] <- TRUE
   x$isCode2[grep("2", x$peak_cd)] <- TRUE
   x$isCode3[grep("3", x$peak_cd)] <- TRUE
   x$isCode4[grep("4", x$peak_cd)] <- TRUE
   x$isCode5[grep("5", x$peak_cd)] <- TRUE
   x$isCode6[grep("6", x$peak_cd)] <- TRUE
   x$isCode7[grep("7", x$peak_cd)] <- TRUE
   x$isCode8[grep("8", x$peak_cd)] <- TRUE
   x$isCode9[grep("9", x$peak_cd)] <- TRUE
   x$anyCodes[grep("A|B|C|D|O|1|2|3|4|5|6|7|8|9", x$peak_cd)] <- TRUE
   x$anyCodes[is.na(x$anyCodes)]  <- FALSE
   x$appearsSystematic[x$isCodeO] <- FALSE
   for(seven in c(x$water_yr[x$isCode7], max(x$water_yr))) {
      yrs <- c(gap[gap == (seven-1)], gap[gap == (seven+1)])
      if(length(yrs) != 0) x$appearsSystematic[x$water_yr == seven] <- FALSE
   }
   return(x)
}

"plotPeaks_batch" <-
function(sites, file=NA, do_plot=TRUE,
                silent=FALSE, envir=NULL, ...) {
  if(! do_plot) file <- NA
  if(is.null(envir)) {
    pkenv <- new.env()
  } else {
    if(is.environment(envir)) {
      pkenv <- envir
    } else {
      stop("envir given but not an environment")
    }
  }

  numpk <- 0
  for(site in sites) {
    if(exists(site, pkenv)) next
    if(! silent) message("Pulling peaks for ",site)
    pk <- NA
    pk <- dataRetrieval::readNWISpeak(site, convert=FALSE)
    suppressWarnings(pk <- splitPeakCodes(pk))
    numpk <- numpk + length(pk$peak_va[pk$appearsSystematic == TRUE])
    if(is.null(pk)) pk <- NA
    assign(site, pk, envir=pkenv)
  }
  if(is.null(envir) & ! silent) message("DONE (num. systematic peaks = ",numpk,")\n")

  numpk <- 0
  empty_sites <- NULL
  if(! is.na(file)) pdf(file, useDingbats=FALSE)
    for(site in sites) {
      if(do_plot & ! silent) message("Plotting ", site, appendLF=FALSE)
      pk <- get(site, envir=pkenv)
      if(length(pk) == 1) {
        if(do_plot & ! silent) message("---empty")
        if(do_plot) {
          plot(0:2,0:2, xlab="", ylab="", xaxt="n", yaxt="n", type="n")
          text( 1,  1,  "EMPTY PEAK FILE")
          mtext(site)
        }
        empty_sites <- c(empty_sites, site)
      } else {
        numpk <- numpk + length(pk$peak_va[pk$appearsSystematic == TRUE])
        if(do_plot) {
          plotPeaks(pk, ...)
          mtext(site)
        }
        if(do_plot & ! silent) message("")
      }
    }
    if(do_plot & ! silent) message("DONE (num. systematic peaks = ",numpk,")")
  if(! is.na(file)) dev.off()
  pkenv <- as.list(pkenv)
  attr(pkenv, "empty_sites") <- empty_sites
  return(pkenv)
}

"ratioPeakMax1Day" <-
function(siteNumber, dvenv=NULL, pkenv=NULL, as.list=FALSE,
                     missing.days=0, rm.ratios.lt1=TRUE, silent=TRUE, ...) {

  siteNumber <- siteNumber[1]

  attrs <- c("url", "siteInfo", "variableInfo", "disclaimer", "statisticInfo",
                   "queryTime", "headerInfo") # named attributes for DVs and peaks by dataRetrieval

  if(! is.null(dvenv)) {
    if(is.environment(dvenv)) {
      if(   exists(siteNumber, envir=dvenv)) {
        dv <-  get(siteNumber, envir=dvenv)
      } else {
        if(! silent) message(siteNumber, " daily values for site ", siteNumber, " are not in dvenv object")
        return(NULL)
      }
    }
  } else {
    if(! silent) message(siteNumber, " reading daily values from NWIS")
    dv <- dataRetrieval::readNWISdv(  siteNumber, parameterCd="00060", ...)
    dv <- dataRetrieval::renameNWISColumns( dv )
    for(str in attrs) attr(dv, which=str) <- NULL
  }
  if(is.null(dv)) {
    if(! silent) message(siteNumber, " dv object is NULL")
    return(NULL)
  }
  if(nrow(dv) == 0) {
    if(! silent) message(siteNumber, " number of rows in dv object is zero")
    return(NULL)
  }
  if(! exists("Date", dv)) {
    if(! silent) message(siteNumber, " daily values do not have a column named Date")
    if(! as.list) return(NULL)
    return(list(dv=dv, pk=NULL))
  }
  if(! exists("Flow", dv)) {
    if(! silent) message(siteNumber, " daily values do not have a column named Flow")
    if(! as.list) return(NULL)
    return(list(dv=dv, pk=NULL))
  }

  dv$year  <- as.integer( strftime(dv$Date, format="%Y", usetz=FALSE) )
  dv$month <- as.integer( strftime(dv$Date, format="%m", usetz=FALSE) )
  dv$water_yr <- dv$year + as.integer(dv$month >= 10)

  if(! is.null(pkenv)) {
    if(is.environment(pkenv)) {
      if(   exists(siteNumber, envir=pkenv)) {
        pk <-  get(siteNumber, envir=pkenv)
      } else {
        if(! silent) message(siteNumber, " peak values for site ", siteNumber, " are not in pkenv object")
        return(NULL)
      }
    }
  } else {
    if(! silent) message("reading peak values from NWIS")
    pk <- dataRetrieval::readNWISpeak(siteNumber, convertType=FALSE  )
    for(str in attrs) attr(pk, which=str) <- NULL
  }
  if(is.null(pk)) {
    if(! silent) message(siteNumber, " peaks nonexistent ( pk  == NULL)")
    if(! as.list) return(NULL)
    return(list(dv=dv, pk=pk))
  }
  if(nrow(pk) == 0) {
    if(! silent) message(siteNumber, " peaks nonexistent (nrow ==    0)")
    if(! as.list) return(NULL)
    return(list(dv=dv, pk=pk))
  }


  pk <- MGBT::splitPeakCodes( pk )
  pk <- pk[! pk$isCodeO, ] # remove all opportunistic peaks
  pk <- pk[pk$appearsSystematic, ]
  if(nrow(pk) == 0) {
    if(! silent) message(siteNumber, " systematic peaks nonexistent")
    if(! as.list) return(NULL)
    return(list(dv=dv, pk=pk))
  }

  myleap_year <- function(year) { # near copy of the lubridate::leap_year() function but only for
    (year %% 4 == 0) & ((year %% 100 != 0) | (year %% 400 == 0)) } # numeric year

  if(nrow(dv) == 0) {
    if(! silent) message(siteNumber, " no rows in dv to aggregate1")
    if(! as.list) return(NULL)
    return(list(dv=dv, pk=pk, mx1d=mx1d, ratios.less.than.one=yy, ratios=zz))
  }
  h  <- aggregate(dv, by=list(dv$water_yr), length);# print("aggregate1")
  h$isLeapYear <- myleap_year(h$Group.1)
  h$days_in_year <- 365
  h$days_in_year[h$isLeapYear] <- 366
  h  <- h[h$Date >= h$days_in_year - missing.days, ]

  if(nrow(h) == 0) {
    if(! silent) message(siteNumber, " no rows in h after missing.days check in aggregate1")
    if(! as.list) return(NULL)
    return(list(dv=dv, pk=pk, mx1d=mx1d, ratios.less.than.one=yy, ratios=zz))
  }

  dv <- dv[dv$water_yr %in% h$Group.1, ]
  if(nrow(dv) == 0) {
    if(! silent) message(siteNumber, " no rows in dv to aggregate2")
    if(! as.list) return(NULL)
    return(list(dv=dv, pk=pk, mx1d=mx1d, ratios.less.than.one=yy, ratios=zz))
  }
  pk <- pk[pk$water_yr %in% h$Group.1, ]

  mx1d <- aggregate(dv, by=list(dv$water_yr), max); #print("aggregate2")
  mx1d$dvflow_count <- NA
  for(water_yr in h$Group.1) {
    mx1d$dvflow_count[mx1d$water_yr == water_yr] <- h$water_yr[h$Group.1 == water_yr]
  }
  if(nrow(pk) == 0) {
    if(! silent) message(siteNumber, " peaks nonexistent after dv water year missing days check")
    if(! as.list) return(NULL)
    return(list(dv=dv, pk=pk, mx1d=mx1d))
  }

  if(nrow(pk) != nrow(mx1d)) {
    if(! silent) message("ALERT: ", siteNumber, " row numbers in the dv and pk tables are not equal")
    zz <- merge(pk, mx1d, by="water_yr", all=TRUE)
    zz <- zz[! is.na(zz$agency_cd.x), ]; zz <- zz[! is.na(zz$agency_cd.y), ]
    zz <- data.frame(site_no     = zz$site_no.x, water_yr  = as.integer(zz$water_yr),
                     dvflow_count= zz$dvflow_count,
                     peak_flow   = zz$peak_va,   mx1d_flow = zz$Flow)
  } else {
    zz <- data.frame(site_no     = pk$site_no,   water_yr  = as.integer(pk$water_yr),
                     dvflow_count=mx1d$dvflow_count,
                     peak_flow   = pk$peak_va,   mx1d_flow = mx1d$Flow)
  }
  zz <- zz[complete.cases(zz), ]
  zz <- zz[zz$peak_flow > 0,   ]
  zz <- zz[zz$mx1d_flow > 0,   ]
  zz$ratio     <- round(      zz$peak_flow  /       zz$mx1d_flow,  digits=8)
  zz$log10diff <- round(log10(zz$peak_flow) - log10(zz$mx1d_flow), digits=8)

  if(rm.ratios.lt1) {
    yy <- zz[zz$ratio <  1, ]
    zz <- zz[zz$ratio >= 1, ]
    if(nrow(zz) == 0) {
      if(! silent) message(siteNumber, " no records after ratio >= 1 check (rm.ratios.lt1)")
      if(! as.list) return(NULL)
      return(list(dv=dv, pk=pk, mx1d=mx1d, ratios.less.than.one=yy, ratios=zz))
    }
  }

  if(as.list) {
    zz <- list(dv=dv, pk=pk, mx1d=mx1d, ratios.less.than.one=yy, ratios=zz)
  }

  return(zz)
}

#COHN****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN  Alternative test: N3 from Barnett & Lewis (adjusted to look for Low Outliers
#COHN     p. 169
#COHN BL_N3 <- function(x,k,n=length(x)){
#COHN           y=sort(x);(sum(y[1:k])-k*mean(y))/sqrt(var(y))}

#Barnett, Vic, and Lewis, Toby, 1994, Outliers in statistical data, 3rd ed.: New York, John Wiley,
#ISBN 978-0471930945 ?

"BLlo" <- function(x, r, n=length(x)) {
   y <- sort(x)
   ( sum(y[1:r]) - r * mean(y) ) / sqrt( var(y) )
}

#"BL_N3" <- function(...) BLlo(...)

# BL_N3 in COHN_MGBT_LowOutliers(R).txt is only used by testMGBvsN3()

"jointPeaks" <-
function(Asite_no="--", Bsite_no="--", appearsSystematic=FALSE, a=0,
         Adf=NULL, Bdf=NULL, ...) {
  cda1 <- "--"
  cda2 <- "--"
  if(is.null(Adf)) {
    PK1 <- NULL
    try( PK1 <- dataRetrieval::readNWISpeak(Asite_no, convert=FALSE), silent=TRUE)
    if(is.null(PK1)) {
      message("readNWISpeak('", Asite_no, "') returned NULL, possible internet hickup, returning NULL")
      return(NULL)
    }
    if(length(PK1) == 0) {
      message("site ", Asite_no, " does not seem to have peaks, returning NULL")
      return(NULL)
    }
    PK1 <- splitPeakCodes(PK1)
    SF1 <- attr(PK1, "siteInfo")         # dataRetrieval appends site information
    PK1 <- as.data.frame(as.matrix(PK1)) # removes the attributes by dataRetrieval
    PK1$peak_va <- as.numeric(PK1$peak_va)
    cda1 <- pmin(SF1$drain_area_va, SF1$contrib_drain_area_va, na.rm=TRUE)
    if(exists("dec_lat_va",   SF1)) PK1$dev_lat_va   <- SF1$dec_lat_va  # defensive programming
    if(exists("dec_long_va",  SF1)) PK1$dev_long_va  <- SF1$dec_long_va # defensive programming
    if(exists("contrib_drain_area_va",  SF1)) PK1$nwiscda_sqmi <- cda1  # defensive programming
  } else {
    PK1 <- Adf
    nm  <- names(PK1)
    if(length(grep("nwiscda_sqmi", nm)) == 1) cda1 <- Adf[, grep("nwiscda_sqmi", nm)][1]
  }

  if(is.null(Bdf)) {
    PK2 <- NULL
    try( PK2 <- dataRetrieval::readNWISpeak(Bsite_no, convert=FALSE), silent=TRUE)
    if(is.null(PK2)) {
      message("readNWISpeak('", Bsite_no, "') returned NULL, possible internet hickup, returning NULL")
      return(NULL)
    }
    if(length(PK2) == 0) {
      message("site ", Bsite_no, " does not seem to have peaks, returning NULL")
      return(NULL)
    }

    PK2 <- splitPeakCodes(PK2)
    SF2 <- attr(PK2, "siteInfo")         # dataRetrieval appends site information
    PK2 <- as.data.frame(as.matrix(PK2)) # removes the attributes by dataRetrieval
    PK2$peak_va <- as.numeric(PK2$peak_va)
    cda2 <- pmin(SF2$drain_area_va, SF2$contrib_drain_area_va, na.rm=TRUE)
    if(exists("dec_lat_va",   SF2)) PK2$dev_lat_va   <- SF2$dec_lat_va  # defensive programming
    if(exists("dec_long_va",  SF2)) PK2$dev_long_va  <- SF2$dec_long_va # defensive programming
    if(exists("contrib_drain_area_va",  SF2)) PK2$nwiscda_sqmi <- cda2  # defensive programming
  } else {
    PK2 <- Bdf
    nm  <- names(PK2)
    if(length(grep("nwiscda_sqmi", nm)) == 1) cda2 <- Bdf[, grep("nwiscda_sqmi", nm)][1]
  }

  strs <- c("appearsSystematic", "anyCodes", "isCode1", "isCode2", "isCode3", "isCode4", "isCode5",
            "isCode6", "isCode7", "isCode8", "isCode9", "isCodeA", "isCodeB", "isCodeC", "isCodeD",
            "isCodeE", "isCodeF", "isCodeO")
  for(str in strs) { # the as.data.frame(as.matrix()) operations above break the logical type cast
    PK1[,str] <- as.logical(PK1[,str])
    PK2[,str] <- as.logical(PK2[,str])
  }

  if(appearsSystematic) {
    PK1z <- PK1[PK1$appearsSystematic, ]
    PK2z <- PK2[PK2$appearsSystematic, ]
  } else {
    PK1z <- PK1
    PK2z <- PK2
  }

  allwy <- sort( unique( c(PK1z$water_yr, PK2z$water_yr) ) )
  D <- NULL
  for(wy in allwy) {
    pk1 <- PK1z$peak_va[PK1z$water_yr == wy]
    if(length(pk1) == 0) pk1 <- NA
    pk2 <- PK2z$peak_va[PK2z$water_yr == wy]
    if(length(pk2) == 0) pk2 <- NA
    df  <- data.frame(water_yr=wy, Asite_no=Asite_no, Apeak_va=pk1, Bsite_no=Bsite_no, Bpeak_va=pk2)
    D   <- rbind(D, df)
  }

  if(nrow(D) == 0) {
    warning("not matching water year peaks in data.frame D, returning NULL")
    return(NULL)
  }
  J <- D[complete.cases(D), ]
  if(nrow(J) == 0) {
    warning("not matching water year peaks in data.frame J, returning NULL")
    return(NULL)
  }


  row.names(J) <- NULL # cleans things up, remember numbers not sync'd when rows deleted, decorative
  J$Apeak_dt <- NA     # stub out
  J$Bpeak_dt <- NA     # stub out
  J$Anwiscda_sqmi <- cda1 # insert the contributing drainage area
  J$Bnwiscda_sqmi <- cda2 # insert the contributing drainage area
  for(wy in J$water_yr) {
    J$Apeak_dt[J$water_yr == wy] <- PK1$peak_dt[PK1$water_yr == wy]
    J$Bpeak_dt[J$water_yr == wy] <- PK2$peak_dt[PK2$water_yr == wy]
  }
  J <- J[, c("water_yr", "Asite_no", "Bsite_no", "Anwiscda_sqmi", "Bnwiscda_sqmi",
                         "Apeak_dt", "Bpeak_dt", "Apeak_va", "Bpeak_va")]
  J$absdays_between_pks <- as.numeric( abs(as.Date(J$Apeak_dt) - as.Date(J$Bpeak_dt)) )
  J$U <- round(lmomco::pp(J$Apeak_va, a=a, sort=FALSE), digits=8)
  J$V <- round(lmomco::pp(J$Bpeak_va, a=a, sort=FALSE), digits=8)

  J$cex <- as.numeric(J$absdays_between_pks/366) + 1
  J$cex[is.na(J$cex)] <- 1
  zz <- list(Asite_no=PK1, Bsite_no=PK2, AB=J)
  return(zz)
}

#COHN:P3_089****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN:P3_089
#COHN:P3_089  Function to compute p-value corresponding to k-th order statistic
#COHN:P3_089     n    sample size
#COHN:P3_089     r    number of truncated observations
#COHN:P3_089     eta  "pseudo-studentized" magnitude of r-th smallest observation
#COHN:P3_089           eta = (X[r]-mean(X[(r+1):n])/sqrt(var(X[(r+1):n]))
#COHN:P3_089
#COHN:P3_089KthOrderPValueOrthoT <- function(n,r,eta){
#COHN:P3_089  integrateV(peta,lower=1e-7,upper=1-1e-7,n=n,r=r,eta=eta)
#COHN:P3_089   }


#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   Improved "integrate" function that de-vectorizes f(x)
#COHN
# Below is a cleanup of Cohn's coding style. Note that Cohn did not pass
# the actual arguments via their names into the integrate() call. This
# Asquith has done. However, the logic can be shortened to avoid a separate
# function for vectorizing TAC's original peta().
#"integrateV" <-
#function(f, lower, upper, ..., subdivisions=100,
#         rel.tol = .Machine$double.eps^0.25, abs.tol = rel.tol,
#         stop.on.error = TRUE, keep.xy = FALSE, aux = NULL) {
#   f2 <- function(x,...){ sapply(x, f, ...) }
#   integrate(f2, lower, upper, ..., subdivisions=100,
#             rel.tol = .Machine$double.eps^0.25, abs.tol = rel.tol,
#             stop.on.error = TRUE, keep.xy = FALSE, aux = NULL)
#}
# This is an updated port via WHA, though function is not needed notice how TAC
# did not actually pass the argument for control the integration on down the
# chain to intergrate().
#"integrateV" <-
#function(f, lower, upper, ..., subdivisions=100,
#         rel.tol = .Machine$double.eps^0.25, abs.tol = rel.tol,
#         stop.on.error = TRUE, keep.xy = FALSE, aux = NULL) {
#   f2 <- function(x, ...) sapply(x, f, ...)
#   integrate(f2, lower, upper, ..., subdivisions=subdivisions,
#             rel.tol = rel.tol, abs.tol = abs.tol,
#             stop.on.error = stop.on.error, keep.xy = keep.xy, aux = aux)
#}

# integrateV in COHN_MGBT_LowOutliers(R).txt is only used by KthOrderPValueOrthoT()
# It would be safer to define integrateV within that function to keep scope local.

# What Cohn is doing is vectorizing a function that he has defined elsewhere with
# only scalar operation. Perhaps too complicated to put an sapply() inside that other
# function. Asquith understands the reasoning and has many cases of his own where
# vectorization needed to be created. Asquith designs though have historically put
# the vectorization within the said function.


"RthOrderPValueOrthoT" <- function(n, r, eta, n.sim=10000, silent=TRUE) {
   if(length(eta) != 1) {
      warning("eta is not intended for vectorization in this function")
      return(NULL)
   }
   eps <- sqrt(.Machine$double.eps)
   A <- eps; B <- 1 - A
   intn <- NULL # WHA addition for trapping numerical integration failure
   try(intn <- integrate(peta, lower=A, upper=B, n=n, r=r, eta=eta), silent=silent)
   if(is.null(intn)) { # WHA addition
      if(! silent) message("Monte Carlo integration fall back")
      petas <- peta(runif(n.sim), n=n,r=r,eta=eta)
      try(intn <- (B-A)*sum(petas)/n.sim, silent=silent)
      if(is.null(intn)) return(list(value=NA, monte.carlo.error=NA, n.sim=n.sim))
      err <- sqrt((B-A)^2 * var(petas) / n.sim)
      intn <- list(value=intn, monte.carlo.error=err, n.sim=n.sim)
   }
   return(intn) # it is critical that this function return at least list(value=###)
}


#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   This routine employs simple Gaussian quadrature to compute the integral
#COHN   (above) much more quickly. However, it is slightly less accurate for
#COHN   tail probabilities
#COHN
#COHN KthOrderPValueOrthoTb <- function(n,r,eta,nterms=50){
#COHN                    x    <-  gauss.quad.prob(nterms)
#COHN                    sapply(x$nodes,peta,n=n,r=r,eta=eta) %*% x$weights
#COHN      }

#"KthOrderPValueOrthoTb" <- function(n, r, eta, nterms=50) {
#   x    <-  gauss.quad.prob(nterms)
#   peta(x$nodes, n=n, r=r, eta=eta) %*% x$weights
#}

# gauss.quad.prob  WHA(05/20/2017): I don't know source of this function

#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   Moments of observations above the threshold, xsi
#COHN
#COHN gtmoms <- function(xsi,k){
#COHN    H  <- function(xsi) dnorm(xsi)/(1-pnorm(xsi))
#COHN    p  <- pnorm(xsi);
#COHN    if(k == 0) return(1);
#COHN    if(k == 1) return(H(xsi));
#COHN    if(k > 1) return((k-1)*gtmoms(xsi,k-2)+H(xsi)*xsi^(k-1))
#COHN }

"gtmoms" <- function(xsi, r) {
   H  <- function(xsi) dnorm(xsi) / (1-pnorm(xsi))
   if(r == 0) return(1)
   if(r == 1) return(H(xsi))
   if(r >  1) return((r-1) * gtmoms(xsi,r-2) + H(xsi)*xsi^(r-1))
}

"MGBT" <- function(...) MGBT17c(...)

#COHN:P3_089 ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN:P3_089
#COHN:P3_089   Multiple Grubbs-Beck Test (MGBT)
#COHN:P3_089     Identifies up to n/2 low outliers
#COHN:P3_089     Modified 28 Jun 2016 to reflect MGBT [Stedinger, Cohn, etc.]
#COHN:P3_089
#COHN:P3_089 ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN:P3_089
#COHN:P3_089MGBT <- function(Q,Alphaout=0.005,Alphazeroin=0.10,n2=floor(length(Q)/2)){
#COHN:P3_089      zt      <- sort(log10(pmax(1e-8,Q)))
#COHN:P3_089      n       <- length(zt)
#COHN:P3_089      pvalueW <-rep(-99,n2);w<-rep(-99,n2)
#COHN:P3_089      j1=0;j2=0
#COHN:P3_089    for(i in 1:n2) {
#COHN:P3_089       w[i]<-(zt[i]-mean(zt[(i+1):n]))/sqrt(var(zt[(i+1):n]))
#COHN:P3_089       pvalueW[i]<-KthOrderPValueOrthoT(n,i,w[i])$value
#COHN:P3_089       if(pvalueW[i]<Alphaout){j1<-i;j2<-i}
#COHN:P3_089       if( (pvalueW[i]<Alphazeroin) & (j2==i-1)){j2<-i}
#COHN:P3_089       }
#COHN:P3_089    return(list(klow=j2,pvalues=pvalueW,LOThresh=ifelse(j2>0,sort(Q)[j2+1],0)))
#COHN:P3_089}

"MGBTcohn2016" <- # arguments match MGBTcohn2013 but the alphas were renamed
function(x, alphaout=0.005, alphazeroin=0.10, n2=floor(length(x)/2),
            napv.zero=TRUE, offset=0) {
   if(is.data.frame(x)) {
      message("x tests as data.frame, helped out with 'x <- as.numeric(x$peak_va)'")
      x <- as.numeric(x$peak_va)
   }
   if(is.character(x)) {
      message("x tests as character, helped out with 'x <- as.numeric(x)'")
      x <- as.numeric(x)
   } else if(! is.numeric(x)) {
      warning("unable to proceed as x is not a numeric vector")
      return(NULL)
   }
   if(any(is.na(x))) {
      message("NAs detected in x, these were removed") # to suppress.
   }
   x <- x[! is.na(x)]
   # WHA notes that by TAC using 1e-8 as "small", he has assumed that the MGBT
   # will always identify these as low outliers. Safe assumption in flood peaks,
   # but it is a weakness. WHA would have kept them as -Inf (we are in R and not
   # FORTRAN) and dealt with -Inf hitting subcomponents of the MBGT and trapping
   # the issue there.

   # WHA error trapping
   if(length(unique(x)) == 1) return(list(pvalues=NA, klow=0, LOThresh=0))

   zt <- sort( log10( pmax(1e-8,x)) )                                # matches MGBTcohn2013
   n  <- length(zt)  # WHA: n <- length(x) would be bit cleaner # matches MGBTcohn2013
   pvalueW <- w <- rep(NA, n2)                                       # matches MGBTcohn2013
   j1 <- j2 <- 0                                                     # matches MGBTcohn2013
   for(i in 1:n2) {                                                  # matches MGBTcohn2013
      w[i] <-   (zt[i] - mean(zt[(i+1):n])) / sqrt(var(zt[(i+1):n])) # matches MGBTcohn2013
            pvalueW[i] <- RthOrderPValueOrthoT(n, i, w[i])$value     # matches MGBTcohn2013
      if(   is.na(pvalueW[i])) {                         # WHA error trapping
                  pvalueW[i] <- ifelse(napv.zero, 0, NA) # WHA error trapping
         if(is.na(pvalueW[i])) next                      # WHA error trapping
      }                                                  # WHA error trapping
      # Next line as long as the p-value is less than alpha_out, we continue to
      # index up because of the direction of the for() loop. This indexing up
      # has the same effect as "sweeping out" from the median (well if
      # n2=length(x)/2) as discussed in 17C so j1 will always attain the
      # kth-largest low values having a p-value smaller than alpha_out. We can
      # conclude from this that 17C is being followed in this regard and that
      # alphaout is the same as LaTeX \alpha_out in 17C. Notice that alphaout
      # will skip over low values whose p-value might not be small enough and
      # declare them as outliers. This then seems to be a greedy approach. We
      # must conclude that j1 then is the index associated with the outward
      # sweep through in implementation, we are moving up the order statistics.
      # 17C says that the outward sweep "STOPS" but in implementation this is
      # not what happens. TAC is dealing with both alphas in one loop!
      if((pvalueW[i] < alphaout)) { j1 <- i; j2 <- i } # matches MGBTcohn2013
      # TAC seems to break from 17C in the notation that follows. First, this
      # alpha must be LaTeX \alpha_in in 17C. Because j2 gets set to i if the
      # condition is jointly true, then it can be seen that it is more visually
      # sweeping in towards the median. The (j2==(i-1)) is harder to "see" but
      # carefully considering its purpose, it ensures that one has a solid
      # continuity of low outliers from i=1 on up to j2. Hence, it is less
      # greedy than j1 by requiring this. j2 must be associated with sweep in
      # as j1 is dealing with sweep out. 17C says that the inward sweep "STOPS"
      # but in implementation this is not what happens. TAC is dealing with both
      # alphas in one loop!
      if((pvalueW[i] < alphazeroin) & (j2==i-1)) j2 <- i  # matches MGBTcohn2013
   }

   # TAC does not include a recursion back to MGBTcohn2016() for the
   # condition j2 == n2 (see MGBTcohn2013) at this point in the code.

   # ---***--------------***--- TAC CRITICAL BUG ---***--------------***---
   # j2 <- min(c(j1,j2)) # WHA tentative completion of the 17C alogrithm!?!
   # HOWEVER MAJOR WARNING. WHA is using a minimum and not a maximum!!!!!!!
   # See MGBT17C() below. In that if the line a few lines above that reads
   # if((pvalueW[i] < alpha1 )) { j1 <- i; j2 <- i }
   # is replaced with if((pvalueW[i] < alpha1 )) j1 <- i
   # then maximum and not the minimum becomes applicable.
   # ---***--------------***--- TAC CRITICAL BUG ---***--------------***---

   zz <- list(pvalues=pvalueW, klow=j2,                        # matches MGBTcohn2013
              LOThresh=ifelse(j2 > 0, sort(x)[j2+1]+offset, 0))# quasi matches MGBTcohn2013
   # WHA: What bothers me is that j1 does not seem to get involved in the final
   # result. Just by quick inspection, this does not seem symmetrical. 17C says
   # "the number of low outliers identified by [MGBT] is the larger of k and
   # m-1"  The "m" appears to be j2 and j1 appears to be the k. TAC does not
   # compute the maximum (larger there of) and with this conclusion is with
   # evidence of several code bundles available verbatim---
   # IS IT POSSIBLE THAT TAC DID NOT COMPLETE THE 17C ALGORITHM IN HIS MGBT-R
   # "port"? Looking closely, without yet proof, m-1 does not translate to j2-1
   # given the structure of TACs algorithm.
   zz$LOThresh[zz$LOThresh < 0] <- 0 # WHA 02/04/2019
   return(zz)   # the offset is by WHA                               # matches MGBTcohn2013
}

#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   Multiple Grubbs-Beck Test (MGBT)
#COHN     Identifies up to n/2 low outliers
#COHN
#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN MGBT <- function(Q,alpha1=0.005,alpha10=0.10,n2=floor(length(Q)/2)){
#COHN       zt      <- sort(log10(pmax(1e-8,Q)))
#COHN       n       <- length(zt)
#COHN       pvalueW <-rep(-99,n2);w<-rep(-99,n2)
#COHN       j1=0;j2=0
#COHN     for(i in 1:n2) {
#COHN        w[i]<-(zt[i]-mean(zt[(i+1):n]))/sqrt(var(zt[(i+1):n]))
#COHN        pvalueW[i]<-KthOrderPValueOrthoT(n,i,w[i])$value
#COHN        if(pvalueW[i]<alpha1){j1<-i;j2<-i}
#COHN        if( (pvalueW[i]<alpha10) & (j2==i-1)){j2<-i}
#COHN        }
#COHN     if(j2==n2){
#COHN       if(n2<length(Q)-5) {   # set a limit of at least 5 retained observations
#COHN         print(paste(" Number of low outliers equals or exceeds limit of ",n2));
#COHN         return(MGBT(Q,alpha1=0.01,alpha10=0.10,n2=length(Q)-5))} # try this
#COHN       else
#COHN         print("MGBT identifies too many low outliers; use caution and judgment")
#COHN     }
#COHN      return(list(klow=j2,pvalues=pvalueW,LOThresh=ifelse(j2>0,sort(Q)[j2+1],0)))
#COHN }

"MGBTcohn2013" <-
function(x, alphaout=0.005, alphazeroin=0.10, n2=floor(length(x)/2),
            napv.zero=TRUE, offset=0) {
   if(is.data.frame(x)) {
      message("x tests as data.frame, helped out with 'x <- as.numeric(x$peak_va)'")
      x <- as.numeric(x$peak_va)
   }
   if(is.character(x)) {
      message("x tests as character, helped out with 'x <- as.numeric(x)'")
      x <- as.numeric(x)
   } else if(! is.numeric(x)) {
      warning("unable to proceed as x is not a numeric vector")
      return(NULL)
   }
   if(any(is.na(x))) {
      message("NAs detected in x, these were removed") # to suppress.
   }
   x <- x[! is.na(x)]
   if(length(unique(x)) == 1) {                           # WHA error trapping
      return(list(pvalues=NA, klow=0, LOThresh=0))        # WHA error trapping
   }                                                      # WHA error trapping
   zt <- sort( log10( pmax(1e-8,x)) )
   n  <- length(zt) # WHA: n <- length(x) could be used instead and cleaner
   pvalueW <- w <- rep(NA, n2)
   j1 <- j2 <- 0
   for(i in 1:n2) {
      w[i] <-  (zt[i] - mean(zt[(i+1):n])) / sqrt(var(zt[(i+1):n]))
           pvalueW[i] <- RthOrderPValueOrthoT(n, i, w[i])$value
      if(   is.na(pvalueW[i])) {                          # WHA error trapping
                  pvalueW[i] <- ifelse(napv.zero, 0, NA)  # WHA error trapping
         if(is.na(pvalueW[i])) next                       # WHA error trapping
      }                                                   # WHA error trapping
      if((pvalueW[i] < alphaout ))   { j1 <- i; j2 <- i }
      if((pvalueW[i] < alphazeroin) & (j2==i-1) ) j2 <- i
   }

   # ---***--------------***--- TAC CRITICAL BUG ---***--------------***---
   # j2 <- min(c(j1,j2)) # WHA tentative completion of the 17C alogrithm!?!
   # HOWEVER MAJOR WARNING. WHA is using a minimum and not a maximum!!!!!!!
   # See MGBT17C() below. In that if the line a few lines above that reads
   # if((pvalueW[i] < alpha1 )) { j1 <- i; j2 <- i }
   # is replaced with if((pvalueW[i] < alpha1 )) j1 <- i
   # then maximum and not the minimum becomes applicable.
   # ---***--------------***--- TAC CRITICAL BUG ---***--------------***---

   # Curious that TAC does two length(x) below, but the length is already known.
   if(j2 == n2) { # TODO, does TAC change his mind on 5 retained observations?
                  # Also alphaout changes.
      if( n2 < n-5 ) { # set limit of at least 5 retained below 1/2 the sample
         message( paste0(" Number of low outliers equals or exceeds limit of ",n2) )
         zz <- MGBTcohn2013(x, alphaout=0.01, alphazeroin=0.10, n2=n-5,
                               napv.zero=napv.zero) # recursion
         return(zz)
      } else {
         message("MGBT identifies too many low outliers, use caution and judgment")
      }
   }
   zz <- list(pvalues=pvalueW, klow=j2,
              LOThresh=ifelse(j2 > 0, sort(x)[j2+1]+offset, 0))# quasi matches MGBTcohn2016
   zz$LOThresh[zz$LOThresh < 0] <- 0 # WHA 02/04/2019
   return(zz)  # the offset is by WHA
}

#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   Multiple Grubbs Beck test (Cohn et al., 2011)
#COHN      p. 169
#COHN MGB <- function(x,k,n=length(x)){
#COHN            y=sort(x);(y[k]-mean(y[(k+1):n]))/sqrt(var(y[(k+1):n]))}

"MGBTcohn2011" <- function(x, r=NULL, n=length(x)) {
   if(is.null(r)) {
      warning("the 'r'th-order truncation statistic requires definition")
      return(NULL)
   }
   # It is deliberate to not go through the data.frame/as.numeric etc here as
   # done for the other MGBT*() functions herein.
   x <- x[! is.na(x)]
   y <- sort(x)
   ( y[r] - mean(y[(r+1):n]) ) / sqrt( var(y[(r+1):n]) )
}

#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   Multiple Grubbs-Beck Test (MGBTnb)
#COHN     This eliminates the backup procedure
#COHN     Added for JL/JRS study 24 Aug 2012 (TAC)
#COHN
#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN MGBTnb <- function(Q,alpha1=0.005,alpha10=0.10,n2=floor(length(Q)/2)){
#COHN       zt      <- sort(log10(pmax(1e-8,Q)))
#COHN       n       <- length(zt)
#COHN       pvalueW <-rep(-99,n2);w<-rep(-99,n2)
#COHN       j1=0;j2=0
#COHN     for(i in 1:n2) {
#COHN        w[i]<-(zt[i]-mean(zt[(i+1):n]))/sqrt(var(zt[(i+1):n]))
#COHN        pvalueW[i]<-KthOrderPValueOrthoT(n,i,w[i])$value
#COHN        if(pvalueW[i]<alpha1){j1<-i;j2<-i}
#COHN     }
#COHN     if( (pvalueW[1]<alpha10) & (j2==0)){j2<-1}
#COHN     if(j2==n2){
#COHN       if(n2<length(Q)-5) {   # set a limit of at least 5 retained observations
#COHN         print(paste(" Number of low outliers equals or exceeds limit of ",n2));
#COHN         return(MGBT(Q,alpha1=0.01,alpha10=0.10,n2=length(Q)-5))} # try this
#COHN       else
#COHN         print("MGBT identifies too many low outliers; use caution and judgment")
#COHN     }
#COHN     return(list(klow=j2,pvalues=pvalueW,LOThresh=ifelse(j2>0,sort(Q)[j2+1],0)))
#COHN }

# In MGBTcohn2013, TAC uses recursion back to that same function. The MGBTnb()
# recurses to MGBTcohn2013 and not back to itself, which appears the only real
# difference between the two.
"MGBTnb" <-
function(x, alphaout=0.005, alphazeroin=0.10, n2=floor(length(x)/2),
            napv.zero=TRUE, offset=0) {
   if(is.data.frame(x)) {
      message("x tests as data.frame, helped out with 'x <- as.numeric(x$peak_va)'")
      x <- as.numeric(x$peak_va)
   }
   if(is.character(x)) {
      message("x tests as character, helped out with 'x <- as.numeric(x)'")
      x <- as.numeric(x)
   } else if(! is.numeric(x)) {
      warning("unable to proceed as x is not a numeric vector")
      return(NULL)
   }
   if(any(is.na(x))) {
      message("NAs detected in x, these were removed") # to suppress.
   }
   x <- x[! is.na(x)]
   # WHA error trapping
   if(length(unique(x)) == 1) return(list(pvalues=NA, klow=0, LOThresh=0))
   zt <- sort( log10(pmax(1e-8,x)) )
   n  <- length(zt)
   pvalueW <- w <- rep(NA, n2) # rep(-99,n2)
   j1 <- j2 <- 0
   for(i in 1:n2) {
      w[i] <- (zt[i]-mean(zt[(i+1):n]))/sqrt(var(zt[(i+1):n]))
         pvalueW[i] <- RthOrderPValueOrthoT(n,i,w[i])$value
      if(   is.na(pvalueW[i])) {                          # WHA error trapping
                  pvalueW[i] <- ifelse(napv.zero, 0, NA)  # WHA error trapping
         if(is.na(pvalueW[i])) next                       # WHA error trapping
      }                                                   # WHA error trapping
      if(pvalueW[i] < alphaout) {  j1 <- i; j2 <- i }
   }
   if(   pvalueW[1] < alphazeroin & j2 == 0 ) j2 <- 1 # Notice this is outside the loop and
   # also is slightly different logic than seen in MGBTcohn2013 and MGBTcohn2016.
   # It is curious that TAC does two length(x) in logic below, but that length is already known.
   if(j2 == n2) {
      if(n2 < n-5) {   # set a limit of at least 5 retained observations
         message(" Number of low outliers equals or exceeds limit of ",n2)
         return(MGBTcohn2013(x, alphaout=0.01, alphazeroin=0.10, n2=n-5),
                                napv.zero=napv.zero) # recursion
      } else {
         message("MGBT identifies too many low outliers; use caution and judgment")
      }
   }
   zz <- list(pvalues=pvalueW, klow=j2,
              LOThresh=ifelse(j2 > 0, sort(x)[j2+1]+offset, 0))# matches MGBTcohn2013
   zz$LOThresh[zz$LOThresh < 0] <- 0 # WHA 02/04/2019
   return(zz)  # the offset is by WHA
}

# This is a slower function in general practice between some omegas (w) get
# doubly computed. This function was developed by WHA explicitly following
# verbatim the language inside the B17C publication:
# https://pubs.er.usgs.gov/publication/tm4B5
# This function serves as a backstop against TAC having a systemtic and
# undetected fault in his R idiom implementation of the MGBT in the various
# incarations credited to him within this source code file.
"MGBT17c.verb" <- # arguments match MGBT17C
function(x, alphaout=0.005, alphain=0, alphazeroin=0.10, n2=floor(length(x)/2),
            napv.zero=TRUE, offset=0, min.obs=0) {
   if(is.data.frame(x)) {
      message("x tests as data.frame, helped out with 'x <- as.numeric(x$peak_va)'")
      x <- as.numeric(x$peak_va)
   }
   if(is.character(x)) {
      message("x tests as character, helped out with 'x <- as.numeric(x)'")
      x <- as.numeric(x)
   } else if(! is.numeric(x)) {
      warning("unable to proceed as x is not a numeric vector")
      return(NULL)
   }
   if(any(is.na(x))) {
      message("NAs detected in x, these were removed") # to suppress.
   }
   x <- x[! is.na(x)]
   n <- length(x); idx <- c(n,n2,rep(NA, 3))
   txt <- c("n", "n2", "ix_alphaout", "ix_alphain", "ix_alphazeroin")
   names(idx) <- txt
   if(length(unique(x)) == 1) return(list(index=idx, klow=0, LOThresh=0))
   if(n < min.obs)            return(list(index=idx, klow=0, LOThresh=0))
   if(n2 > n) {
      warning("n2 > n, which does not make not logical sense")
      return(NULL)
   }
   small <- sqrt(.Machine$double.eps); j1 <- j2 <- 0
   x <- sort(x); zt <- log10( pmax(small, x) )

   for(i in seq(n2,1,by=-1)) { # NOTE LITERALLY SWEEP OUT as per 17C
      w  <- (zt[i]-mean(zt[(i+1):n])) / sqrt(var(zt[(i+1):n]))
      pv <- RthOrderPValueOrthoT(n, i, w)$value
      if(napv.zero & is.na(pv)) pv <- 0
      if(is.na(pv)) next
      if( pv >= alphaout) { j1 <- i; next } # effect of sweep out, NOTE GREATER THAN EQUAL
      break
   }

   j1 <- j1 - 1 # this is critical!
   for(i in seq(1,n2,by=+1)) { # NOTE LITERALLY SWEEP IN as per 17C
      w  <- (zt[i]-mean(zt[(i+1):n])) / sqrt(var(zt[(i+1):n]))
      pv <- RthOrderPValueOrthoT(n, i, w)$value
      if(napv.zero & is.na(pv)) pv <- 0 # NA handling
      if(is.na(pv)) next # protection from a NA leaking into these
      if((pv <= alphazeroin ) & (j2==i-1)) { j2 <- i; next } # effect of sweep in,
      # NOTE LESS THAN EQUAL and NOT LESS THAN
      break
   }
   k <- max(j1,j2) # as per B17C
   if(k >= n) return(list(index=idx, klow=0, LOThresh=0))
   idx <- c(n,n2,j1,NA,j2); names(idx) <- txt
   zz <- list(index=idx, klow=k,
              LOThresh=ifelse(k > 0, x[k+1]+offset, 0))
   zz$LOThresh[zz$LOThresh < 0] <- 0 # WHA 02/04/2019
   return(zz)
}


# This code base follows more along lines of WHA idioms implementing B17C in
# the spirit of TAC's coding and faster than the verbatim implementation mentioned
# above. This function is to be the user-level version of the MGBT.
"MGBT17c" <- # arguments match MGBTcohn2013 but the alphas were renamed to match 17C
function(x, alphaout=0.005, alphain=0, alphazeroin=0.10, n2=floor(length(x)/2),
            napv.zero=TRUE, offset=0, min.obs=0) {
   if(is.data.frame(x)) {
      message("x tests as data.frame, helped out with 'x <- as.numeric(x$peak_va)'")
      x <- as.numeric(x$peak_va)
   }
   if(is.character(x)) {
      message("x tests as character, helped out with 'x <- as.numeric(x)'")
      x <- as.numeric(x)
   } else if(! is.numeric(x)) {
      warning("unable to proceed as x is not a numeric vector")
      return(NULL)
   }
   if(any(is.na(x))) {
      message("NAs detected in x, these were removed") # to suppress.
   }
   x <- x[! is.na(x)]
   n <- length(x); idx <- c(n,n2,rep(NA, 3))
   txt <- c("n", "n2", "ix_alphaout", "ix_alphain", "ix_alphazeroin");
   if(length(unique(x)) == 1) return(list(index=idx, omegas=NA, x=NA, pvalues=NA,
                               klow=0, LOThresh=0, message="all data are equal"))
   if(n < min.obs) return(list(index=idx, omegas=NA, x=NA, pvalues=NA,
                               klow=0, LOThresh=0, message="too few data"))
   if(n2 > n) {
      warning("n2 > n, which does not make not logical sense")
      return(NULL)
   }
   small <- sqrt(.Machine$double.eps); j1 <- j2 <- 0
   x <- sort(x); zt <- log10( pmax(small, x) )

   # 17C says sweeps out and in and STOP each time, which is the effect we seek, but for
   # simplicity motivated by desire to return p-values for all 1:n2 values  we must
   # expend CPU time computing all omegas and pvalues. This was TAC motivation (at least
   # concluded) for a single loop. Below are R idioms by WHA.
   w  <- sapply(1:n2, function(i) (zt[i]-mean(zt[(i+1):n])) /
                                     sqrt(var(zt[(i+1):n])) )
   pv <- sapply(1:n2, function(i) RthOrderPValueOrthoT(n, i, w[i])$value) # p-values
   sw <- mgbt.sweeper(pv, alphaout=alphaout, alphain=alphain, alphazeroin=alphazeroin)
   k <- sw$index
   # don't know of this is next line is legitimately accessible but trap anyway
   if(k >= n) return(list(index=idx, omegas=NA, x=NA, pvalues=NA, klow=0, LOThresh=0))
   idx <- c(n,n2,sw$sweeps[1],sw$sweeps[2],sw$sweeps[3]); names(idx) <- txt
   zz <- list(index=idx, omegas=w[1:n2], x=x[1:n2], pvalues=pv, klow=k,
              LOThresh=ifelse(k > 0, x[k+1]+offset, 0))
   zz$LOThresh[zz$LOThresh < 0] <- 0 # WHA 02/04/2019
   return(zz)
}


"mgbt.sweeper" <-
function(pvs, alphaout=0.005, alphain=0, alphazeroin=0.1) {
   j1 <- j2 <- j3 <- 0 # outward, inward from 1+ outward, inward sweep
   n2 <- length(pvs)   # the js are numer of low outliers and not the index of the threshold
   if(any(is.na(pvs))) stop("NA p-value, wrapper code should make decision")
   for(i in seq(n2,1,-1)) if(pvs[i] <    alphaout) { j1 <- i;  break }
   if(j1 < n2) { # Late addition station 08020500 causes j1 to be n2 [A WHA fix to TAC.]
      # with Q=c(19500, 17400, 9350, 3760, 8410, 15200, 5370, 22300, 21500, 6590, 17300)
      for(i in j1+1:n2) if(pvs[i] >=     alphain) { j2 <- i-1; break }
   } else if(j1 == n2) {
      # This means that the last pv is < alphaout
   } else {
      stop("should not be here in logic")
   }
   for(i in         1:n2) if(pvs[i] >= alphazeroin) { j3 <- i-1; break }
   js <- c(j1,j2,j3); names(js) <- c("ix_alphaout", "ix_alphain", "ix_alphazeroin")
   return(list(index=max(js), sweeps=js))
}

# outward sweep number of low outliers
# inward sweep number of low outliers
# other inward sweep number of low outliers

# Peer Review Notes: Chandramauli Awasthi, North Carolina State University (07/23/2019)
# I tested the package for some of the sites using MGBTcohn2016 and MGBTnb
# functions. How are these two functions different, as they are found to be
# giving slightly different results in few cases?

# WHA (07/23/2019): Mauli, note that my comments in MGBT.R indicate that MGBTnb
# has some conditional
#  if(   pvalueW[1] < alphazeroin & j2 == 0 ) j2 <- 1
# that resides outside a loop compared to MGBTcohn2016 (or MGBTcohn2013). The
# MGBTnb() uses sometype of recursion. I do not understand what TAC meant to
# do with MGBTnb() or what type of experiment is was pursuing. Please note that
# MGBTcohn2016/2013 and MGBTnb() are retained for historical reference and are
# not intended for deployment. As to differences that you have seen in the
# performance between the two, this is related to subtle differences at the time
# in TAC's thoughts. The differences are restricted to whether the
# X_k or X_(k+1) order statistics are chosen as the result of the MGBT.
# Finally, by design the "official" implementation of this package is
# "MGBT" <- function(...) MGBT17c(...) # which is the first line of this file.

#COHN ****|==|====-====|====-====|====-====|====-====|====-====|====-====|==////////
#COHN
#COHN   Inverse function:  Computing critical point given probability
#COHN     Note that initial guess is based on approximating integral by
#COHN          function value at 50%-ile.
#COHN
#CritK<-function(n,r,p){
#  guess1 = uniroot(function(eta)(peta(.5,n,r,eta)-p),
#           interval=c(-10,10))$root
#  uniroot(function(x,n,r) {KthOrderPValueOrthoT(n,r,x)$value-p},
#           interval=guess1+c(-1,1),n=n,r=r)$root
#  }

"CritK" <- function(n, r, p) {

  "gfunc" <- function(eta) (peta(0.5, n, r, eta) - p)
  guess1 <- NULL
  try(guess1  <- uniroot(gfunc, interval=c(-10,10))$root)
  if(is.null(guess1)) {
     warning("could not uniroot guess1, do not know what to do, need TAC, NA returned")
     return(NA)
  }

  "kfunc" <- function(x,n,r) { RthOrderPValueOrthoT(n, r, x)$value - p }
  critk <- NULL
  try(critk <- uniroot(kfunc, interval=guess1+c(-1,1), n=n, r=r)$root)
  if(is.null(critk)) {
     warning("could not uniroot kfunc, do not know what to do, need TAC, NA returned")
     return(NA)
  }
  return(critk)
}

# README (path ./MGBT/inst/INSTALL_HELP.md)

#### Author:           William H. Asquith
#### Point of contact: William H. Asquith (wasquith@usgs.gov)

***
***

This file describes one of the more "raw" methods of installing the **MGBT** _R_ package from its original sources or from the download of a tagged version of the repository. This is an obviously helpful tutorial in the event that non-CRAN installation through the **remotes** package does not function properly. (See also the top-level `README.md` of this package.)

The instructions here depict the use of **RStudio** (accessed November 26, 2019, at https://rstudio.com/) on the Windows platform for **MGBT** installation. A note is made that the actual installation simply uses the `install.packages()` function from the standard **utils** package.

***

1. The process begins by downloading the `.tar.gz` format of the source code from the repository, which in this case is the "master" of the repository as shown in the screenshot that follows. Other `.tar.gz` versions could come from any "tagged" version of the repository. The author notes that downloading the default `.zip` format, even though that is a common archive format on standard PCs, does not appear to work.

<img src='inst/www/step1_src_MGBT_install.png' height='320' align="middle" />

***

2. After the file is downloaded, which is titled `MGBT-master.tar.gz`, from the **RStudio** interface, the _Tools --> Install packages_ menu option is selected. Then the _Install from:_ is switched, if needed, to the "Package Archive File (.tgz, .tar.gz)" option. The switch may or may not then spawn a file selector dialog box, but such dialog can be also triggered by the _Browse..._ button. The user needs to navigate their file system to the location of the `MGBT-master.tar.gz` and choose that file. The following screenshot summarizes this step.

<img src='inst/www/step2_src_MGBT_install.png' height='460' align="middle" />

***

3. Inspect the installation process for success and test loading the **MGBT** package. The following screenshot summarizes this "step." It is seen that that the `install.packages()` function was appropriately configured. The author notes the warning message about the **Rtools** executable, which evidently is code specific to the Windows platform (accessed on November 26, 2019, at https://cran.r-project.org/bin/windows/Rtools/). The warning message appears harmless, which might be the case because the **MGBT** package does have compiled code, just native _R_.

<img src='inst/www/step3_src_MGBT_install.png' height='977' align="middle" />

\encoding{utf8}
\name{MGBT}
\alias{MGBT}
\alias{MGBTnb}
\alias{MGBTcohn2016}
\alias{MGBTcohn2013}
\alias{MGBTcohn2011}
\alias{MGBT17c}
\alias{MGBT17c.verb}
\title{Multiple Grubbs--Beck Test (MGBT) for Low Outliers }
\description{
Perform the Multiple Grubbs--Beck Test (MGBT; Cohn and others, 2013) for low outliers (LOTs, low-outlier threshold; potentially influential low floods, PILFs) that is implemented in the USGS-PeakFQ software (USGS, 2014; Veilleux and others, 2014) for implementation of Bulletin 17C (B17C) (England and others, 2018). The test internally transforms the data to logarithms (base-10) and thus is oriented for positively distributed data but accommodates zeros in the dataset.

The essence of the MGBT, given the order statistics \eqn{x_{[1:n]} \le x_{[2:n]} \le \cdots \le x_{[(n-1):n]} \le x_{[n:n]}}, is the statistic
\deqn{GB_r = \omega_r =
\frac{ x_{[r:n]} - \mathrm{mean}\{x_{[(r+1)\rightarrow n:n]}\} }
     {\sqrt{\mathrm{var}\{x_{[(r+1)\rightarrow n:n]}\}}}\mbox{,}
}
which is can be computed by \code{MGBTcohn2011} that is a port a function of TAC's used in a testing script that is reproduced in the \bold{Examples} of \code{\link{RSlo}}. Variations of this pseudo-standardization scheme are shown for \code{\link{BLlo}} and \code{\link{RSlo}}. Also, \eqn{GB_r} is the canonical form of the variable \code{eta} in TAC sources and \code{peta}=\code{\link{peta}} will be its associated probability.
}
\usage{
MGBT(...) # A wrapper on MGBT17C()---This is THE function for end users.

     MGBT17c(x, alphaout=0.005, alphain=0, alphazeroin=0.10,
                n2=floor(length(x)/2), napv.zero=TRUE, offset=0, min.obs=0)
MGBT17c.verb(x, alphaout=0.005, alphain=0, alphazeroin=0.10,
                n2=floor(length(x)/2), napv.zero=TRUE, offset=0, min.obs=0)

MGBTcohn2016(x, alphaout=0.005, alphazeroin=0.10, n2=floor(length(x)/2),
                napv.zero=TRUE, offset=0)
MGBTcohn2013(x, alphaout=0.005, alphazeroin=0.10, n2=floor(length(x)/2),
                napv.zero=TRUE, offset=0)
      MGBTnb(x, alphaout=0.005, alphazeroin=0.10, n2=floor(length(x)/2),
                napv.zero=TRUE, offset=0)

MGBTcohn2011(x, r=NULL, n=length(x)) # only computes the GB_r, not a test
}
\arguments{
  \item{...}{Arguments to pass to the MGBT family of functions;}
  \item{x}{The data values and note that base-10 logarithms of these are computed internally except for the operation of the \code{MGBTcohn2011} function, which does not (see \bold{Examples} for \code{\link{RSlo}}). Also protection from zero or negative values is made by the \R function \code{pmax}, and these values are replaced with a \dQuote{small} value of \code{1e-8} and tacitly TAC has assumed that p-values for these will be significantly small and truncated away;}
  \item{alphaout}{Literally the \eqn{\alpha_\mathrm{out}} of Bulletin 17C. This is the significance level of the \dQuote{sweep out} portion of MGBT;}
  \item{alphain}{This is the significance level of the \dQuote{sweep in} portion of MGBT but starts at one plus the order statistic identified by \code{alphaout};}
  \item{alphazeroin}{Literally the \eqn{\alpha_\mathrm{in}} of Bulletin 17C. This is the significance level of the \dQuote{sweep in} portion of MGBT;}
  \item{napv.zero}{A logical switch to reset a returned \code{NA} from \code{\link{RthOrderPValueOrthoT}} to zero. This is a unique extension by WHA based on large-scale batch testing of the USGS peak-values database (see \bold{Note}). This being said, the fall-back to Monte Carlo integration if the numerical integration fails, seems to mostly make this argument superfluous;}
  \item{offset}{The offset, if not \code{NA}, is added from the threshold unless the threshold itself is already zero. In practical application, this offset, if set, would likely be a negative quantity. This argument is a unique extension by WHA;}
  \item{min.obs}{The minimum number of observations. This option is provided to streamline larger applications, but the underlying logic in \code{MGBT17C} is robust and on failures because of small sizes return a threshold of \code{0} anyway;}
  \item{n2}{The number of \code{n2}-smallest values to be evaluated in the MGBT;}
  \item{r}{The number of truncated observations, which can be though of the rth order statistic and below; and}
  \item{n}{The number of observations. It is not clear that TAC intended \code{n} to be not equal to the sample size but TAC chose to not keep the length of \code{x} as determined internally to the function but to have it also available as an argument. Functions \code{\link{BLlo}} and \code{\link{RSlo}} also were designed similarly.}
}
\value{
  The MGBT results as an \R \code{list}:
  \item{index}{The sample size \eqn{n}, the value for \code{n2}, and the three indices of the \dQuote{sweep out,} \dQuote{sweep in,} and \dQuote{sweep in from zero} processing (only for \code{MGBT17c} as this is an extension from TAC);}
  \item{omegas}{The \eqn{GB_r = \omega_r} statistics for which the p-values in \code{pvalues} are shown. These are mostly returned for aid in debugging and verification of the algorithms;}
  \item{x}{The \code{n2}-smallest values in increasing order (only for \code{MGBT17c} as this is an extension from TAC);}
  \item{pvalues}{The p-values of the \code{n2}-smallest values of the sample (not available for \code{MGBT17c} because of algorithm design for speed);}
  \item{klow}{The number of low outliers detected;}
  \item{LOThresh}{The low-outlier threshold for the \code{klow+1} index of the sample (and possibly adjusted by the \code{offset}) or simply zero; and}
  \item{message}{Possibly message in event of some internal difficulty.}
The inclusion of \code{x} in the returned value is to add symmetry because the p-values are present. The inclusion of \eqn{n} and \code{n2} might make percentage computations of inward and outward sweep indices useful in exploratory analyses. Finally, the inclusion of the sweep indices is important as it was through inspection of these that the problems in TAC sources were discovered.
}
\note{
\bold{Porting from TAC sources}---TAC used MGBT for flood-frequency computations by a call
\preformatted{
  oMGBT <- MGBT(Q=o_in@qu[o_in@typeSystematic])
}
in file \code{P3_089(R).txt}, and note the named argument \code{Q=} but consider in the definition \code{Q} is not defined as a named argument. For the \pkg{MGBT} package, the \code{Q} has been converted to a more generic variable \code{x}. Development of TAC's B17C version through the \code{P3_089(R).txt} or similar sources will simply require \code{Q=} to be removed from the \code{MGBT} call.

The original MGBT algorithms in \R by TAC will throw some errors and warnings that required testing elsewhere for completeness (non-\code{NULL}) in algorithms \dQuote{up the chain.} These errors appear to matter materially in pratical application in large-scale batch processing of USGS Texas peak-values data by WHA and GRH. For package \pkg{MGBT}, the TAC computations have been modified to wrap a \code{try()} around the numerical integration within \code{\link{RthOrderPValueOrthoT}} of \code{\link{peta}}, and insert a \code{NA} when the integration fails if and only if a second integration (fall-back) attempt using Monte Carlo integration fails as well.

The following real-world data were discovered to trigger the error/warning messages. These example data crash TAC's MGBT and the data point of 25 cubic feet per second (cfs) is the culpret. If a failure is detected, Monte Carlo integration is attempted as a fall-back procedure using defaults of \code{\link{RthOrderPValueOrthoT}}, and if that integration succeeds, \code{MGBT}, which is not aware, simply receives the p-value. If Monte Carlo integration also fails, then for the implementation in package \pkg{MGBT}, the p-value is either a \code{NA} (\code{napv.zero=FALSE}) or set to zero if \code{napv.zero=TRUE}. Evidence suggests that numerical difficulties are encountered when small p-values are involved.
\preformatted{
  # Peak streamflows for 08385600 (1952--2015, systematic record only)
  #https://nwis.waterdata.usgs.gov/nwis/peak?site_no=08385600&format=hn2
  Data  <- c(  8100, 3300,  680, 14800,  25.0, 7310, 2150, 1110, 5200, 900, 1150,
   1050,  880, 2100, 2280, 2620,   830,  4900,  970,  560,  790, 1900, 830,  255,
   2900, 2100,    0,  550, 1200,  1300,   246,  700,  870, 4350,  870, 435, 3000,
    880, 2650,  185,  620, 1650,   680, 22900, 3290,  584, 7290, 1690, 2220, 217,
   4110,  853,  275, 1780, 1330,  3170,  7070, 2660) # cubic feet per second (cfs)
  MGBT17c(     Data, napv.zero=TRUE)$LOThres    # [1] 185
  MGBT17c.verb(Data, napv.zero=TRUE)$LOThres    # [1] 185
  MGBTcohn2016(Data, napv.zero=TRUE)$LOThres    # [1] 185
  MGBTcohn2013(Data, napv.zero=TRUE)$LOThres    # [1] 185
  MGBTnb(      Data, napv.zero=TRUE)$LOThres    # [1] 185
}
Without having the fall-back Monte Carlo integration in \code{\link{RthOrderPValueOrthoT}}, if \code{napv.zero=} \code{FALSE}, then the low-outlier threshold is 25 cfs, but if \code{napv.zero=TRUE}, then the low-outlier threshold is 185 cfs, which is the value matching USGS-PeakFQ (v7.1) (FORTRAN code base). Hence, the recommendation that \code{napv.zero=TRUE} for the default, though such a setting will for this example will still result in 185 cfs because Monte Carlo integration can not be turned off for the implementation here.

Noting that USGS-PeakFQ (7.1) reports the p-value for the 25 cfs as 0.0002, a test of the \pkg{MGBT} implementation with the backup Monte Carlo integration in \code{\link{RthOrderPValueOrthoT}} shows a p-value of 1.748946e-04 for 25 cfs, which is congruent with the 0.0002 of USGS-PeakFQ (v7.1). Another Monte Carlo integration produced a p-value of 1.990057e-04 for 25 cfs, and thus another result congruent with USGS-PeakFQ (v7.1) is evident.

Using original TAC sources, here are the general errors that can be seen:
\preformatted{
   Error in integrate(peta, lower = 1e-07, upper = 1 - 1e-07, n = n, r = r, :
   the integral is probably divergent In addition:
   In pt(q, df = df, ncp = ncp, lower.tail = TRUE) :
     full precision may not have been achieved in 'pnt[final]'
}
For both the internal implementations of \code{\link{RthOrderPValueOrthoT}} and \code{\link{peta}} error trapping is present to return a \code{NA}. It is not fully known whether the integral appears divergent when \code{pt()} (probability of the t-distribution) reaches an end point in apparent accuracy or not---although, this is suspected. For this package, a \code{suppressWarnings()} has been wrapped around the call to \code{pt()} in \code{\link{peta}} as well as in one other computation that at least in small samples can result in a square root of a negative number (see \bold{Note} under \code{\link{peta}}).

There is another error to trap for this package. If all the data values are identical, a low-outlier threshold set at that value leaks back. This is the motivation for a test added by WHA using \code{length(unique(x)) == 1} in the internals of \code{MGBTcohn2016}, \code{MGBTcohn2013}, and \code{MGBTnb}.

A known warning message might be seen at least in microsamples:
\preformatted{
   MGBT(c( 1, 26300))       # throws    warnings, zero is the threshold
     # In EMS(n, r, qmin) : value out of range in 'lgamma'
   MGBT(c( 1, 26300, 2600)) # throws no warnings, zero is the threshold
}
The author wraps a \code{suppressWarnings()} on the line in \code{\link{EMS}} requiring \code{lgamma()}. This warning appears restricted to nearly a degenerate situation anyway and failure will result in \code{\link{peta}} and therein the situation results in a p-value of unity and hence no risk of identifying a threshold.

Regarding \code{n=length(x)} for \code{MGBTcohn2011}, it is not clear whether TAC intended \code{n} to be not equal to the sample size. TAC chose to not determine the length of \code{x} internally to the function but to have it available as an argument. Also \code{\link{BLlo}} and \code{\link{RSlo}} were designed similarly.

\bold{(1) Lingering Issue of Inquiry}---TAC used a \code{j1} index in \code{MGBTcohn2016}, \code{MGBTcohn2013}, and \code{MGBTnb}, and this index is used as part of \code{alphaout}. The \code{j1} and is not involved in the returned content of the MGBT approach. This seems to imply a problem with the \dQuote{sweep out} approach but TAC inadvertantly seems to make \dQuote{sweep out} work in \code{MGBTnb} but therein creates a \dQuote{sweep in} problem. The \dQuote{sweep in} appears fully operational in \code{MGBTcohn2016} and \code{MGBTcohn2013}. Within the source for \code{MGBTcohn2016} and \code{MGBTcohn2013}, a fix can be made with just one line after the \code{n2} values have been processed. Here is the WHA commentary within the sources, and it is important to note that the fix is not turned on because \bold{use of \code{MGBT17c} via the wrapper \code{MGBT} is the recommended interface}:
\preformatted{
   # ---***--------------***--- TAC CRITICAL BUG ---***--------------***---
   # j2 <- min(c(j1,j2)) # WHA tentative completion of the 17C alogrithm!?!
   # HOWEVER MAJOR WARNING. WHA is using a minimum and not a maximum!!!!!!!
   # See MGBT17C() below. In that if the line a few lines above that reads
   # if((pvalueW[i] < alpha1 )) { j1 <- i; j2 <- i }
   # is replaced with if((pvalueW[i] < alpha1 )) j1 <- i
   # then maximum and not the minimum becomes applicable.
   # ---***--------------***--- TAC CRITICAL BUG ---***--------------***---
}

\bold{(2) Lingering Issue of Inquiry}---TAC used recursion in \code{MGBTcohn2013} and \code{MGBTnb} for a condition \code{j2 == n2}. This recursion does not exist in \code{MGBTcohn2016}. TAC seems to have explored the idea of a modification to the \code{n2} to be related to an idea of setting a limit of at least five retained observations below half the sample (default \code{n2}) when up to half the sample is truncated away. Also in the recursion, TAC resets the two alpha's with \code{alphaout=0.01} from the default of \code{alpha1=0.005} and \code{alphazeroin=0.10}. This reset means that had TAC hardwired these inside \code{MGBTcohn2013}, which partially defeats the purpose of having them as arguments in the first place.

\bold{(3) Lingering Issue of Inquiry}---TAC used recursion in \code{MGBTcohn2013} and \code{MGBTnb} for a condition \code{j2 == n2} but the recursion calls \code{MGBTcohn2013}. Other comments about the recursion in Inquiry (2) about the alpha's are applicable here as well. More curious about \code{MGBTnb} is that the \code{alphazeroin} is restricted to a test on only the p-value for the smallest observation and is made outside the loop through the data. The test is \code{if(pvalueW[1] <} \code{alphazeroin &} \code{j2 == 0)} \code{j2 <- 1}, which is a logic flow that differs from that in \code{MGBTcohn2013} and \code{MGBTcohn2016}.

\bold{On the Offset Argument}---The MGBT approach identifies the threshold as the first order statistic (\eqn{x_{[r+1:n]}}) above that largest \dQuote{outlying} order statistic \eqn{x_{[r:n]}} with the requisite small p-value (see the example in the \bold{Examples} section herein). There is a practical application in which a nudge on the MGBT-returned low-outlier threshold could be useful. Consider that the optional \code{offset} argument is added to the threshold unless the threshold itself is already zero. The offset should be small, and as an example \eqn{{-}0.001} would be a magnitude below the resolution of the USGS peak-values database.

Why? If algorithms other than USGS-PeakFQ are involved in frequency analyses, the concept of \dQuote{in} or \dQuote{out} of analyses, respectively, could be of the form \code{xin <-} \code{x[x > threshold]} and \code{xlo <-} \code{x[x <= threshold]}. Note the \dQuote{less than or equal to} and its assocation with those data to be truncated away, which might be more consistent with the idea of truncation level of left-tail censored data in other datasets for which the MGBT approach might be used. For example, the \code{x2xlo()} function of the \pkg{lmomco} package by Asquith (2019) uses such logic in its implementation of conditional probability adjustment for the presence of low outliers. This logic naturally supports a default truncation for zeros.

Perhaps one reason for the difference in how a threshold is implemented is based on two primary considerations. First, if restricted to choosing a threshold to the sample order statistics, then the \code{MGBT} approach, by returning the first (earliest) order statistics that is not statistically significant, requires \code{x[x < threshold]} for the values to leave out. TAC clearly intended this form. Second, TAC seems to approach the zero problem with \code{MGBT} by replacing all zeros with \code{1e-8} as in
\preformatted{
  log10(pmax(1e-8,x))
}
immediately before the logarithmic transformation. This might be a potential weak link in \code{MGBT}. It assumes that \code{1e-8} is small, which it certainly is for the problem of flood-frequency analysis using peaks in cubic feet per second. TAC has hardwired this value. Reasonable enough.

But such logic thus requires the \code{MGBT} to identify these pseudo-zeros (now \code{1e-8}) in all circumstances as low outliers. Do the algorithms otherwise do this? This approach is attractive for B17C because one does not have to track a subsample of those values greater than zero because the low-outlier test will capture them. An implementation such as Asquith (2019) automatically removes zero without the need to use a low-outlier identification method; hence, Asquith's choice of \code{x[x <= threshold]} with \code{threshold=0} by default for the values to leave out. The inclusion of \code{offset} permits cross compatibility for \pkg{MGBT} package purposes trancending Bulletin 17C.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, \code{P3_089(R).txt}---Named \code{MGBT} + \code{MGBTnb}
}
\references{
Asquith, W.H., 2019, lmomco---L-moments, trimmed L-moments, L-comoments, censored \cr L-moments, and many distributions: R package version 2.3.2 (September 20, 2018), accessed March 30, 2019, at \doi{10.32614/CRAN.package.lmomco}.

Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.

Cohn, T.A., England, J.F., Berenbrock, C.E., Mason, R.R., Stedinger, J.R., and Lamontagne, J.R., 2013, A generalized Grubbs-Beck test statistic for detecting multiple potentially influential low outliers in flood series: Water Resources Research, v. 49, no. 8, pp. 5047--5058, \doi{10.1002/wrcr.20392}.

England, J.F., Cohn, T.A., Faber, B.A., Stedinger, J.R., Thomas Jr., W.O., Veilleux, A.G., Kiang, J.E., and Mason, R.R., 2018, Guidelines for determining flood flow frequency Bulletin 17C: U.S. Geological Survey Techniques and Methods, book 4, chap. 5.B, 148 p., \doi{10.3133/tm4B5}

U.S. Geological Survey (USGS), 2018, PeakFQ---Flood frequency analysis based on Bulletin 17B and recommendations of the Advisory Committee on Water Information (ACWI) Subcommittee on Hydrology (SOH) Hydrologic Frequency Analysis Work Group (HFAWG), version 7.2.

Veilleux, A.G., Cohn, T.A., Flynn, K.M., Mason, R.R., Jr., and Hummel, P.R., 2014, Estimating magnitude and frequency of floods using the PeakFQ 7.0 program: U.S. Geological Survey Fact Sheet 2013--3108, 2 p., \doi{10.3133/fs20133108}.
}
\author{W.H. Asquith}
\seealso{\code{\link{RthOrderPValueOrthoT}}
}
\examples{
\donttest{
# USGS 08066300 (1966--2016) # cubic feet per second (cfs)
#https://nwis.waterdata.usgs.gov/nwis/peak?site_no=08066300&format=hn2
Values <- c(3530, 284, 1810, 9660,  489,  292, 1000,  2640, 2910, 1900,  1120, 1020,
   632, 7160, 1750,  2730,  1630, 8210, 4270, 1730, 13200, 2550,  915, 11000, 2370,
  2230, 4650, 2750,  1860, 13700, 2290, 3390, 5160, 13200,  410, 1890,  4120, 3930,
  4290, 1890, 1480, 10300,  1190, 2320, 2480, 55.0,  7480,  351,  738,  2430, 6700)
MGBT(Values) # Results LOT=284 cfs leaving 55.0 cfs (p-value=0.0119) censored.
#$index
#      n      n2    ix_alphaout     ix_alphain   ix_alphazeroin
#     51      25              0              0                1
#$omegas
# [1] -3.781980 -2.268554 -2.393569 -2.341027 -2.309990 -2.237571
# [7] -2.028614 -1.928391 -1.720404 -1.673523 -1.727138 -1.671534
#[13] -1.661346 -1.391819 -1.293324 -1.246974 -1.276485 -1.272878
#[19] -1.280917 -1.310286 -1.372402 -1.434898 -1.226588 -1.237743
#[25] -1.276794
#$x
# [1]   55  284  292  351  410  489  632  738  915 1000 1020 1120 1190 1480 1630 1730
#[17] 1750 1810 1860 1890 1890 1900 2230 2290 2320
#$pvalues
# [1] 0.01192184 0.30337879 0.08198836 0.04903091 0.02949836 0.02700114 0.07802324
# [8] 0.11185553 0.31531749 0.34257170 0.21560086 0.25950150 0.24113157 0.72747052
#[15] 0.86190920 0.89914152 0.84072131 0.82381908 0.78750571 0.70840262 0.55379730
#[22] 0.40255392 0.79430336 0.75515103 0.66031442
#$LOThresh
#[1] 284

# The USGS-PeakFQ (v7.1) software reports:
#   EMA003I-PILFS (LOS) WERE DETECTED USING MULTIPLE GRUBBS-BECK TEST   1     284.0
#      THE FOLLOWING PEAKS (WITH CORRESPONDING P-VALUES) WERE CENSORED:
#            55.0    (0.0123)
# As a curiosity, see Examples under ASlo().#}

\donttest{
# MGBTnb() has a sweep in problem.
SweepIn <- c(1, 1, 3200, 5270, 26300, 38400, 8710, 23200, 39300, 27800, 21000,
  21000, 21500, 57000, 53700, 5720, 10700, 4050, 4890, 10500, 26300, 16600, 20900,
  21400, 10800, 8910, 6360)  # sweep in and out both identify index 2.
MGBT17c(SweepIn, alphaout=0)$LOThres # LOT = 3200 # force no sweep outs
MGBTnb(SweepIn)$LOThres              # LOT = 3200 # because sweep out is the same!
MGBTnb(SweepIn, alphaout=0) # LOT = 1  # force no sweep outs, it fails.}
}
\keyword{MGBT}
\keyword{Multiple Grubbs--Beck Test}
\keyword{low outlier (definition)}
\keyword{Bulletin 17C}

\encoding{utf8}
\name{VMS}
\alias{VMS}
\title{Covariance matrix of M and S}
\description{
Compute the covariance matrix of \eqn{M} and \eqn{S} given \eqn{q_\mathrm{min}}.  Define the vector of four moment expectations
\deqn{E_{i\in 1,2} = \Psi\bigl(\Phi^{(-1)}(q_\mathrm{min}), i\bigr)\mbox{,}}
where \eqn{\Psi(a,b)} is the \code{\link{gtmoms}} function and \eqn{\Phi^{(-1)}} is the inverse of the standard normal distribution. Define the scalar quantity \eqn{Es = } \code{EMS(n,r,qmin)[2]} as the expectation of \eqn{S} using the \code{\link{EMS}} function, and define the scalar quantity \eqn{E_s^2 = E_2 - E_1^2} as the expectation of \eqn{S^2}. Finally, compute the covariance matrix \eqn{COV} of \eqn{M} and \eqn{S} using the \code{\link{V}} function:
\deqn{COV_{1,1} = V_{1,1}\mbox{,}}
\deqn{COV_{1,2} = COV_{2,1} = V_{1,2}/2Es\mbox{,}}
\deqn{COV_{2,2} = E_s^2 - (E_s)^2\mbox{.}}
}
\usage{
VMS(n, r, qmin)
}
\arguments{
  \item{n}{The number of observations;}
  \item{r}{The number of truncated observations; and}
  \item{qmin}{A nonexceedance probability threshold for \eqn{X > q_\mathrm{min}}.}
}
\value{
  A 2-by-2 covariance \code{matrix}.
}
\note{
Because the \code{gtmoms} function is naturally vectorized and TAC sources provide no protection if \code{qmin} is a vector (see \bold{Note} under \code{\link{EMS}}). For the implementation here, only the first value in \code{qmin} is used and a warning issued if it is a vector.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, \code{P3_089(R).txt}---Named \code{VMS}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.
}
\author{W.H. Asquith consulting T.A. Cohn sources}
\seealso{\code{\link{EMS}}, \code{\link{V}}, \code{\link{gtmoms}}
}
\examples{
VMS(58,2,.5) # Note that [1,1] is the same as [1,1] for Examples under V().
#            [,1]        [,2]
#[1,] 0.006488933 0.003279548
#[2,] 0.003279548 0.004682506
}
\keyword{moments}
\keyword{moments (conditional)}
\keyword{utility functions}

\encoding{utf8}
\name{peta}
\alias{peta}
\title{Probability of Eta}
\description{
Compute \code{peta}, which is the survival probability of the t-distribution for \code{eta =} \eqn{\eta}.

Define \eqn{b_r} as the inverse (quantile) of the Beta distribution for nonexceedance probability \eqn{F \in (0,1)} having two shape parameters (\eqn{\alpha} and \eqn{\beta}) as
\deqn{b_r = \mathrm{Beta}^{(-1)}(F; \alpha, \beta) = \mathrm{Beta}^{(-1)}(F; r, n+1-r)\mbox{,}}
for sample size \eqn{n} and number of truncated observations \eqn{r} and note that \eqn{b_r \in (0,1)}. Next, define \eqn{z_r} as the \eqn{Z}-score for \eqn{b_r}
\deqn{z_r = \Phi^{(-1)}(b_r)\mbox{,}}
where \eqn{\Phi^{(-1)}(\cdots)} is the inverse of the standard normal distribution.

Compute the covariance matrix \eqn{COV} of \eqn{M} and \eqn{S} from \code{\link{VMS}} as in \code{COV = VMS(n, r, qmin=br)}, and from which define
\deqn{\lambda = COV_{1,2} / COV_{2,2}\mbox{,}}
which is a covariance divided by a variance, and then define
\deqn{\eta_p = \lambda + \eta\mbox{.}}

Compute the expected values of \eqn{M} and \eqn{S} from \code{\link{EMS}} as in \eqn{EMp = } \code{EMp = EMS(n, r, qmin=br)}, and from which define
\deqn{\mu_{Mp} = EMp_1 - \lambda\times EMp_2\mbox{,}}
\deqn{\sigma_{Mp} = \sqrt{COV_{1,1} - COV_{1,2}^2/COV_{2,2}}\mbox{.}}

Compute the conditional moments from \code{\link{CondMomsChi2}} as in \eqn{momS2 =} \code{CondMomsChi2(n,r,zr)}, and from which define
\deqn{df = 2 momS2_1^2 / momS2_2\mbox{,}}
\deqn{\alpha = momS2_2 / momS2_1\mbox{,}}

}
\usage{
peta(pzr, n, r, eta)
}
\arguments{
  \item{pzr}{The probability level of a Beta distribution having shape1 \eqn{\alpha = r} and shape2 \eqn{\beta = n+1-r};}
  \item{n}{The number of observations;}
  \item{r}{The number of truncated observations; and}
  \item{eta}{The Grubbs--Beck statistic (\eqn{GB_r}, see \code{\link{MGBT}}).}
}
\value{
  The probability of the \code{eta} value.
}
\details{
Currently (2019), context is lost on the preformatted note of code note below. It seems possible that the intent by WHA was to leave a trail for future revisitation of the Beta distribution and its access, which exists in native \R code.
\preformatted{
      zr       <- qnorm(qbeta(the.pzr, shape1=r, shape2=n+1-r))
      CV       <- VMS(n, r, qmin=pnorm(zr))
}
}
\note{
Testing a very large streamgage dataset in Texas with GRH, shows at least one failure of the following computation was encountered for a short record streamgage numbered 08102900.
\preformatted{
  # USGS 08102900 (data sorted, 1967--1974)
  #https://nwis.waterdata.usgs.gov/nwis/peak?site_no=08102900&format=hn2
  Peaks <- c(40, 45, 53, 55, 88) # in cubic feet per second (cfs)
  MGBT(Peaks)
  # Here is the line in peta(): SigmaMp <- sqrt(CV[1,1] - CV[1,2]^2/CV[2,2])
  # *** In sqrt(CV[1, 1] - CV[1, 2]^2/CV[2, 2]) : NaNs produced
}
In implementation, a \code{suppressWarnings()} is wrapped on the \code{SigmaMp}. If the authors make no action in response to \code{NaN}, then the low-outlier threshold is 53 cubic feet per second (cfs) with a p-value for 40 cfs as 0.81 and 45 cfs as 0.0. This does not seem logical. The \code{is.finite} catch in the next line (see sources) is provisional under a \enc{naïve}{naive} assumption that the negative in the square root has barely undershot. The function is immediately exited with the returned p-value set to unity. Testing indicates that this is a favorable innate trap here within the \pkg{MGBT} package and will avoid higher up error trapping in larger application development.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, \code{P3_089(R).txt}---Named \code{peta}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.
}
\author{ W.H. Asquith consulting T.A. Cohn source}
\seealso{\code{\link{EMS}}, \code{\link{VMS}}, \code{\link{CondMomsChi2}}, \code{\link{gtmoms}}
}
\examples{
peta(0.4, 58, 2, -2.3006)
#[1] 0.298834
}
\keyword{Grubbs--Beck statistic (probability level)}
\keyword{Grubbs--Beck statistic}

\encoding{utf8}
\name{BLlo}
\alias{BLlo}
\title{Barnett and Lewis Test Adjusted for Low Outliers }
\description{
The Barnett and Lewis (1995, p. 224; \eqn{T_{\mathrm{N}3}}) so-labeled \dQuote{N3 method} with TAC adjustment to look for low outliers. The essence of the method, given the order statistics \eqn{x_{[1:n]} \le x_{[2:n]} \le \cdots \le x_{[(n-1):n]} \le x_{[n:n]}}, is the statistic
\deqn{BL_r = T_{\mathrm{N}3} =
\frac{ \sum_{i=1}^r x_{[i:n]} - r \times \mathrm{mean}\{x_{[1:n]}\} }
     {\sqrt{\mathrm{var}\{x_{[1:n]}\}}}\mbox{,}}
for the mean and variance of the observations.  Barnett and Lewis (1995, p. 218) brand this statistic as a test of the \dQuote{\eqn{k \ge 2} upper outliers} but for the \pkg{MGBT} package \dQuote{lower} applies in TAC reformulation. Barnett and Lewis (1995, p. 218) show an example of a modification for two low outliers as \eqn{(2\overline{x} - x_{[2:n]} - x_{[1:n]})/s} for the mean \eqn{\mu} and standard deviation \eqn{s}. TAC reformulation thus differs by a sign. The \eqn{BL_r} is a sum of internally studentized deviations from the mean:
\deqn{SP(t) \le {n \choose k} P\biggl(\bm{t}(n-2) > \biggr[\frac{n(n-2)t^2}{r(n-r)(n-1)-nt^2}\biggl]^{1/2}\biggr)\mbox{,}}
where \eqn{\bm{t}(df)} is the t-distribution for \eqn{df} degrees of freedom, and this is an inequality when
\deqn{t \ge \sqrt{r^2(n-1)(n-r-1)/(nr+n)}\mbox{,}}
where \eqn{SP(t)} is the probability that \eqn{T_{\mathrm{N}3} > t} when the inequality holds. For reference, Barnett and Lewis (1995, p. 491) example tables of critical values for \eqn{n=10} for \eqn{k \in 2,3,4} at 5-percent significant level are \eqn{3.18}, \eqn{3.82}, and \eqn{4.17}, respectively. One of these is evaluated in the \bold{Examples}.
}
\usage{
BLlo(x, r, n=length(x))
}
\arguments{
  \item{x}{The data values and note that base-10 logarithms of these are not computed internally;}
  \item{r}{The number of truncated observations; and}
  \item{n}{The number of observations.}
}
\value{
  The value for \eqn{BL_r}.
}
\note{
Regarding \code{n=length(x)}, it is not clear that TAC intended \code{n} to be not equal to the sample size. TAC chose to not determine the length of \code{x} internally to the function but to have it available as an argument. Also \code{\link{MGBTcohn2011}} and \code{\link{RSlo}} were designed similarly.
}
\source{
\code{LowOutliers_jfe(R).txt} and \code{LowOutliers_wha(R).txt}---Named \code{BL_N3}
}
\references{
Barnett, Vic, and Lewis, Toby, 1995, Outliers in statistical data: Chichester, John Wiley and Sons, ISBN~0--471--93094--6.

Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.
}
\author{ W.H. Asquith consulting T.A. Cohn sources }
\seealso{\code{\link{MGBTcohn2011}}, \code{\link{RSlo}}
}
\examples{
# See Examples under RSlo()

\donttest{ # WHA experiments with BL_r()
n <- 10; r <- 3; nsim <- 10000; alpha <- 0.05; Tcrit <- 3.82
BLs <- Ho <- RHS <- SPt <- rep(NA, nsim)
EQ <- sqrt(r^2*(n-1)*(n-r-1)/(n*r+n))
for(i in 1:nsim) { # some simulation results shown below
   BLs[i] <- abs(BLlo(rnorm(n), r)) # abs() correcting TAC sign convention
   t  <- sqrt( (n*(n-2)*BLs[i]^2) / (r*(n-r)*(n-1)-n*BLs[i]^2) )
   RHS[i] <- choose(n,r)*pt(t, n-2, lower.tail=FALSE)
   ifelse(t >= EQ, SPt[i] <- RHS[i], SPt[i] <- 1) # set SP(t) to unity?
   Ho[i]  <- BLs[i] > Tcrit
}
results <- c(quantile(BLs, prob=1-alpha), sum(Ho /nsim), sum(SPt < alpha)/nsim)
names(results) <- c("Critical_value", "Ho_rejected", "Coverage_SP(t)")
print(results) # minor differences are because of random number seeding
# Critical_value    Ho_rejected Coverage_SP(t)
#      3.817236       0.048200       0.050100 }
}
\keyword{low outlier (definition)}

\encoding{utf8}
\name{ASlo}
\alias{ASlo}
\title{Regression of a Heuristic Method for Identification of Low Outliers in Texas Annual Peak Streamflow }
\description{
Asquith and others (1995) developed a regression equation based on the first three moments of non-low-outlier truncated annual peak streamflow data in an effort to semi-objectively compute low-outlier thresholds for log-Pearson type III (Bulletin 17B consistent; IACWD, 1982) analyses (Asquith and Slade, 1997). A career hydrologist in for USGS in Texas, Raymond M. Slade, Jr., was particularly emphatic that aggressive low-outlier identification is needed for Texas hydrology as protection from mixed population effects.

WHA and RMS heuristically selected low-outlier thresholds for 262 streamgages in Texas with at least 20 years from unregulated and unurbanized watersheds. These thresholds were then regressed, along with help from Linda Judd, against the product moments of the logarithms (base-10) of the whole of the sample data (zeros not included). The regression equation is
\deqn{\log_{10}[AS_{\mathrm{Texas}}(\mu, \sigma, \gamma)]=1.09\mu-0.584\sigma+0.14\gamma - 0.799\mbox{,}}
where \eqn{AS_{\mathrm{Texas}}} is the low-outlier threshold, \eqn{\mu} is the mean, \eqn{\sigma} is the standard deviation, and \eqn{\gamma} is skew. The R-squared is 0.75, and those authors unfortunately do not appear to list a residual standard error. The suggested limitations are \eqn{1.9 < \mu < 4.842}, \eqn{0.125 < \sigma < 1.814}, and \eqn{-2.714 < \gamma < 0.698}.

The \eqn{AS_{\mathrm{Texas}}} equation was repeated in a footnote in Asquith and Roussel (2009, p. 19) because of difficulty in others acquiring copies of Asquith and others (1995). (File \code{AsquithLOT(1995).pdf} with this package is a copy.) Low-outlier thresholds using this regression were applied before the development of a generalized skew map in Texas (Judd and others, 1996) in turn used by Asquith and Slade (1997). A comparison of \eqn{AS_{\mathrm{Texas}}} to the results of MGBT is shown in the \bold{Examples}.

\bold{The \code{ASlo} equation is no longer intended for any practical application with the advent of the MGBT approach.} It is provided here for historical context only and shows a heuristic line of thought independent from the mathematical rigor provided by TAC and others leading to MGBT. The \eqn{AS_{\mathrm{Texas}}} incidentally was an extensive topic of long conversation between WHA and TAC at the National Surface Water Conference and Hydroacoustics Workshop (USGS Office of Surface Water), March 28--April 1, 2011, Tampa, Florida. The conversation was focused on the critical need for aggressive low-outlier identification in arid to semi-arid regions such as Texas. TAC was showcasing MGBT on a poster.
}
\usage{
ASlo(mu, sigma, gamma)
}
\arguments{
  \item{mu}{The arithmetic mean of the logarithms of non-low-outlier truncated annual peak streamflow data (zeros removed);}
  \item{sigma}{The standard deviation of the logarithms of non-low-outlier truncated annual peak streamflow data (zeros removed); and}
  \item{gamma}{The skewness (product moment) of the logarithms of non-low-outlier truncated annual peak streamflow data (zeros removed).}
}
\value{
  The value for the regression equation \eqn{AS_{\mathrm{Texas}}(\mu, \sigma, \gamma)} after re-transformation.
}
\references{
Asquith, W.H., 2019, lmomco---L-moments, trimmed L-moments, L-comoments, censored \cr L-moments, and many distributions: R package version 2.3.2 (September 20, 2018), accessed March 30, 2019, at \url{https://cran.r-project.org/package=lmomco}.

Asquith, W.H., Slade, R.M., and Judd, Linda, 1995, Analysis of low-outlier thresholds for log-Pearson type III peak-streamflow frequency analysis in Texas, in Texas Water *95, American Society of Civil Engineers First International Conference, San Antonio, Texas, 1995, Proceedings: San Antonio, Texas, American Society of Civil Engineers, pp. 379--384.

Asquith, W.H., and Slade, R.M., 1997, Regional equations for estimation of peak-streamflow frequency for natural basins in Texas: U.S. Geological Survey Water-Resources Investigations Report 96--4307, 68 p., \url{https://pubs.usgs.gov/wri/wri964307/}

Asquith, W.H., and Roussel, M.C., 2009, Regression equations for estimation of annual peak-streamflow frequency for undeveloped watersheds in Texas using an L-moment-based, PRESS-minimized, residual-adjusted approach: U.S. Geological Survey Scientific Investigations Report 2009--5087, 48 p., \url{https://pubs.usgs.gov/sir/2009/5087/}.

Interagency Advisory Committee on Water Data (IACWD), 1982, Guidelines for determining flood flow frequency: Bulletin 17B of the Hydrology Subcommittee, Office of Water Data Coordination, U.S. Geological Survey, Reston, Va., 183 p.

Judd, Linda, Asquith, W.H., and Slade, R.M., 1996, Techniques to estimate generalized skew coefficients of annual peak streamflow for natural basins in Texas: U.S. Geological Survey Water Resources Investigations Report 96--4117, 28 p., \url{https://pubs.usgs.gov/wri/wri97-4117/}
}
\source{
Original \R by WHA for this package.
}
\author{ W.H. Asquith}
\examples{
# USGS 08066300 (1966--2016) # cubic feet per second (cfs)
#https://nwis.waterdata.usgs.gov/nwis/peak?site_no=08066300&format=hn2
Peaks <- c(3530, 284, 1810, 9660,  489,  292, 1000,  2640, 2910, 1900,  1120, 1020,
   632, 7160, 1750,  2730,  1630, 8210, 4270, 1730, 13200, 2550,  915, 11000, 2370,
  2230, 4650, 2750,  1860, 13700, 2290, 3390, 5160, 13200,  410, 1890,  4120, 3930,
  4290, 1890, 1480, 10300,  1190, 2320, 2480, 55.0,  7480,  351,  738,  2430, 6700)
#ASlo(3.3472,     0.4865,    -0.752)     # moments from USGS-PeakFQ (v7.1)
 ASlo(3.34715594, 0.4865250, -0.7517086) # values from lmomco::pmoms(log10(Peaks))
# computes 288 cubic feet per second, and now compare this to MGBT()
# MGBT(Peaks)$LOThres # computes 284 cubic feet per second
# ---***--------------***--- Remarkable similarity! ---***--------------***---
# ---***--------------***--- Not true in all cases. ---***--------------***---
}
\keyword{Texas low outliers}
\keyword{low outlier (definition)}
\keyword{low outlier (Texas)}
\keyword{Bulletin 17B}

\encoding{utf8}
\name{RSlo}
\alias{RSlo}
\title{Rosner RST Test Adjusted for Low Outliers}
\description{
The Rosner (1975) method or the essence of the method, given the order statistics \eqn{x_{[1:n]} \le x_{[2:n]} \le \cdots \le x_{[(n-1):n]} \le x_{[n:n]}}, is the statistic:
\deqn{RS_r =
\frac{ x_{[r:n]} - \mathrm{mean}\{x_{[(r+1)\rightarrow(n-r):n]}\} }
     {\sqrt{\mathrm{var}\{x_{[(r+1)\rightarrow(n-r):n]}\}}}\mbox{,}
}
}
\usage{
RSlo(x, r, n=length(x))
}
\arguments{
  \item{x}{The data values and note that base-10 logarithms of these are not computed internally;}
  \item{r}{The number of truncated observations; and}
  \item{n}{The number of observations.}
}
\value{
  The value for \eqn{RS_r}.
}
\note{
Regarding \code{n=length(x)}, it is not clear that TAC intended \code{n} to be not equal to the sample size. TAC chose to not determine the length of \code{x} internally to the function but to have it available as an argument. Also \code{\link{MGBTcohn2011}} and \code{\link{BLlo}} were similarly designed.
}
\source{
\code{LowOutliers_jfe(R).txt} and \code{LowOutliers_wha(R).txt}---Named \code{RST}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.

Rosner, Bernard, 1975, On the detection of many outliers: Technometrics, v. 17, no. 2, pp. 221--227.
}
\author{ W.H. Asquith consulting T.A. Cohn sources }
\seealso{\code{\link{MGBTcohn2011}}, \code{\link{BLlo}}
}
\examples{
\donttest{
# Long CPU time, arguments slightly modified to run faster and TAC had.
# TAC has maxr=n/2 (the lower tail) but WHA has changed to maxr=3 for
# speed to get the function usable during automated tests.
testMGBvsN3 <- function(n=100, maxr=3, nrep=10000) { # TAC named function
   for(r in 1:maxr) {
      set.seed(123457)
      res1 <- replicate(nrep, { x <- sort(rnorm(n))
                 c(MGBTcohn2011(x,r),BLlo(x,r),RSlo(x,r)) })
      r1 <- rank(res1[1,]); r2 <- rank(res1[2,]); r3 <- rank(res1[3,])
      v <- quantile(r1,.1); h <- quantile(r2,.1)
      plot(r1,r2)
      abline(v=v, col=2); abline(h=h, col=2)
      message(' BLlo ',r, " ", cor(r1,r2), " ", mean((r1 <= v) & (r2 <= h)))
      v <- quantile(r1,.1); h <- quantile(r2,.1)
      plot(r1,r3)
      abline(v=v, col=2); abline(h=h, col=2)
      mtext(paste0("Order statistic iteration =",r," of ",maxr))
      message('RSTlo ',r, " ", cor(r1,r3), " ", mean((r1 <= v) & (r3 <= h)))
   }
}
testMGBvsN3() #}
}
\keyword{low outlier (definition)}

\encoding{utf8}
\name{V}
\alias{V}
\title{Covariance matrix of M and S-squared}
\description{
Compute the covariance matrix of \eqn{M} and \eqn{S^2} (S-squared) given \eqn{q_\mathrm{min}}. Define the vector of four moment expectations
\deqn{E_{i\in 1,2,3,4} = \Psi\bigl(\Phi^{(-1)}(q_\mathrm{min}), i\bigr)\mbox{,}}
where \eqn{\Psi(a,b)} is the \code{\link{gtmoms}} function and \eqn{\Phi^{(-1)}} is the inverse of the standard normal distribution. Using these \eqn{E}, define a vector \eqn{C_{i\in 1,2,3,4}} as a system of nonlinear combinations:
\deqn{C_1 = E_1\mbox{,}}
\deqn{C_2 = E_2 -     E_1^2\mbox{,}}
\deqn{C_3 = E_3 - 3E_2E_1 + 2E_1^3\mbox{, and}}
\deqn{C_4 = E_4 - 4E_3E_1 + 6E_2E_1^2 - 3E_1^4\mbox{.}}
Given \eqn{k = n - r} from the arguments of this function, compute the symmetrical covariance matrix \eqn{COV} with variance of \eqn{M} as
\deqn{COV_{1,1} = C_2/k\mbox{,}}
the covariance between \eqn{M} and \eqn{S^2} as
\deqn{COV_{1,2} = COV_{2,1} = \frac{C_3}{\sqrt{k(k-1)}}\mbox{, and}}
the variance of \eqn{S^2} as
\deqn{COV_{2,2} = \frac{C_4 -  C_2^2}{k} + \frac{2C_2^2}{k(k-1)}\mbox{.}}
}
\usage{
V(n, r, qmin)
}
\arguments{
  \item{n}{The number of observations;}
  \item{r}{The number of truncated observations; and}
  \item{qmin}{A nonexceedance probability threshold for \eqn{X > q_\mathrm{min}}.}
}
\value{
  A 2-by-2 covariance \code{matrix}.
}
\note{
Because the \code{\link{gtmoms}} function is naturally vectorized and TAC sources provide no protection if \code{qmin} is a vector (see \bold{Note} under \code{\link{EMS}}). For the implementation here, only the first value in \code{qmin} is used and a warning issued if it is a vector.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, \code{P3_089(R).txt}---Named \code{V}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.
}
\author{W.H. Asquith consulting T.A. Cohn sources}
\seealso{\code{\link{EMS}}, \code{\link{VMS}}, \code{\link{gtmoms}}
}
\examples{
V(58,2,.5)
#            [,1]        [,2]
#[1,] 0.006488933 0.003928333
#[2,] 0.003928333 0.006851120
}
\keyword{moments}
\keyword{moments (conditional)}
\keyword{utility functions}

\encoding{utf8}
\name{splitPeakCodes}
\alias{splitPeakCodes}
\title{Split the Peak Discharge Qualifications Codes into Separate Columns }
\description{
Split the U.S. Geological Survey (USGS) peak discharge qualifications codes (Asquith and others, 2017; U.S. Geological Survey, 2021) in the \code{peak_cd} column of a peak-streamflow data retrieval from the USGS National Water Information System (NWIS) (U.S. Geological Survey, 2019) in a \code{data.frame} into separate columns of the input \code{data.frame}. The NWIS system stores all the codes within a single database field. It can be useful for graphical (\code{\link{plotPeaks}}) or other statistical study to have single logical variable for each of the codes, and such is the purpose of this function. Note because of the \code{appearsSystematic} field is based computations involving the \code{water_yr} (water year), this function needs the \code{\link{makeWaterYear}} to have been run first; however, the function will autodetect and call that function internally if needed and those effects are provided on the returned \code{data.frame}. (See also \code{\link{plotPeaks}}; see also the \code{inst/legend/} subdirectory of this package for a script to produce a legend as well as \code{inst/legend/legend_camera.pdf}, which has been dressed up in a vector graphics editing program.)
}
\usage{
splitPeakCodes(x, all_peaks_na_okay=FALSE)
}
\arguments{
  \item{x}{A \code{data.frame} having a mandatory column titled \code{peak_cd} with discharge qualification codes. Except for a check on only one station being present in \code{site_no} column, no other information in \code{x} is consulted or otherwise used; and}
  \item{all_peaks_na_okay}{A logical controlling whether a test on all the peak values (\code{peak_va}) being \code{NA} is made and if all the peak values are missing, then \code{NULL} is returned. Because much of this package is built around working with real peak discharges, the default is to be rejectionary to gage-height only streamgages. However, the \code{\link{plotPeaks}} function does have logic to work out ways to make plots of gage-height only streamgages.}
}
\value{
  The \code{x} as originally inputted is returned with the addition of these columns:
  \item{appearsSystematic}{The \code{appearsSystematic} column produced by the \code{splitPeakCodes} function is intended to provide a type of \emph{canonical} flag on which to subset the record, which can be important for many statistical procedures;}
  \item{anyCodes}{Are \emph{any} of the codes that follow present for a given record (row, water year) in the input data;}
  \item{isCode1}{Is a discharge qualification code of 1 present for a given record---Streamflow is a maximum daily average;}
  \item{isCode2}{Is a discharge qualification code of 2 present for a given record---Streamflow is an estimate;}
  \item{isCode3}{Is a discharge qualification code of 3 present for a given record---Streamflow affected by dam failure;}
  \item{isCode4}{Is a discharge qualification code of 4 present for a given record---Streamflow is less than indicated value, which is the minimum recordable value at this site;}
  \item{isCode5}{Is a discharge qualification code of 5 present for a given record---Streamflow affected to an unknown degree by regulation or diversion;}
  \item{isCode6}{Is a discharge qualification code of 6 present for a given record---Streamflow is affected by regulation or diversion;}
  \item{isCode7}{Is a discharge qualification code of 7 present for a given record---Streamflow is a historical peak;}
  \item{isCode8}{Is a discharge qualification code of 8 present for a given record---Streamflow is actually greater than the indicated value;}
  \item{isCode9}{Is a discharge qualification code of 9 present---Streamflow is affected by snow melt, hurricane, ice-jam, or debris-dam breakup;}
   \item{isCodeA}{Is a discharge qualification code of A present for a given record---Year of occurrence is unknown or not exact;}
   \item{isCodeB}{Is a discharge qualification code of B present for a given record---Month or day of occurrence is unknown or not exact;}
   \item{isCodeC}{Is a discharge qualification code of C present for a given record---All or part of the record is affected by urbanization, mining, agricultural changes, channelization, or other anthropogenic activity;}
   \item{isCodeD}{Is a discharge qualification code of D present for a given record---Base streamflow changed during this year;}
   \item{isCodeE}{Is a discharge qualification code of E present for a given record---Only annual peak streamflow available for this year;}
   \item{isCodeF}{Peak supplied by another agency;}
   \item{isCodeO}{Is a discharge qualification code of E present for a given record---Opportunistic value not from systematic data collection. By extension, the presence of this code will trigger the \code{appearsSystematic} to false even if the peak itself otherwise appears part of systematic record from the gap analysis, and}
   \item{isCodeR}{Revised.}
}
\note{
Concerning \code{appearsSystematic}: All records but missing discharges are assumed as systematic records unless the peak streamflows are missing or peaks have a code 7 but there are existing years on either side of individual peaks coded as 7. The logic also handles a so-called \dQuote{roll-on} and \dQuote{roll-off} of the data by only testing the leading or trailing year---except this becomes a problem if multiple stations are involved, so the code will return early with a warning. Importantly, it is possible that some code 7s can be flagged as systematic and these are not necessarily in error. Testing indicates that some USGS Water Science Centers (maintainers of databases) have historically slightly different tendencies in application of the code 7. The USGS NWIS database does not actually contain a field indicating that a peak was collected as part of systematic operation and hence that peak is part of an assumed random sample. Peaks with gage height only are flagged as nonsystematic by fiat---this might not be the best solution over all, but because virtually all statistics use the discharge column this seems okay (feature is subject to future changes).

Concerning \code{anyCodes}, only the regular expression \code{A|B|C|D|O|1|2|3|4|5|6|7|8|9} will trigger setting \code{anyCodes} to \code{TRUE}. For example, it is felt that codes \code{E}, \code{F}, and \code{R} are basically meaningless for statistic study and do not represent the \dQuote{special circumstances} of information as reflected in the aforementioned list of others codes. The codes \code{Bd} and \code{Bm} are uninformative too and the column appending by \code{\link{makeWaterYear}} accommodates the necessary structure for further study of date precision if needed by a developer.

A comprehensive listing of peak codes (\code{peak_cd}) and peak gage heights (\code{gage_cd}) follows (U.S. Geological Survey, 2021):
\preformatted{
peak_cd
  1	----> Discharge is a Maximum Daily Average
  2	----> Discharge is an Estimate
  3	----> Discharge affected by Dam Failure
  4	----> Discharge less than indicated value which is Minimum Recordable
            Discharge at this site
  5	----> Discharge affected to unknown degree by Regulation or Diversion
  6	----> Discharge affected by Regulation or Diversion
  7	----> Discharge is an Historic Peak
  8	----> Discharge actually greater than indicated value
  9	----> Discharge due to Snowmelt, Hurricane, Ice-Jam or Debris Dam breakup
  A	----> Year of occurrence is unknown or not exact
 Bd	----> Day of occurrence is unknown or not exact
 Bm	----> Month of occurrence is unknown or not exact
  C	----> All or part of the record affected by Urbanization, Mining,
            Agricultural changes, Channelization, or other
  D	----> Base Discharge changed during this year
  E	----> Only Annual Maximum Peak available for this year
  F	----> Peak supplied by another agency
  O	----> Opportunistic value not from systematic data collection
  R	----> Revised

gage_cd
  1	----> Gage height affected by backwater
  2	----> Gage height not the maximum for the year
  3	----> Gage height was at a different site and/or datum
  4	----> Gage height below minimum recordable elevation
  5	----> Gage height is an estimate
  6	----> Gage height datum changed during this year
  7	----> Debris, mud, or hyper-concentrated flow
  8	----> Gage height tidally affected
 Bd	----> Day of occurrence is unknown or not exact
 Bm	----> Month of occurrence is unknown or not exact
  F	----> Peak supplied by another agency
  R	----> Revised
}

The authors of the \pkg{MGBT} package want to make a remark that the code 4 in many parts of the United States might not reflect the complete idea of \dQuote{which is minimum recordable discharge at this site.} More generally, the code 4 should be thought of a censoring flag for a \dQuote{less then discharge} (Asquith and others, 2018). There are streamgages that in theory could be recording zero to infinite discharge, but then when information is compiled for the year, the annual peak might be flagged as a less than some threshold and have no bearing on the idea of a minimum recordable for the streamgage itself. Such a situation could stem from multiple gage height recording apparatus measuring culvert-like flow circumstances (Asquith and others, 2018).
}
\references{
Asquith, W.H., Harwell, G.R., and Winters, K.E., 2018, Annual and approximately quarterly series peak streamflow derived from interpretations of indirect measurements for a crest-stage gage network in Texas through water year 2015: U.S. Geological Survey Scientific Investigations Report 2018--5107, 24 p., \doi{10.3133/sir20185107}.

Asquith, W.H., Kiang, J.E., and Cohn, T.A., 2017, Application of at-site peak-streamflow frequency analyses for very low annual exceedance probabilities: U.S. Geological Survey Scientific Investigation Report 2017--5038, 93 p., \doi{10.3133/sir20175038}.

U.S. Geological Survey, 2019, USGS water data for the Nation: U.S. Geological Survey National Water Information System database, accessed October 11, 2019, at \doi{10.5066/F7P55KJN}.

U.S. Geological Survey, 2021, Peak Streamflow and Stage Special Conditions (PEAK.peak_cd and PEAK.gage_ht_cd).
}
\author{ W.H. Asquith}
\seealso{\code{\link{makeWaterYear}}, \code{\link{plotPeaks}}
}
\examples{
\dontrun{
  # The dataRetrieval package is not required by MGBT algorithms.
  PK <- dataRetrieval::readNWISpeak("08167000", convertType=FALSE)
  PK <- splitPeakCodes(PK)
  names(PK) # See that the columns are there.}
}
\keyword{NWIS operator}

\encoding{utf8}
\name{makeWaterYear}
\alias{makeWaterYear}
\title{Make Water Year Column}
\description{
Make water year, year, month, and day columns from the date stamp of a U.S. Geological Survey peak-streamflow data retrieval from the National Water Information System (NWIS) (U.S. Geological Survey, 2019) in an \R \code{data.frame} into separate columns of the input \code{data.frame}. Note that NWIS (through \pkg{dataRetrieval} package) will insert padded zeros for missing month and years with in the peak-streamflow data when the variable type conversion is false (see \bold{Examples}). This function will changes those zeros in the separated columns to missing.
}
\usage{
makeWaterYear(x, datestr="peak_dt")
}
\arguments{
  \item{x}{A \code{data.frame} having a default column titled \code{peak_dt} that is in a character format. No other information in \code{x} is consulted or otherwise used. A feature of of the function is that \code{x} could also be simply a vector of characters in the basic \code{"YYYY-MM-DD"}, \code{"YYYY-MM"}, or \code{"YYYY"} structure for the conversions; and}
  \item{datestr}{The column name containing the date on which to parse for creation of new columns. The default is aligned with USGS peak-streamflow data.}
}
\value{
  The \code{x} is returned containing only these columns (if \code{x} is \code{is.character}) or with the addition of these columns:
  \item{year_va}{The calendar year extracted from \code{peak_dt};}
  \item{month_va}{The optional month extracted from \code{peak_dt};}
  \item{day_va}{The optional day extracted from \code{peak_dt}; and}
  \item{water_yr}{The water year, which is not equal to \code{year_va} if \code{month_va} is greater than or equal to 10 (October).}
}
\author{W.H. Asquith}
\seealso{\code{\link{splitPeakCodes}}, \code{\link{plotPeaks}}
}
\references{
U.S. Geological Survey, 2019, USGS water data for the Nation: U.S. Geological Survey National Water Information System database, accessed October 11, 2019, at \doi{10.5066/F7P55KJN}.
}
\examples{
print(makeWaterYear(c("1888-07-00", "1889", "1889-11", "1891-03-04")))
#        date year_va month_va day_va water_yr
#1 1888-07-00    1888        7     NA     1888
#2       1889    1889       NA     NA     1889
#3    1889-11    1889       11     NA     1890
#4 1891-03-04    1891        3      4     1891

\dontrun{
  # The dataRetrieval package is not required by MGBT algorithms.
  PK <- dataRetrieval::readNWISpeak("08167000", convertType=FALSE)
  PK <- makeWaterYear(PK) # Note: The convertType=FALSE is critical.
  names(PK) # See that the columns are there and see that there are padded
  # zeros for missing days for "1869-07-00" and "1936-09-00" and the
  # day_va column will have NAs in lieu of 0s in these examples}
}
\keyword{NWIS operator}
\keyword{Water Year}

\encoding{utf8}
\name{plotPeaks}
\alias{plotPeaks}
\title{Plot Peak Streamflows with Emphasis on Peak Discharge Qualification Codes}
\description{
Plot U.S. Geological Survey peak streamflows in \code{peak_va} and discharge qualifications codes in the \code{peak_cd} columns of a peak-streamflow data retrieval from the National Water Information System (NWIS) (U.S. Geological Survey, 2019). This code makes use of the \code{add.log.axis} function from the \pkg{lmomco} package, and because this is the only instance of this dependency, the \pkg{lmomco} package is treated as a suggested package and not as a dependency for the \pkg{MGBT} package. The \code{add.log.axis} function also is used for the basis of the logarithmic plot within the \code{\link{plotFFQevol}} function.

This function accommodates the plotting of various nuances of the peak including less than (code 4), greater than (code 8), zero peaks (plotted by a green tick on the horizontal axis), and peaks that are missing but the gage height was available (plotted by a light-blue tick on the horizontal axis if the \code{showGHyrs} argument is set). So-called code 5, 6, and 7 peaks are plotted by the numeric code as the plotting symbol, and so-called code C peaks are plotted by the letter \dQuote{C.} The very unusual circumstances of codes 3 and O are plotted by the letters \dQuote{D} (dam failure) and \dQuote{O} (opportunistic), respectively. These codes are are summarized within \code{\link{splitPeakCodes}}. The greater symbology set is described in the directory \code{MGBT/inst/legend} of the package sources.

The logic herein also makes allowances for \dQuote{plotting} gage-height only streamgages but this requires that the \code{\link{splitPeakCodes}} function was called with the \code{all_peak_na_okay} set to \code{TRUE}. The gap analysis for gage-height only streamgages or streamgages for which the site changes from gage-height only to discharge and then back again or other permutations will likely result in the gap lines not being authoritative. The sub-bottom axis ticks for the gage-height only water years should always be plotting correctly.
}
\usage{
plotPeaks(x, codes=TRUE, lot=NULL, site="",
             xlab="", ylab="", xlim=NULL, ylim=NULL,
             xlim.inflate=TRUE, ylim.inflate=TRUE, aux.y=NULL,
             log.ticks=c(1, 2, 3, 5, 8), mtext.site=TRUE,
             show48=FALSE, showDubNA=FALSE, showGHyrs=TRUE,
             make_text_for_all_na_peaks=TRUE, ...)
}
\arguments{
  \item{x}{A \code{data.frame} having a mandatory column titled \code{peak_va} and \code{peak_cd}. It is advised to have run \code{\link{makeWaterYear}} and \code{\link{splitPeakCodes}} first, but if columns resulting from those two functions are are not detected, then the water year column and code split are made internally but not returned;}
  \item{codes}{A logical to trigger use of character plotting characters and not symbols;}
  \item{lot}{The low-outlier threshold, but if omitted, then \code{\link{MGBT}} is triggered internally. Use of \code{lot=0} is a mechanism to effectively bypass the plotting of a low-outlier threshold because a logarithmic vertical axis is used, and this setting would bypass a call to \code{\link{MGBT}};}
  \item{site}{An optional character string for a plot title, and in practice, this is expected to be a streamgage location;}
  \item{xlab}{An optional x-label of the plot;}
  \item{ylab}{An optional y-label of the plot;}
  \item{xlim}{An optional x-limit of the plot;}
  \item{ylim}{An optional y-limit of the plot;}
  \item{xlim.inflate}{A logical to trigger nudging the horizontal axis left/right to the previous/future decade;}
  \item{ylim.inflate}{A logical to trigger nudging the vertical axis down/up to the nearest sane increment of log10-cycles. This inflation also includes a \eqn{\pm}0.01 log10-cycle adjustment to the lower and upper values of the limit. This ensures (say) that a 50,000 cubic feet per second maximum, which is on a sane increment, is nudged up enough to make the upper limit 60,000 instead. The point itself plots at 50,000 cfs;}
  \item{aux.y}{An optional set of values to pack into the data just ahead of the the vertical axis limits computation;}
  \item{log.ticks}{The argument \code{logs} of the \code{add.log.axis} function;}
  \item{show48}{A logical, if \code{codes} is set, will draw a \dQuote{4} and \dQuote{8} for those respective codes instead of a reddish dot;}
  \item{mtext.site}{A logical, if set, will call \code{mtext(site, line=0.5))} as the function completes. This is handy for emplacing the contents of \code{site} at the top of the plot but by having a logical switch for this, the \code{site} can be used for internal purposes, such as warnings, specific to this site. Setting \code{mtext.site=FALSE} might be preferable in data mining exercises with very many streamgages to help isolate those streamgages with data or lack thereof that cause \pkg{MGBT} problems;}
  \item{showDubNA}{A logical, if set, will draw a \eqn{\pm}half-year polygon for the water years having both \code{NA} discharge (\code{peak_va}) and \code{NA} gage height (\code{gage_ht}). To some analysts, it might be useful to study the degree of these double empty entries in contrast to just the records of such years not even being present in NWIS. Double empty entries could represent incomplete data handling on part of the USGS or inadvertently missing zero discharges for \code{peak_va};}
  \item{showGHyrs}{A logical to trigger the light-blue special ticks for the water years having only gage height. The horizontal limits will be inflated accordingly if gage heights are outside the general discharge record;}
  \item{make_text_for_all_na_peaks}{A logical to trigger a message to be centered in the middle of the plot alert the reader that there are no peaks to for the streamgage (\dQuote{There are no non-missing peaks for this streamgage.}); and }
  \item{...}{Additional arguments to pass to \code{plot}.}
}
\value{
  No values are returned; this function is used for its graphical side effects.
}
\note{
The file \code{inst/legend/legend_camera.pdf} has been configured also to be embedded into this user manual as shown below:

\if{html}{\figure{plotPeakslegend.jpg}}
\if{latex}{\figure{plotPeakslegend.pdf}{options: width=5.5in, trim=1in 0.2in 0.5in 0.6in, clip}}
}
\author{W.H. Asquith}
\references{
U.S. Geological Survey, 2019, USGS water data for the Nation: U.S. Geological Survey National Water Information System database, accessed October 11, 2019, at \doi{10.5066/F7P55KJN}.
}
\seealso{\code{\link{makeWaterYear}}, \code{\link{splitPeakCodes}}, \code{\link{plotFFQevol}}
}
\examples{
\dontrun{
  # The dataRetrieval package is not required by MGBT algorithms.
  # Note that makeWaterYear() is not needed because splitPeakCodes() requires
  # the water_yr for gap analyses, and will call makeWaterYear() if it needs to.
  PK <- dataRetrieval::readNWISpeak("08167000", convertType=FALSE)
  PK <- splitPeakCodes(makeWaterYear(PK))
  plotPeaks(PK, codes=TRUE, showGHyrs=FALSE, site="08167000",
                xlab="Water year", ylab="Discharge, cfs") # }

\dontrun{
  # The dataRetrieval package is not required by MGBT algorithms.
  # An example with zero flows
  PK <- dataRetrieval::readNWISpeak("07148400", convertType=FALSE)
  PK <- splitPeakCodes(PK)
  plotPeaks(PK, codes=TRUE,  showDubNA=TRUE, site="07148400",
                xlab="Water year", ylab="Discharge, cfs") # }

\dontrun{
  # The dataRetrieval package is not required by MGBT algorithms.
  PK <- dataRetrieval::readNWISpeak("08329935", convertType=FALSE)
  PK <- splitPeakCodes(PK)
  plotPeaks(PK, codes=TRUE, showDubNA=TRUE, site="08329935",
                xlab="Water year", ylab="Discharge, cfs") # }

\dontrun{
  PK <- dataRetrieval::readNWISpeak("02146285", convertType=FALSE)
  PK <- splitPeakCodes(PK, all_peaks_na_okay=TRUE)
  plotPeaks(PK, site="02146285") # see likely message about all
  # peaks being missing assume that this streamgage forevermore
  # is a gage-height only site }

\dontrun{
  PK <- dataRetrieval::readNWISpeak("02317600", convertType=FALSE)
  PK <- splitPeakCodes(PK, all_peaks_na_okay=TRUE)
  plotPeaks(PK, site="02317600") }
}
\keyword{graphics}

\encoding{utf8}
\name{RthOrderPValueOrthoT}
\alias{RthOrderPValueOrthoT}
\title{P-value for the Rth Order Statistic}
\description{
Compute the p-value for the \eqn{r}th order statistic

\deqn{\eta(r,n) = \frac{x_{[r:n]} - \mathrm{mean}\{x_{[(r+1)\rightarrow n:n]}\}}
                 {\sqrt{\mathrm{var}\{x_{[(r+1)\rightarrow n:n]}\}}}\mbox{.}}

This function is the cumulative distribution function of the Grubbs--Beck statistic (\code{eta} = \eqn{GB_r(p)}). In distribution notation, this is equivalent to saying \eqn{F(GB_r)} for nonexceedance probability \cr
\eqn{F \in (0,1)}. The inverse or quantile function \eqn{GB_r(F)} is \code{\link{CritK}}.
}
\usage{
RthOrderPValueOrthoT(n, r, eta, n.sim=10000, silent=TRUE)
}
\arguments{
  \item{n}{The number of observations;}
  \item{r}{The number of truncated observations; and}
  \item{eta}{The pseudo-studentized magnitude of \eqn{r}th smallest observation;}
  \item{n.sim}{The sample size to attempt a Monte Carlo integration in case the numerical integration via \code{integrate()} encounters a divergent integral; and}
  \item{silent}{A logical controlling the silence of \code{try}.}
}
\value{
  The value a two-column \R \code{matrix}.
}
\note{
The extension to Monte Carlo integration in event of failure of the numerical integration an extension is by WHA. The \bold{Note} for \code{\link{MGBT}} provides extensive details in the context of a practical application.

Note that in conjunction with \code{RthOrderPValueOrthoT}, TAC provided an enhanced numerical integration interface (\code{integrateV()}) to \code{integrate()} built-in to \R. In fact, all that TAC did was wrap a vectorization scheme using \code{sapply()} on top of \code{\link{peta}}. The issue is that \code{\link{peta}} was not designed to be vectorized. WHA has simply inserted the \code{sapply} \R idiom inside \code{\link{peta}} and hence vectorizing it and removed the need in the \pkg{MGBT} package for the \code{integrateV()} function in the TAC sources.

TAC named this function with the \code{Kth} order. In code, however, TAC uses the variable \code{r}. WHA has migrated all references to \code{Kth} to \code{Rth} for systematic consistency. Hence, this function has been renamed to \code{RthOrderPValueOrthoT}.

TAC also provides a \code{KthOrderPValueOrthoTb} function and notes that it employs simple Gaussian quadrature to compute the integral much more quickly. However, it is slightly less accurate for tail probabilities. The Gaussian quadrature is from a function \code{gauss.quad.prob()}, which seems to not be found in the TAC sources given to WHA.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, \code{P3_089(R).txt}---\cr
Named \code{KthOrderPValueOrthoT} + \code{KthOrderPValueOrthoTb}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.
}
\author{W.H. Asquith consulting T.A. Cohn sources}
\seealso{\code{\link{MGBT}}, \code{\link{CritK}}
}
\examples{
# Running next line without the $value will show:
#0.001000002 with absolute error < 1.7e-05 # This is output from the integrate()
# function, which means that the numerical integration worked.
RthOrderPValueOrthoT(58, 2, -3.561143)$value

\donttest{
# Long CPU time
CritK(58, 2, RthOrderPValueOrthoT(58, 2, -3.561143)$value)
#[1] -3.561143  # Therefore CritK() is the inverse of this function.}

\donttest{
# Long CPU time
# Monte Carlo distribution of rth pseudo-studentized order statistic (TAC note)
testRthOrderPValueOrthoT <- function(nrep=1E4, r=2, n=100,
               test_quants = c(0.05,0.1,0.5,0.9,0.95),  ndigits=3, seed=1) {
   set.seed(seed)
   z <- replicate(nrep, { x <- sort(rnorm(n)); xr <- x[r]; x2 <- x[(r+1):n]
                         (xr - mean(x2))/sqrt(var(x2)) })
     res <- sapply(quantile(z, test_quants), function(q) {
                 c(q, RthOrderPValueOrthoT(n,r,q)$value) })
   round(res,ndigits)
}

nsim <- 1E4
for(n in 50) {   # original TAC sources had c(10,15,25,50,100,500)
   for(r in 5) { # original TAC sources had 1:min(10,floor(n/2))
      message("n=",n, " and r=",r)
      print(testRthOrderPValueOrthoT(nrep=nsim, n=n, r=r))
   }
}
# Output like this will be seen
# n=50 and r=5
#         5%    10%    50%    90%    95%
#[1,] -2.244 -2.127 -1.788 -1.523 -1.460
#[2,]  0.046  0.096  0.499  0.897  0.946
# that shows simulated percentages near the theoretical

# To get the MSE of the results (TAC note). See WHA note on a change below and
# it is suspected that TAC's "tests" might have been fluid in the sense that
# he would modify as needed and did not fully design as Examples for end users.
rr <- rep(0,10)
for(n in 50) {   # original TAC sources had c(10,15,25,50,100,500)
   for(r in 5) { # original TAC sources had 1:min(10,floor(n/2))
      message("n=",n, " and r=",r)
      for(i in 1:10) { # The [1,1] is WHA addition to get function to run.
         # extract the score for the 5% level
         rr[i] <- testRthOrderPValueOrthoT(nrep=nsim, n=n, r=r, seed=i)[1,1]
      }
      message("var (MSE):", sqrt(var(rr/100)))
   }
}
# Output like this will be seen
# n=50 and r=5
# var (MSE):6.915361322608e-05 }

\donttest{
#  Long CPU time
#  Monte Carlo computation of critical values for special cases (TAC note)
CritValuesMC <-
function(nrep=50, kvs=c(1,3,0.25,0.5), n=100, ndigits=3, seed=1,
         test_quants=c(0.01,0.10,0.50)) {
   set.seed(seed)
   k_values <- ifelse(kvs >= 1, kvs, ceiling(n*kvs))
   z  <- replicate(nrep, {
      x <- sort(rnorm(n))
      sapply(k_values, function(r) {
          xr <- x[r]; x2 <- x[(r+1):n]
         (xr-mean(x2)) / sqrt(var(x2)) })  })
   res <- round(apply(z, MARGIN=1, quantile, test_quants), ndigits)
   colnames(res) <- k_values; return(res)
}

# TAC example. Note that z acquires its square dimension from test_quants
# but Vr is used in the sapply(). WHA has reset Vr to
n=100; nrep=10000; test_quants=c(.05,.5,1); Vr=1:10 # This Vr by TAC
z <- CritValuesMC(n=n, nrep=nrep, test_quants=test_quants)
Vr <- 1:length(z[,1]) # WHA reset of Vr to use TAC code below. TAC Vr bug?
HH <- sapply(Vr, function(r) RthOrderPValueOrthoT(n, r, z[1,r])$value)
TT <- sapply(Vr, function(r) RthOrderPValueOrthoT(n, r, z[2,r])$value) #}
}
\keyword{hypothesis test}

\encoding{utf8}
\name{critK10}
\alias{critK10}
\title{Single Grubbs--Beck Critical Values for 10-percent Test as used in Bulletin 17B }
\description{
Return the critical values at the 10-percent (\eqn{\alpha_\mathrm{17B} = 0.10}) significance level for the single Grubbs--Beck test as in Bulletin 17B (IACWD, 1982).
}
\usage{
critK10(n)
}
\arguments{
  \item{n}{The number of observations.}
}
\value{
  The critical value for sample size \eqn{n} unless it is outside the range \eqn{10 \le n \le 149} for which the critical value is \code{NA}.
}
\note{
In the context of \code{critK10}, TAC defines a \code{.kngb()} function, which is recast as \code{KJRS()} in the \bold{Examples}. The function appears to be an approximation attributable to Jery R. Stedinger. The \bold{Examples} show a \dQuote{test} as working notes of TAC.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, not \code{P3_089(R).txt}---Named \code{critK10}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.

Interagency Advisory Committee on Water Data (IACWD), 1982, Guidelines for determining flood flow frequency: Bulletin 17B of the Hydrology Subcommittee, Office of Water Data Coordination, U.S. Geological Survey, Reston, Va., 183 p.
}
\author{W.H. Asquith consulting T.A. Cohn sources}
\seealso{\code{\link{CritK}}
}
\examples{
critK10(58)
#[1] 2.824

\donttest{ # Modified slightly from TAC sources (Original has the # Not run:)
# KJRS() is the ".kngb()" function in TAC sources
n <- 10:149; KJRS <- function(n) -0.9043+3.345*sqrt(log10(n))-0.4046*log10(n)
result <- data.frame(n=n, Ktrue=sapply(n, critK10), # 17B single Grubbs--Beck
                          KJRS= sapply(n, KJRS   )) # name mimic of TAC sources }

\dontrun{ # Near verbatim from TAC sources, GGBK() does not work, issues a stop().
# KJRS() is the ".kngb()" function in TAC sources
n <- 10:149; KJRS <- function(n) -0.9043+3.345*sqrt(log10(n))-0.4046*log10(n)
result <- data.frame(n=n, Ktrue=sapply(n, critK10), # 17B single Grubbs--Beck
                          KJRS= sapply(n, KJRS   ), # name mimic of TAC sources
                          KTAC= sapply(n, GGBK   )) # name mimic of TAC sources}
}
\keyword{Bulletin 17B}
\keyword{Single Grubbs--Beck Test (critical values)}
\keyword{critical values}

\encoding{utf8}
\name{peakTMtoHRS}
\alias{peakTMtoHRS}
\title{Peak Time to Decimal Hours or Compute Mean and Standard Deviations}
\description{
Parse the \code{peak_tm} (peak time) field from the USGS National Water Information System into decimal hours or compute the circular mean and standard deviations of the hours.
}
\usage{
peakTMtoHRS(x, type=c("asis", "musd"), parseHHMM=TRUE)
}
\arguments{
  \item{x}{A colon delimited hour:minutes format of \code{peak_tm} of retrieved peak streamflows as would be reported through \code{\link{splitPeakCodes}};}
  \item{type}{Return a vector of the decimal hours or compute the mean and standard deviations by circular mathematics; and}
  \item{parseHHMM}{A logical to parse the \code{HH:MM} notation from decimal hours in \code{x}, and setting to false will treat time has being in decimal hours.}
}
\value{
  A vector of decimal hours (24-hour clock) or the circular mean and standard deviation.
}
\source{
Consultation of \pkg{circular} package sources.
}
\references{
Mardia, K.V., 1972, Statistics of directional data: London, Academic Press, London, sec. 26.5, p. 617.

Lund, U., Agostinelli, C., Arai, H., Gagliardi, A., \enc{García-Portugués}{Garcia-Portugues}, E., Giunchi, D., Irisson, JO., Pocernich, M., and Rotolo, F., 2024, circular---Circular statistics: R package version 0.5-1, dated August 29, 2024, accessed March 17, 2025, at \doi{10.32614/CRAN.package.circular}.
}
\author{ W.H. Asquith }
\seealso{\code{\link{splitPeakCodes}}
}
\examples{
\dontrun{
  pk <- dataRetrieval::readNWISpeak("08167000", convertType=FALSE)
  pk <- splitPeakCodes(pk, all_peaks_na_okay=TRUE)
  pk <- pk[! is.na(pk$peak_va) & pk$peak_va >= median(pk$peak_va, na.rm=TRUE),]
  pk$hours <- peakTMtoHRS(pk)
  print(peakTMtoHRS(pk, type="musd"))

  circ <- circular::circular(pk$hours, type="angle", units="hours",
                   template="clock24", rotation="clock")
  mu <- as.vector(circular::mean.circular(circ, na.rm=TRUE))
  mu <- ifelse(mu < 0, 24 + mu, mu)
  print(c(mu, circular::sd.circular(circ, na.rm=TRUE)))
  circular::plot.circular(circ) #}

\dontrun{
  LAarea <- c("10263500", "10263900", "10264530", "11085000", "11092450", "11097000",
              "11098000", "11101250", "11102300", "11108134", "11109525", "11109550")
  PK <- new.env(); frm0000 <- NULL; n <- 0
  for(site in LAarea) {
    pk <- dataRetrieval::readNWISpeak(site, convert=FALSE)
    pk <- MGBT::splitPeakCodes(pk); pk <- pk[! is.na(pk$peak_tm),]
    pk <- pk[pk$peak_tm != "00:00", ]; pk <- pk[pk$peak_tm != "12:00", ]
    if(nrow(pk) == 0) { message("skipping", site); next }
    pk$hrs <- peakTMtoHRS(pk)
    musd <- peakTMtoHRS(pk, type="musd"); n <- n + musd[3]
    frm0000 <- c(frm0000, musd[1])
  }
  names(  frm0000) <- NULL
  frm0000[frm0000 > 12] <- 24 - frm0000[frm0000 > 12]
  musd <- peakTMtoHRS(frm0000, type="musd", parseHHMM=FALSE)
  message(paste0("For ", length(LAarea), " streamgages in Los Angeles county\n",
          "and ", n, " occurrences of peak streamflow time has a\n",
          "mean site-mean of ", round(musd[1], digits=2), " hours from midnight."))
}
}
\keyword{peak time}

\encoding{utf8}
\name{CondMomsChi2}
\alias{CondMomsChi2}
\title{Conditional Moments: N.B. Moments employ only observations above Xsi }
\description{
Compute the \eqn{\chi^2}-conditional moments (Chi-squared distributed moments) based on only those (\eqn{n} \eqn{-} \eqn{r}) observations above a threshold \eqn{X_{si}} for a sample size of \eqn{n} and \eqn{r} number of truncated observations. The first moment is \code{(gtmoms(xsi,2) -} \code{gtmoms(xsi,1)^2)} that is in the first returned column. The second moment (variance of S-squared) is \code{V(n,r,pnorm(xsi))[2,2]} that is in the second returned column. Further mathematical details are available under functions \code{\link{gtmoms}} and \code{\link{V}}.
}
\usage{
CondMomsChi2(n, r, xsi)
}
\arguments{
  \item{n}{The number of observations;}
  \item{r}{The number of truncated observations; and}
  \item{xsi}{The lower threshold (see \code{\link{gtmoms}}).}
}
\value{
  The value a two-column, one-row \R \code{matrix}.
}
\note{
TAC sources define a \code{\link{CondMomsZ}} function along with this function. However, the \code{\link{CondMomsZ}} function appears to not be used for any purpose. Only the \code{CondMomsChi2} is needed for the MGBT test.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, \code{P3_089(R).txt}---Named \code{CondMomsChi2}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.
}
\author{ W.H. Asquith consulting T.A. Cohn sources }
\seealso{\code{\link{CondMomsZ}}, \code{\link{gtmoms}}
}
\examples{
CondMomsChi2(58, 2, -3.561143)
#          [,1]       [,2]
#[1,] 0.9974947 0.03574786

# Note that CondMomsChi2(58, 2, -3.561143)[2] == V(58, 2, pnorm(-3.561143))[2,2]
}
\keyword{moments}
\keyword{moments (conditional)}

\encoding{utf8}
\name{plotFFQevol}
\alias{plotFFQevol}
\title{Plot Flood-Frequency in Time}
\description{
Plot the temporal evolution of peak-streamflow frequency using the method of L-moments on the systematic record, assuming that \code{appearsSystematic} fully identifies the systematic record (see \code{\link{splitPeakCodes}}), but have the results align on the far right side to other results. These other results are intended to be an official best estimate of the peak-streamflow frequency using all available information and are generally anticipated to be from Bulletin 17C (B17C) (England and others, 2018). The motivation for this function is that some have contacted one of the authors with a desire to show a temporal evolution of the estimates of the 2-, 10-, 100-, and 500-year peak-streamflow values but simultaneously have \dQuote{current} (presumably a modern year [not necessarily the last year of record]) results. A target year (called the \code{final_water_yr}) is declared. Base10 logarithmic offsets from the so-called final quantile values at that year are computed. These then are used additively to correct the temporal evolution of the L-moment based peak-streamflow frequency values.

The code herein makes use of the \code{f2flo}, \code{lmoms}, \code{parpe3}, \code{quape3}, \code{T2prob}, \code{x2xlo} functions from the \pkg{lmomco} package, and because this is the only instance of this dependency, the \pkg{lmomco} package is treated as a suggested package and not as a dependency for the \pkg{MGBT} package. The \bold{Examples} use the \pkg{dataRetrieval} package for downloading peak streamflow data.
}
\usage{
plotFFQevol(pkenv, lot=NULL, finalquas=NULL, log10offsets=NULL,
          minyrs=10, byr=1940, edgeyrs=c(NA, NA), logyaxs=TRUE,
          lego=NULL, maxs=NULL, mins=NULL, names=NULL, auxeyr=NA,
          xlab="Water Year", ylab="Peak streamflow, in cubic feet per second",
          title="Time Evolution of Peak-Frequency Streamflow Estimates\n",
          data_ext="_data.txt", ffq_ext="_ffq.txt", path=tempdir(),
          showfinalyr=TRUE, usewyall=FALSE, silent=TRUE, ...)
}
\arguments{
  \item{pkenv}{A \code{environment} having a mandatory column titled \code{peak_va} and \code{peak_cd}. It is required to have run \code{\link{makeWaterYear}} and \code{\link{splitPeakCodes}} first;}
  \item{lot}{A vector of low-outlier thresholds for the stations stored in \code{pkenv};}
  \item{finalquas}{A \code{data.frame} (see \bold{Examples}) of the final peak-streamflow frequency values to be soon (matched!) at the right side of the plot. These values could stem from B17C-like analyses using all the features therein. These values force, through an offsetting method in log10-space, the method of L-moment results in time to \dQuote{land} to the results at the end (right side).}
  \item{log10offsets}{An optional offset \code{data.frame} (see \bold{Examples}) to add to \code{finalquas} so that the final right side method of L-moment results match the results given in \code{finalquas};}
  \item{minyrs}{The minimum number of years (sample size) before trying to fit a log-Pearson type III distribution by method of L-moments---the \code{lot} will be honored through a conditional distribution truncation;}
  \item{byr}{The beginning year for which to even start consultation for peaks in time frequency analysis;}
  \item{edgeyrs}{The optional bounding years for the horizontal axis but potentially enlarged by the data as well as the setting of \code{usewyall};}
  \item{logyaxs}{A logical to trigger a logarithmic plot. The logarithmic plot is based on the function \code{\link{plotPeaks}} and is more sophisticated. The linear option is provided herein as some users have deemed logarithmic axes too complicated for engineers to understand (seriously);}
  \item{lego}{The year at which to start the legend;}
  \item{maxs}{The upper limit of the vertical axis;}
  \item{mins}{The lower limit of the vertical axis;}
  \item{names}{An optional character string vector to be used as a plot title;}
  \item{auxeyr}{An optional auxiliary ending year to superceed the \code{final_water_yr} from the \code{finalquas}. The \code{auxeyr} could lead to interesting conflict in the graphic. For example, a 500-year value being less than the 100-year. If a distribution swings between sign of the skew parameter and because the offsets are computed at a the discrete point in time (\code{final_water_yr}), then the offset could be too large at the 500-yr level and cause overlap (see \bold{Examples});}
  \item{xlab}{An optional x-label of the plot;}
  \item{ylab}{An optional y-label of the plot;}
  \item{title}{An optional super title for the plot to be shown above \code{names};}
  \item{data_ext}{An optional file name extension to the data (only water year, peak streamflow value, and \code{appearsSystematic}) retained in this file relative to the greater results inside the \code{pkenv}. If an output file is not desired, set to \code{NA};}
  \item{ffq_ext}{An optional file name extension to the flood-flow frequency (water year and 2, 10, 100, and 500 year) output. If an output file is not desired, set to \code{NA};}
  \item{path}{The path argument for the output files with a default to a temporary directory for general protection of the user;}
  \item{showfinalyr}{A logical to control whether a line is drawn showing the location of the final water year (the year of the quantile estimates, \code{final_water_yr}) that  spans top to bottom on the graphic;}
  \item{usewyall}{A logical to consult the \code{isCode7} for determination of the horizontal axis limits;}
  \item{silent}{A logical for some status update messaging; and}
  \item{...}{Additional arguments to pass to \code{plot}.}
}
\value{
  The log10-offset values are returned if not given otherwise this function is used for its graphical side effects.
}
\author{W.H. Asquith}
\source{
Earlier code developed by W.H. Asquith in about 2016.
}
\references{
England, J.F., Cohn, T.A., Faber, B.A., Stedinger, J.R., Thomas Jr., W.O., Veilleux, A.G., Kiang, J.E., and Mason, R.R., 2018, Guidelines for determining flood flow frequency Bulletin 17C: U.S. Geological Survey Techniques and Methods, book 4, chap. 5.B, 148 p., \doi{10.3133/tm4B5}.
}
\seealso{\code{\link{plotPeaks}}
}
\examples{
\dontrun{
# The dataRetrieval package is not required by MGBT algorithms.
opts <- options(scipen=7)
opar <- par(no.readonly=TRUE)
par(mgp=c(3,0.5,0), las=1) # going to tick inside, change some parameters

names <- c("08167000 Guadalupe River at Comfort, Tex.")
stations <- c("08167000"); LOT <- c(3110)
maxs <- c(525000); lego <- c(1940)

PKS <- new.env()
for(station in "08167000") {
  message("Pulling peaks for ",station)
  data <- dataRetrieval::readNWISpeak(station, convertType=FALSE)
  data <- splitPeakCodes(MGBT::makeWaterYear(data))
  assign(station, data, PKS)
}

# This example should run fine though the resulting curves will end in 2015 because
# this is the declared ending year of 2015. Data points after 2015 will be shown
# but the FFQ values will be the values plotted. Yes, other return periods are shown
# here and dealt with internally, but only the 2, 10, 100, and 500 are drawn.
FFQ <- data.frame(site_no="08167000", final_water_yr=2015,
                  Q002= 3692, Q005= 21000, Q010= 40610, Q025= 69740,
                  Q050=91480, Q100=111600, Q200=129400, Q500=149200)
# Now compute the offsets associated with those given above.
OFFSET <- plotFFQevol(PKS, lot=LOT, finalquas=FFQ)
# Notice the plotFFQevol() is called twice. One call is to compute the
# offsets, and the next is to use them and make a pretty plot.
plotFFQevol(PKS, lot=LOT, finalquas=FFQ, log10offsets=OFFSET,
            maxs=maxs, mins=rep(0,length(maxs)), names=names,
            lego=lego, logyaxs=FALSE, edgeyrs=c(1940,2020), usewyall=TRUE)

# Now a change up, lets say these values are good through the year 1980, and yes,
# these are the same values shown above.
FFQ$final_water_yr <- 1980
OFFSET <- plotFFQevol(PKS, lot=LOT, finalquas=FFQ) # compute offsets
# Now using auxeyr=2020, will trigger the evolution through time and the results in
# 1980 will match these given in the FFQ. One will see (presumably for many years
# after 2017) the crossing of the 500 year to the 100 year in about 1993.
plotFFQevol(PKS, lot=LOT, finalquas=FFQ, log10offsets=OFFSET,
            maxs=maxs, mins=rep(0,length(maxs)), names=names, auxeyr=2020,
            lego=lego, logyaxs=FALSE, edgeyrs=c(1940,2020), usewyall=FALSE)

# Now back to the original FFQ but in log10 space and plotPeaks().
FFQ$final_water_yr <- 2017
OFFSET <- plotFFQevol(PKS, lot=LOT, finalquas=FFQ) # compute offsets
# Now using logyaxs=TRUE and some other argument changes
plotFFQevol(PKS, lot=LOT, finalquas=FFQ, log10offsets=OFFSET, title="",
            maxs=maxs, mins=rep(0,length(maxs)), names=names, auxeyr=2020,
            lego=NULL, logyaxs=TRUE, edgeyrs=c(1890,2020), usewyall=TRUE,
            showfinalyr=FALSE)
options(opts) # restore the defaults
par(opar)     # restore the defaults}
}
\keyword{graphics}

\encoding{utf8}
\name{EMS}
\alias{EMS}
\title{Expected values of M and S}
\description{
Compute expected values of \eqn{M} and \eqn{S} given \eqn{q_\mathrm{min}} and define the quantity
\deqn{z_r = \Phi^{(1)}(q_\mathrm{min})\mbox{,}}
where \eqn{\Phi^{(1)}(\cdot)} is the inverse of the standard normal distribution. As result, \eqn{q_\mathrm{min}} is itself a probability because it is an argument to the \code{qnorm()} function. The expected value \eqn{M} is defined as
\deqn{M = \Psi(z_r, 1)\mbox{,}}
where \eqn{\Psi(a,b)} is the \code{\link{gtmoms}} function. The \eqn{S} requires the conditional moments of the Chi-square (\code{\link{CondMomsChi2}}) defined as the two value vector \eqn{\mbox{}_2S} that provides the values \eqn{\alpha = \mbox{}_2S_1^2 / \mbox{}_2S_2} and \eqn{\beta = \mbox{}_2S_2 / \mbox{}_2S_1}. The \eqn{S} is then defined by
\deqn{S = \sqrt{\beta}\biggl(\frac{\Gamma(\alpha+0.5)}{\Gamma(\alpha)}\biggr)\mbox{.}}
}
\usage{
EMS(n, r, qmin)
}
\arguments{
  \item{n}{The number of observations;}
  \item{r}{The number of truncated observations? (confirm); and}
  \item{qmin}{A nonexceedance probability threshold for \eqn{X > q_\mathrm{min}}.}
}
\value{
  The expected values of \eqn{M} and \eqn{S} in the form of an \R \code{vector}.
}
\note{
TAC sources call on the explicit first index of \eqn{M} as literally \dQuote{\code{Em[1]}} for the returned vector, which seems unnecessary. This is a potential weak point in design because the \code{gtmoms} function is naturally vectorized and could potentially produce a vector of \eqn{M} values. For the implementation here, only the first value in \code{qmin} is used and a warning otherwise issued. Such coding prevents the return value from \code{EMS} accidentally acquiring a length greater than two. For at least samples of size \eqn{n=2}, over-ranging in a call to \code{lgamma(alpha)} happens for \code{alpha=0}. A \code{suppressWarnings()} is wrapped around the applicable line. The resulting \code{NaN} cascades up the chain, which will end up inside \code{\link{peta}}, but therein \code{SigmaMp} is not finite and a p-value of unity is returned.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, \code{P3_089(R).txt}---Named \code{EMS}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.
}
\author{W.H. Asquith consulting T.A. Cohn sources}
\seealso{\code{\link{CondMomsChi2}}, \code{\link{EMS}}, \code{\link{VMS}}, \code{\link{V}}, \code{\link{gtmoms}}
}
\examples{
EMS(58,2,.5)
#[1] 0.7978846 0.5989138

#  Monte Carlo experiment to test EMS and VMS functions
"test_EMS" <- function(nrep=1000, n=100, r=0, qr=0.2, ss=1) { # TAC named function
   set.seed(ss)
   Moms <- replicate(n=nrep, {
          x <- qnorm(runif(n-r,min=qr,max=1));
          c(mean(x),var(x))}); xsi <- qnorm(qr);
          list(
    MeanMS_obs = c(mean(Moms[1,]), mean(sqrt(Moms[2,])), mean(Moms[2,])),
    EMS        = c(EMS(n,r,qr), gtmoms(xsi,2) - gtmoms(xsi,1)^2),
    CovMS2_obs = cov(t(Moms)),
    VMS2       = V(n,r,qr),
    VMS_obs    = array(c(var(     Moms[1,]),
                         rep(cov( Moms[1,], sqrt(Moms[2,])),2),
                         var(sqrt(Moms[2,]))),    dim=c(2,2)),
    VMS        = VMS(n,r,qr)  )
}
test_EMS()
}
\keyword{moments}
\keyword{moments (conditional)}
\keyword{utility functions}

\encoding{utf8}
\name{plotPeaks_batch}
\alias{plotPeaks_batch}
\title{Plot for More than One Streamgage that Peak Streamflows with Emphasis on Peak Discharge Qualification Codes}
\description{
Plot U.S. Geological Survey peak streamflows for multiple streamgages. This function is a wrapper on calls to \code{\link{plotPeaks}} with data retrieval occurring just ahead.
}
\usage{
plotPeaks_batch(sites, file=NA, do_plot=TRUE,
                       silent=FALSE, envir=NULL, ...)
}
\arguments{
  \item{sites}{A list of USGS site identification numbers for which one-by-one, the peaks will be pulled from \code{dataRetrieval::}\code{readNWISpeak};}
  \item{file}{A portable document format output path and file name, setting to \code{NA} will plot into the running application unless \code{do_plot=FALSE};}
  \item{do_plot}{A logical triggering the plotting operations. This is a useful feature for test or for a iterative use as seen in the second \bold{Example}. A setting of false will set \code{file=NA} internally regardless of argument given;}
  \item{silent}{A logical to control status messaging;}
  \item{envir}{An optional and previously populated enviroment by site number storing a table of peaks from the \code{\link{splitPeakCodes}} function (see \bold{Examples}); and}
  \item{...}{Additional arguments to pass to \code{\link{plotPeaks}}.}
}
\value{
  A list is returned by streamgage of the retrieved data with an attribute \code{empty_sites} storing those site numbers given for which no peaks were retrieved/processed. The data structure herein recognizes a \code{NA} for a site.
}
\author{W.H. Asquith}
\references{
U.S. Geological Survey, 2019, USGS water data for the Nation: U.S. Geological Survey National Water Information System database, accessed October 11, 2019, at \doi{10.5066/F7P55KJN}.
}
\seealso{\code{\link{plotPeaks}}
}
\examples{
\dontrun{
  # The dataRetrieval package is not required by MGBT algorithms, but needed
  # for the the plotPeaks_batch() function.
  sites <- c("07358570", "07358280", "07058980", "07258000")
  PK <- plotPeaks_batch(sites, xlab="WATER YEAR",         lot=0,
                               ylab="STREAMFLOW, IN CFS", file=NA) # }

\dontrun{
  # In this example, two calls to plotPeaks_batch() are made. The first is to use
  # the function as a means to cache the retrieval of the peaks without the
  # plotting overhead etc. The second is to actually perform the plotting.
  pdffile <- tempfile(pattern = "peaks", tmpdir = tempdir(), fileext = ".pdf")
  sites <- c("08106300", "08106310", "08106350", "08106500") # 08106350 no peaks

  PK <- plotPeaks_batch(sites, do_plot=FALSE)    # a hack to use its wrapper on
  # dataRetrieval::readNWISpeak() to get the peaks retrieved and code parsed by
  # splitPeakCodes() and then we can save the PK for later purposes as needed.

  empty_sites <- attr(PK, "empty_sites") # we harvest 08106350 as empty
  message("EMPTY SITES: ", paste(empty_sites, collapse=", "))

  PK <- as.environment(PK) # now flipping to environment for the actually plotting
                           # plotting pass to follow next and retrieval is not made
  # save(empty_sites, PK, file="PEAKS.RData") # a way to save the work for later
  PK <- plotPeaks_batch(sites, xlab="WATER YEAR",        lot=0, envir=PK,
                               ylab="STREAMFLOW, IN CFS", file=pdffile)
  message("Peaks plotted in file=", pdffile) # }
}
\keyword{graphics}

\encoding{utf8}
\name{GGBK}
\alias{GGBK}
\title{Cohn Approximation for New Generalized Grubbs--Beck Critical Values for 10-Percent Test }
\description{
Compute Cohn's approximation of critical values at the 10-percent significance level for the new generalized Grubbs--Beck test.
}
\usage{
GGBK(n)
}
\arguments{
  \item{n}{The number of observations.}
}
\value{
  The critical value of the test was to be returned, but a \code{stop()} is issued instead because there is a problem with the function's call of \code{\link{CritK}} (see \bold{Note}).
}
\note{
In TAC sources, \code{GGBK} is the consumer of two global scope functions \code{fw()} and \code{fw1()}. These should be defined within the function to keep the scope local as they are unneeded anywhere else in TAC sources, and these thus have local scope in the implementation for the \pkg{MGBT} package.

\bold{A BUG FIX NEEDED---}Note that TAC has a problem in sources in that this function is incomplete. The function \code{\link{CritK}} is the issue, that function requires three arguments and appears to work (see \bold{Examples} under \code{\link{CritK}}), but TAC code passes four in the context of \code{GGBK}. At present (packaging of \pkg{MGBT}), it is not known if the word \dQuote{generalized} in this test has the same meaning as \dQuote{multiple} in the Multiple Grubbs--Beck Test. Also in TAC sources, it is as yet unclear what the \dQuote{new} in the title of this function means.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, not \code{P3_089(R).txt}---Named \code{GGBK}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.
}
\author{ W.H. Asquith consulting T.A. Cohn sources }
\seealso{\code{\link{critK10}}, \code{\link{CritK}}
}
\examples{
\dontrun{
GGBK(34) # but presently the function does not work }
}
\keyword{Generalized Grubbs--Beck Test (critical values)}

\encoding{utf8}
\name{MGBT-package}
\docType{package}
\alias{MGBT-package}
\title{Multiple Grubbs--Beck Low-Outlier Test}
\description{
The \pkg{MGBT} package provides the Multiple Grubbs--Beck low-outlier test (MGBT) (Cohn and others, 2013), and almost all users are only interested in the function \code{\link{MGBT}}. This function explicitly wraps the recommended implementation of the test, which is \code{\link{MGBT17c}}. Some other studies of low-outlier detection and study of the MGBT and related topic can be found in Cohn and others (2019), Lamontagne and Stedinger (2015), and Lamontagne and others (2016).

The package also provides some handy utility functions for non-interpretive processing of U.S. Geological Survey National Water Information System (NWIS) annual-peak streamflow data. These utilities include \code{\link{makeWaterYear}} that adds the water year and parses the date-time stamp into useful subcomponents, \code{\link{splitPeakCodes}} that splits the peak discharge qualification codes, and that \code{\link{plotPeaks}} plots the peak time series with emphasis on visualization of select codes, zero flow values, and missing records (annual gaps).

The context of this package is useful to discuss. When logarithmic transformations of data prior to parameter estimation of probability models are used and interest in the the right-tail of the distribution exists, the MGBT is effective in adding robustness to flood-frequency analyses. Other similar distributed earth-system data analyses could also benefit from the test. The test can be used to objectively identify \dQuote{low outliers} (generic) or specific to floods, \dQuote{potentially influential low floods} (PILFs)---in general, these terms are synonymous.

Motivations of the \pkg{MGBT} package are related to the so-called \dQuote{Bulletin 17C} guidelines (England and others, 2018) for flood-frequency analyses. These are updated guidelines to those in Bulletin 17B (IACWD, 1982).  Bulletin 17C (B17C) are Federal guidelines for performing flood-frequency analyses in the United States. The MGBT is implemented in the U.S. Geological Survey (USGS)-PeakFQ software (USGS, 2014; Veilleux and others, 2014), which implements much of B17C (England and others, 2018).

The MGBT test is especially useful in practical applications in which small (possibly uninteresting) events (low-magnitude tail, left-tail side) can occur from divergent populations or processes than those forming the high-magnitude tail (right-tail side) of the probability distribution. One such large region of the earth is much of the arid to semi-arid hydrologic setting for much of Texas for which a heuristic method predating and unrelated to MGBT was used for low-outlier threshold identification (see \code{\link{ASlo}}). Arid and semi-arid regions are particularly sensitive to the greater topic motivating the MGBT (Timothy A. Cohn, personal commun., 2007).

\bold{Note on Sources and Historical Preservation}---Various files (\code{.txt}) of \R code are within this package and given and are located within the directory \code{/inst/sources}. The late Timothy A. Cohn (TAC) communicated \R code to WHA (author William H. Asquith) in August 2013 for computation of MGBT within a flood-frequency project of WHA's. The August 2013 code is preserved verbatim in file \code{LowOutliers_wha(R).txt}, which also contains code by TAC to related concepts. Separately, TAC communicated \R code to JFE (contributor John F. England) in 2016 (actually over many years they had extensive and independent communication from those with WHA) for computation of MGBT and other low-outlier related concepts. This 2016 code is preserved verbatim in file \code{LowOutliers_jfe(R).txt}. TAC also communicated \R code to JFE for computation of MGBT and other low-outlier related concepts for production of figures for the MGBT paper (Cohn and others, 2013). (Disclosure, here it is unclear whether the \R code given date as early as this paper or before when accounting for the publication process.)

The code communications are preserved verbatim in file \code{FigureMacros_jfe(R)}\code{.txt} for which that file is dependent on \code{P3_075_jfe(R).txt}. The \code{_jfe} has been added to denote stewardship at some point by JFE. The \code{P3_075_jfe(R).txt} though is superseded by \code{P3_089(R)}\code{.txt} in which TAC was editing as late as the summer of 2016. The \code{P3_089(R).txt} comes to WHA through Julie E. Kiang (USGS, May 2016). This file should be considered TAC's canonical and last version for MGBT as it appears in the last set of algorithms TAC while he was working on development of a USGS-PeakFQ-like Bulletin 17C implementation in \R. As another historical note, file \code{P3_085_wha(R).txt} is preserved verbatim and was given to WHA at the end of November 2015 less than two weeks before TAC dismissed himself for health reasons from their collaboration on a cooperative research project in cooperation with the U.S. Nuclear Regulatory Commission (Asquith and others, 2017).

Because of a need for historical preservation at this juncture, there is considerable excess and directly-unrelated code to MGBT and low-outlier identification in the aforementioned files though MGBT obviously is contained therein. In greater measure, much of the other code is related to the expected moments algorithm (EMA) for fitting the log-Pearson type III distribution to annual flood data. The \pkg{MGBT} package is purely organized around MGBT and related concepts in a framework suitable for more general application than the purposes of B17C and thus the contents of the \code{P3_###(R).txt} series of files. \bold{It is, however, the prime objective of the \pkg{MGBT} package to be nearly plug-in replacement for code presently (2019) bundled into \code{P3_###(R).txt} or derivative products. Therefore, any code related to B17C herein must not be considered canonical in any way.}

\bold{Notes on Bugs in Sources by TAC}---During the core development phase of this package made in order to stabilized history left by TAC and other parts intimately known by JFE (co-author and contributor), several inconsistencies to even bugs manifested. These need very clear discussion.

First, there is the risk that the author (WHA) ported TAC-based \R to code in this package and introduced new problems. Second, there is a chance that TAC had errors and (or) WHA has misunderstood some idiom. Third, as examples herein show, there is (discovery circa June 2017) a great deal of evidence that TAC incompletely ported from presumably earlier(?) FORTRAN, which forms the basis of the USGS-PeakFQ software, that seems correct, into \R---very subtle and technical issues are involved. Fourth, WHA and GRH (contributor George \dQuote{Rudy} Herrmann) in several very large batch processing tests (1,400+ time series of real-world peak streamflows) on Texas data pushed limits of \R numerics, and these are discussed in detail as are WHA's compensating mechanisms. Several questions of apparent bugs or encounters with the edges of \R performance would be just minutes long conversations with TAC, but this is unfortunately not possible.

In short, it appears that several versions of MGBT by TAC in \R incorrectly performed a computation known as \dQuote{swept out} from the median (\code{\link{MGBTcohn2016}} and \code{\link{MGBTcohn2013}}). Curiously a specific branch (\code{\link{MGBTnb}}) seems to fix that but caused a problem in a computation known as \dQuote{sweep in} from the first order statistic.

Further, numerical cases can be found triggering divergent integral warnings from \code{integrate()}---WHA compensated by adding a Monte Carlo integration routine as backup. It is beyond the scope here to speculate on FORTRAN code performance. In regards to \R and numerically, cases can be found trigging numerical precision warnings from the cumulative distribution function of the t-distribution (\code{pt()})---WHA compensated by setting p-value to limiting small value (zero). Also numerically, cases can be found triggering a square-root of a negative number in \code{\link{peta}}---WHA compensates by effectively using a vanishingly small probability of the t-distribution. TAC's MGBT approach fails if all data values are equal---WHA compensates by returning a default result of a zero-value MGBT threshold. This complexity leads to a lengthy \bold{Examples} section in this immediate documentation as well as in the \code{\link{MGBT}} function. All of these issues finally led WHA to preserve within the \pkg{MGBT} package several MGBT-focused implementations as distinct functions.

\bold{Note on Mathematic Nomenclature}---On first development of this package, the mathematics largely represent the port from the sources into a minimal structure to complete description herein. TAC and others have published authoritative mathematics elsewhere. The primary author (WHA) deliberately decided to build the \pkg{MGBT} package up from the TAC sources first. Little reference to TAC's publications otherwise is made.
}
\references{
Asquith, W.H., Kiang, J.E., and Cohn, T.A., 2017, Application of at-site peak-streamflow frequency analyses for very low annual exceedance probabilities: U.S. Geological Survey Scientific Investigation Report 2017--5038, 93 p., \doi{10.3133/sir20175038}.

Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.

Cohn, T.A., Barth, N.A., England, J.F., Jr., Faber, B.A., Mason, R.R., Jr., and Stedinger, J.R., 2019, Evaluation of recommended revisions to Bulletin 17B: U.S. Geological Survey Open-File Report 2017--1064, 141 p., \doi{10.3133/ofr20171064}.

Cohn, T.A., England, J.F., Berenbrock, C.E., Mason, R.R., Stedinger, J.R., and Lamontagne, J.R., 2013, A generalized Grubbs--Beck test statistic for detecting multiple potentially influential low outliers in flood series: Water Resources Research, v. 49, no. 8, pp. 5047--5058.

England, J.F., Cohn, T.A., Faber, B.A., Stedinger, J.R., Thomas Jr., W.O., Veilleux, A.G., Kiang, J.E., and Mason, R.R., 2018, Guidelines for determining flood flow frequency Bulletin 17C: U.S. Geological Survey Techniques and Methods, book 4, chap. 5.B, 148 p., \doi{10.3133/tm4B5}

Interagency Advisory Committee on Water Data (IACWD), 1982, Guidelines for determining flood flow frequency: Bulletin 17B of the Hydrology Subcommittee, Office of Water Data Coordination, U.S. Geological Survey, Reston, Va., 183 p.

Lamontagne, J.R., and Stedinger, J.R., 2015, Examination of the Spencer--McCuen outlier-detection test for log-Pearson type 3 distributed data: Journal of Hydrologic Engineering, v. 21, no. 3, pp. 04015069:1--7.

Lamontagne, J.R., Stedinger, J.R., Yu, Xin, Whealton, C.A., and Xu, Ziyao, 2016, Robust flood frequency analysis---Performance of EMA with multiple Grubbs--Beck outlier tests: Water Resources Research, v. 52, pp. 3068--3084.

U.S. Geological Survey (USGS), 2018, PeakFQ---Flood frequency analysis based on Bulletin 17B and recommendations of the Advisory Committee on Water Information (ACWI) Subcommittee on Hydrology (SOH) Hydrologic Frequency Analysis Work Group (HFAWG), version 7.2: Accessed November 29, 2018, at \url{https://www.usgs.gov/tools/peakfq}.

Veilleux, A.G., Cohn, T.A., Flynn, K.M., Mason, R.R., Jr., and Hummel, P.R., 2014, Estimating magnitude and frequency of floods using the PeakFQ 7.0 program: U.S. Geological Survey Fact Sheet 2013--3108, 2 p., \doi{10.3133/fs20133108}.
}
\author{William H. Asquith (WHA) \email{wasquith@usgs.gov} }
\seealso{
   \code{\link{MGBT}}, \code{\link{splitPeakCodes}}, \code{\link{plotPeaks}}, \code{\link{readNWISwatstore}}
}
\examples{
\donttest{
# Peaks for 08165300 (1968--2016, systematic record only)
#https://nwis.waterdata.usgs.gov/nwis/peak?site_no=08385600&format=hn2
Peaks <- c(3200, 44, 5270, 26300, 1230, 55, 38400, 8710, 143, 23200, 39300, 1890,
  27800, 21000, 21000, 124, 21, 21500, 57000, 53700, 5720, 50, 10700, 4050, 4890, 1110,
  10500, 475, 1590, 26300, 16600, 2370, 53, 20900, 21400, 313, 10800, 51, 35, 8910,
  57.4, 617, 6360, 59, 2640, 164, 297, 3150, 2690)

MGBTcohn2016(Peaks)
#$klow
#[1] 24
#$pvalues
# [1] 0.8245714657 0.7685258183 0.6359392507 0.4473443285 0.2151390091 0.0795065159
# [7] 0.0206034851 0.0036001474 0.0003376923 0.0028133490 0.0007396869 0.0001427225
#[13] 0.0011045550 0.0001456356 0.0004178758 0.0004138897 0.0123954279 0.0067934260
#[19] 0.0161448464 0.0207025800 0.0483890616 0.0429628125 0.0152045539 0.0190853626
#$LOThresh
#[1] 3200

# ---***--------------***--- Note the mismatch ---***--------------***---
#The USGS-PeakFQ (v7.1) software reports:
#EMA003I-PILFS (LOS) WERE DETECTED USING MULTIPLE GRUBBS-BECK TEST  16    1110.0
#  THE FOLLOWING PEAKS (WITH CORRESPONDING P-VALUES) WERE CENSORED:
#            21.0    (0.8243)
#            35.0    (0.7680)
#            44.0    (0.6349)
#            50.0    (0.4461)      # Authors' note:
#            51.0    (0.2150)      # Note that the p-values from USGS-PeakFQv7.1 are
#            53.0    (0.0806)      # slightly different from those emanating from R.
#            55.0    (0.0218)      # These are thought to be from numerical issues.
#            57.4    (0.0042)
#            59.0    (0.0005)
#           124.0    (0.0034)
#           143.0    (0.0010)
#           164.0    (0.0003)
#           297.0    (0.0015)
#           313.0    (0.0003)
#           475.0    (0.0007)
#           617.0    (0.0007)
# ---***--------------***--- Note the mismatch ---***--------------***---

# There is a problem somewhere. Let us test each of the TAC versions available.
# Note that MGBTnb() works because the example peaks are ultimately a "sweep out"
# problem. MGBT17c() is a WHA fix to TAC algorithm, whereas, MGBT17c.verb() is
# a verbatim, though slower, WHA port of the written language in Bulletin 17C.
MGBTcohn2016(Peaks)$LOThres # LOT=3200  (WHA sees TAC problem with "sweep out".)
MGBTcohn2013(Peaks)$LOThres # LOT=16600 (WHA sees TAC problem with "sweep out".)
MGBTnb(Peaks)$LOThres       # LOT=1110  (WHA sees TAC problem with "sweep in". )
MGBT17c(Peaks)$index        # LOT=1110  (sweep indices:
                            #   ix_alphaout=16, ix_alphain=16, ix_alphazeroin=0)
MGBT17c.verb(Peaks)$index   # LOT=1110  (sweep indices:
                            #   ix_alphaout=16, ix_alphain=NA, ix_alphazeroin=0)

# Let us now make a problem, which will have both "sweep in" and "sweep out"
# characteristics, and note the zero and unity outliers for the "sweep in" to grab.
Peaks <- c(0,1,Peaks)
MGBTcohn2016(Peaks)$LOThres # LOT=3150         ..ditto..
MGBTcohn2013(Peaks)$LOThres # LOT=16600        ..ditto..
MGBTnb(Peaks)$LOThres       # LOT=1110         ..ditto..
MGBT17c(Peaks)$index        # LOT=1110  (sweep indices:
                            #   ix_alphaout=18, ix_alphain=18, ix_alphazeroin=2
MGBT17c.verb(Peaks)$index   # LOT=1110  (sweep indices:
                            #   ix_alphaout=18, ix_alphain=NA, ix_alphazeroin=2

#The USGS-PeakFQ (v7.1) software reports:
#    EMA003I-PILFS (LOS) WERE DETECTED USING MULTIPLE GRUBBS-BECK TEST  17    1110.0
#      THE FOLLOWING PEAKS (WITH CORRESPONDING P-VALUES) WERE CENSORED:
#               1 ZERO VALUES
#             1.0    (0.0074)
#            21.0    (0.4305)
#            35.0    (0.4881)
#            44.0    (0.3987)
#            50.0    (0.2619)
#            51.0    (0.1107)
#            53.0    (0.0377)
#            55.0    (0.0095)
#            57.4    (0.0018)
#            59.0    (0.0002)
#           124.0    (0.0018)
#           143.0    (0.0006)
#           164.0    (0.0002)
#           297.0    (0.0010)
#           313.0    (0.0002)
#           475.0    (0.0005)
#           617.0    (0.0005) #}
}
\keyword{Multiple Grubbs--Beck Test}
\keyword{Bulletin 17B}
\keyword{Bulletin 17C}

\encoding{utf8}
\name{ratioPeakMax1Day}
\alias{ratioPeakMax1Day}
\title{Relations Between Peak Streamflows to 1-Day Annual Maxima}
\description{
Both daily mean streamflow and instantaneous peak streamflows for many U.S. Geological Survey streamgages are available within the National Water Information System (NWIS). Peak streamflows are aggregated by water year (October 1 to September 30) NWIS and such streamflows are especially useful for flood frequency computations. Daily mean streamflows can be processed on a water year basis and 1-day annual maxima computed, and these maxima represent a 1-day volume as it were and are useful for flood volume frequency computations.

By strict definition, peak streamflows must equal or exceed 1-day annual maxima. Further 1-day annual maxima themselves should also underestimate a true volume maxima because a 1-day time step (midnight to midnight) is not a moving 24-hour or finer time-step resolution through streamflow hydrographs. Therefore, there are two mechanisms for 1-day maxima to be lesser than the peaks.

If a watershed is very large, flood hydrographs can be expected to be temporally wide and slow changing, such situations might have peak and 1-day annual maxima effectively equal to each other. Alternatively, if watershed is very small, flood hydrographs can be expected to be temporally of short duration and the peaks much larger than the 1-day annual maxima.

In terms of frequency computations, the peaks represent one dataset and the 1-day annual maxima another. There is not a restriction that the 1-day annual maxima be contemporaneously timed with the peak; although the hydrograph producing the peak is often aligned with maximum volume. The computations described herein consider the peak and the 1-day annual maxima to be simply distinct events and no processing of contemporaneousness is made aside from the simple water year designation.

This function supports optional caching of daily streamflow values and the peak streamflow values by environment keyed on the streamflow identification number. If a given cache is not provided, then the retrieval of the daily and (or) peak data are made using the \pkg{dataRetrieval} package. For the daily value cache, simply a data frame is needed of the \code{Date} and \code{Flow} columns from a previous or similar operation of the \code{dataRetrieval::readNWISdv()} function. For the peak value cache, simply a data frame from a previous operation of \code{dataRetrieval::readNWISpeak()} function is needed. The peak table internally is processed through \code{\link{splitPeakCodes}} to isolate peak streamflows by water year that appear to be part of systematic data collection activities. So called \dQuote{opportunistic} peaks are ignored, but so-called \dQuote{historical} peaks might be retained if the record appears as systematic.
}
\usage{
ratioPeakMax1Day(siteNumber, dvenv=NULL, pkenv=NULL, as.list=FALSE,
                 missing.days=0, rm.ratios.lt1=TRUE, silent=TRUE, ...)
}
\arguments{
  \item{siteNumber}{USGS streamgage identification number and nomenclature matches that of the \pkg{dataRetrieval} package. This function does not support a vector of site numbers and only the first will be silently used.}
  \item{dvenv}{An optional \R \code{environment} previously populated with daily mean streamflows. The environment needs the USGS streamgage identification number as the key and the value needs to be a data frame having just the two columns \code{Date} and \code{Flow};}
  \item{pkenv}{An optional \R \code{environment} previously populated peak streamflows. The environment needs the USGS streamgage identification number as the key;}
  \item{as.list}{Return an extensive list of the various tables produced internally, a ratio table for ratios less than 1, and the ratio table. The later table could include ratios less than 1 if \code{rm.ratios.lt1} is false.}
  \item{missing.days}{The number of permissible missing days in a given year on which to still report it as a complete water year, which is the \dQuote{annual} basis of the aggregation for the 1-day annual maxima.}
  \item{rm.ratios.lt1}{Remove records for which the ratio of peak to 1-day maxima is less than 1. Situations such as this might exist because of (1) minor differences in the numerical values, (2) outright erroneous information in NWIS and a user might want to contact the local USGS office operating the streamgage and possibly the database could be legitimately fixed, or (3) other reasons. The default as true is likely the more suitable for data-mining endeavors rather than trying to \dQuote{find} erroneous situations.}
  \item{silent}{Suppress informative calls to \code{message()}.}
  \item{...}{Additional arguments to pass to \code{dataRetrieval::}\code{readNWISdv()}.}
}
\value{
  An \R \code{data.frame} (the \dQuote{ratio table}) is returned for a false \code{as.list}:
  \item{site_no}{The streamgage identification number;}
  \item{water_yr}{The water year and support only for water year (October 1 through September 30) is made because USGS peak streamflows are uniquely water-year based;}
  \item{dvflow_count}{The number of days available in the water year with regard to a condition that a complete water year has greater than or equal to 365 minus \code{missing.days}. This logic does support leap year and therefore the minimum \code{dvflow_count} for a given year would be either be 363 or 364 if \code{missing.days=2}.}
  \item{peak_flow}{The reported peak streamflow water year identified as stemming from systematic data collection activities in accordance to algorithm within the \code{\link{splitPeakCodes}} for a year for which annual 1-day maxima exist;}
  \item{mx1d_flow}{The 1-day annual maxima streamflow (midnight to midnight) for the water year. Note, that the 1-day annual maxima are themselves underestimated because a moving window through unit values (the instantaneous streamflows, such as hourly or 15 minute) are not considered in the computations and therefore no \dQuote{fixed-interval bias correction} is involved in the operation of this function.}
  \item{ratio}{Ratio of the \code{peak_flow} to the \code{mx1d_flow}. Because a fixed-interval bias correction on the 1-day maxima was not involved in the computations (see \code{mx1d_flow}), this ratio is thought of as biased high. The values are rounded to eight digits, which is more than enough in practical circumstances but also at data-mining scales will produce somewhat smaller file sizes.}
  \item{log10diff}{The base-10 logarithmic difference between the \code{peak_flow} and the \code{mx1d_flow} defined as the logarithmic peak minus the logarithmic maxima. Because a fixed-interval bias correction on the 1-day maxima was not involved in the computations (see \code{mx1d_flow}), this ratio is thought of as biased high. The values are rounded to eight digits, which is more than enough in practical circumstances but also at data-mining scales will produce somewhat smaller file sizes.}
}
\seealso{ \code{\link{splitPeakCodes}} }
\examples{
\dontrun{
  site <- "08062700" # Example of 1980 peak being lower than 1-day maxima.
  D <- ratioPeakMax1Day(site, missing.days=0, rm.ratios.lt1=FALSE)
  print(D[D$ratio < 1,]) # 1980 on 18 Sep 2024 retrieval tests #}

\dontrun{
  # site <- "08019200" # Example of peaks and maxima nearly identical.
    site <- "08167000" # Example of peaks and maxima being quite different.
  D <- ratioPeakMax1Day(site, missing.days=7) # permit a week of missing
  if(class(D) == "list") D <- D$ratios # if user added as.list=TRUE to call
  por <- paste0(min(D$water_yr), "-", max(D$water_yr),
                 " (", nrow(D), " processed water years)")
  plot(D$mx1d_flow, D$peak_flow, pch=21, col="black", bg="white", log="xy",
       xlab="Water year 1-day annual maxima streamflow, in cfs",
       ylab="Water year annual peak streamflow, in cfs")
  abline(0, 1); mtext(site, line=0.25)
  legend("topleft",    "Equal value line", bty="n", lty=1, pch=NA)
  legend("bottomright", por,               bty="n", lty=0, pch=NA)

  # Continuing, consider the two data sets via their order statistics, with us also
  # breaking the joint water year coupling in some study
  peakQ <- sort( log10(D$peak_flow) ) # order statistics of the peaks
  mx1dQ <- sort( log10(D$mx1d_flow) ) # order statistics of the maxima
  # let us for demonstration purposes only, use a log-normal distribution
  # and hence the use of log10() in the previous two calls
  peak_mu <- mean( peakQ ); peak_sd <- sd( peakQ ) # mean + std deviation
  mx1d_mu <- mean( mx1dQ ); mx1d_sd <- sd( mx1dQ ) # mean + std deviation

  FF <- seq(0.005, 0.995, by=0.005) # nonexceedances for drawing curves
  qFF <- qnorm(FF) # standard normal variates to transform x-axis
  peakPP  <- rank(peakQ) / (length(peakQ) + 1) # Weibull plotting position
  mx1dPP  <- rank(mx1dQ) / (length(mx1dQ) + 1) # Weibull plotting position
  peakFFQ <- 10^qnorm(FF, mean=peak_mu, sd=peak_sd) # Log-normal curves
  mx1dFFQ <- 10^qnorm(FF, mean=mx1d_mu, sd=mx1d_sd) # Log-normal curves

  ylim <- range(c(1, peakQ / mx1dQ, log10(D$peak_flow) / log10(D$mx1d_flow)))
  x <- log10(D$mx1d_flow);     y <- log10(D$peak_flow) / log10(D$mx1d_flow)
  plot(mx1dQ, peakQ / mx1dQ, type="l", col="darkgreen", pch=21, bg="lightgreen",
       xlab="log10(water year 1-day annual maxima streamflow)", ylim=ylim, las=1,
       ylab="Ratio of annual peak to 1-day annual maxima")
  lines(x[order(x)], y[order(x)], pch=23, col="salmon4",   bg="salmon1"   )
  points(mx1dQ, peakQ / mx1dQ,    pch=21, col="darkgreen", bg="lightgreen", cex=1.3)
  points(log10(D$mx1d_flow), log10(D$peak_flow) / log10(D$mx1d_flow),
         pch=23, col="salmon4", bg="salmon1", cex=0.8)
  txt <- c("Joint ratio by separate sorting of peak and annual maxima",
           "Ratio by maintaining water year as the connection")
  legend("bottomright", txt, cex=0.8, bty="n", pch=c(21, 23), lty=c(1, 1), bg="white",
         col=c("darkgreen", "salmon4"), pt.cex=1, pt.bg=c("lightgreen", "salmon1"))
  mtext(site, line=0.25)

  plot(qnorm(FF), peakFFQ, type="n", log="y",
       ylim=range(c(10^peakQ, 10^mx1dQ, peakFFQ, mx1dFFQ)),
       xlab="Standard normal variate", ylab="Flood magnitude, in cfs")
  points(qnorm(peakPP), 10^peakQ, pch=21, col="red",  bg="white")
  points(qnorm(peakPP), 10^mx1dQ, pch=22, col="blue", bg="white")
  lines(qFF, peakFFQ, col="red",  lty=2); lines(qFF, mx1dFFQ, col="blue", lty=2)
  txt <- c("Peak streamflow frequency to data shown",
           "1-day annual maxima frequency to data shown",
           "Observed annual peak streamflow",
           "Observed 1-day annual maxima streamflow")
  legend("topleft", txt, bty="n", lty=c(2, 2, NA, NA), bg="white",
          pch=c(NA, NA, 21, 22), col=c("red", "blue", "red", "blue"))
  mtext(site, line=0.25) # }

\dontrun{
  # Continuation of the previous dontrun{} block, we now try to "use" the ratio by
  # imagining what flood frequency would look like without having the peaks themselves.
  # The context is to imagine streamflow data stemming from nonUSGS streamflow sources
  # and there are no peaks available. Perhaps one could study analog watersheds of the
  # USGS and create a statistical model to predict a correction of the 1-day to the
  # peaks based on watershed properties and derived pseudo-peaks. First two "obvious"
  # ways to look at the peak to maxima ratios by means of the ratios:
  phi_simpl <-    mean(D$ratio)                                      # arithmetic
  phi_geomu <- cumprod(D$ratio)[length(D$ratio)]^(1/length(D$ratio)) # geometric

  # Then, let us think about the coupling between the two datasets by their order
  # statistics. This is potentially more informative because structurally the frequency
  # curves themselves derived in some methods from the order statistics (L-moments or
  # product spacings).
  phi_infor <- mean(peakQ / mx1dQ) # mean ratio of the log10 flows
  phi_log10 <- mean(D$log10diff)   # mean log10 offset (phi_geomu == 10^phi_log10)

  # Continuing with simple log-normal distribution model, now correct maxima
  mx1d_mu_simpl <- mean( log10( phi_simpl * 10^mx1dQ ) )
  mx1d_sd_simpl <-   sd( log10( phi_simpl * 10^mx1dQ ) )
  mx1dFFQ_simpl <- 10^qnorm(FF, mean=mx1d_mu_simpl, sd=mx1d_sd_simpl)
  # Correct maxima by goemetric mean
  mx1d_mu_geomu <- mean( log10( phi_geomu * 10^mx1dQ ) )
  mx1d_sd_geomu <-   sd( log10( phi_geomu * 10^mx1dQ ) )
  mx1dFFQ_geomu <- 10^qnorm(FF, mean=mx1d_mu_geomu, sd=mx1d_sd_geomu)
  mx1dFFQ_log10 <- 10^(  log10(mx1dFFQ) + phi_log10  ) # This is the same as if
  # the geometric mean were used as shown for mx1dFFQ_geomu. It seems more concise
  # to think of a simple log10 offset than geometric mean.

  # Now, consider relative variation as a constant and therefore the variation
  # (standard deviation) likely needs some scaling as well. We use the CV of the
  # original sample and after the mean is rescaled, the standard deviation is too.
  cv <- sd( mx1dQ ) / mean( mx1dQ ) # coefficient of variation (CV)
  mx1d_mu_infor <- mean( mx1dQ ) *     phi_infor
  mx1d_sd_infor <-            cv * mx1d_mu_infor
  mx1dFFQ_infor <- 10^qnorm(FF, mean=mx1d_mu_infor, sd=mx1d_sd_infor)

  cols <- c("red", "blue", "turquoise3", "purple", "palegreen4", "red", "blue")
  plot(qnorm(FF), peakFFQ, type="n", log="y",
       ylim=range(c(10^peakQ, 10^mx1dQ, peakFFQ, mx1dFFQ)),
       xlab="Standard normal variate", ylab="Flood magnitude, in cfs")
  points(qnorm(peakPP), 10^peakQ, pch=21, col=cols[1], bg="white", lty=2)
  points(qnorm(peakPP), 10^mx1dQ, pch=21, col=cols[2], bg="white", lty=2)
  lines(qFF, peakFFQ, col=cols[1], lty=2); lines(qFF, mx1dFFQ, col=cols[2], lty=2)
  lines(qFF, mx1dFFQ_simpl, col=cols[3], lwd=2)
  lines(qFF, mx1dFFQ_log10, col=cols[4], lwd=2)
  lines(qFF, mx1dFFQ_infor, col=cols[5], lwd=3)
  legend("topleft", c("Peak streamflow frequency to data shown",
   "1-day annual maxima frequency to data shown", "Pseudo-peak frequency by mean-ratio",
   "Pseudo-peak frequency by log10-offset", "Pseudo-peak frequency by mean-cv-logratio",
   "Observed annual peak streamflow", "Observed 1-day annual maxima streamflow"),
         bty="n", pch=c(NA, NA, NA, NA, NA, 21, 21), pt.cex=1, bg="white",
         lwd=c(1, 1, 2, 2, 3, NA, NA), lty=c(2, 2, 1, 1, 1, NA, NA), col=cols)
  mtext(site, line=0.25) # }
}
\keyword{data retrieval}
\keyword{annual daily maxima}
\keyword{peak streamflow}
\keyword{ratio peak to 1-day maxima}

\encoding{utf8}
\name{jointPeaks}
\alias{jointPeaks}
\title{Construct Joint Peak Object and Empirical Probabilities}
\description{
Construct a joint peak object by retrieval of peak streamflows for two sites and joining the union of the peaks by water year and create empirical probabilities of the bivariate relation. This function is especially helpful when one is studying joint probabilities in peaks.
}
\usage{
jointPeaks(Asite_no="--", Bsite_no="--", appearsSystematic=FALSE, a=0,
           Adf=NULL, Bdf=NULL, ...)
}
\arguments{
  \item{Asite_no}{The \dQuote{A site} (think horizontal axis) \code{siteNumber} (\pkg{dataRetrieval} parlance) and usually required for this function as a character identifier of a streamgage and is usually an 8 digit number (USGS data tables use \code{site_no} as the column title). The default is deliberate to provide flexibility if \code{Adf} is provided;}
  \item{Bsite_no}{The \dQuote{B site} (think horizontal axis) \code{siteNumber} (\pkg{dataRetrieval} parlance) and usually required for this function  as a character identifier of a streamgage and is usually an 8 digit number (USGS data tables use \code{site_no} as the column title). The default is deliberate to provide flexibility if \code{Bdf} is provided;}
  \item{appearsSystematic}{Whereas one might want only to do the univariate marginal distributional analysis and not rely on non-systematic record without more sophisticated analysis methods of censored information, a decision is made to have the default for this argument false so as to maximize information into the deep tail of the bivariate relation between the peaks at the two sites. Remember that if copulas were to be used in analyses of the joint distribution that such operations are in probability space and not the real space of the analysis. We think better deep-tail information is available if the default is false. See also the \code{\link{splitPeakCodes}} function for more information on \code{appearsSystematic};}
  \item{a}{A value for the plotting-position formula from package \pkg{lmomco} \code{pp} function, default is \eqn{a = 0}, which returns the Weibull plotting positions;}
  \item{Adf}{An optional data frame of \dQuote{peaks} to supercede the retrieval of peaks. This feature is useful if a user want to inject another quantity, such as flood volume, into this function. The \code{Adf} is required to have a \code{water_yr}, \code{peak_dt}, \code{peak_va}, \code{appearsSystematic}, and \code{nwiscda_sqmi} or columns not actually needed such as the user could also provide \code{dev_lat_va}, \code{dev_long_va};}
  \item{Bdf}{An optional data frame of \dQuote{peaks} to supercede the retrieval of peaks. This feature is useful if a user want to inject another quantity, such as flood volume, into this function. The \code{Bdf} is required to have a \code{water_yr}, \code{peak_dt}, \code{peak_va}, \code{appearsSystematic}, and \code{nwiscda_sqmi} or columns not actually needed such as the user could also provide \code{dev_lat_va}, \code{dev_long_va}; and}
  \item{...}{Other arguments to pass (if ever needed).}
}
\value{
  A list is returned with elements:
  \item{Asite_no}{Data frame of the entire period of record for the \code{Asite_no} streamgage after the \code{\link{splitPeakCodes}} call with the addition of decimal latitude and longitude values and the contributing drainage area from the attribution information from the \code{dataRetrieval::readNWISpeak()};}
  \item{Bsite_no}{Data frame of the entire period of record for the \code{Bsite_no} streamgage after the \code{\link{splitPeakCodes}} call with the addition of decimal latitude and longitude values and the contributing drainage area from the attribution information from the \code{dataRetrieval::readNWISpeak()};}
  \item{AB}{Data frame as the joint peak object containing the pairwise matches in peaks and influenced or not by \code{appearsSystematic} argument. The data frame contains the \code{water_yr} (USGS definition of October 1 through September and numbered by the year for which it ends), both site numbers (\code{Asite_no}, \code{Bsite_no}), the contributing drainage areas from USGS database (\code{Anwiscda_sqmi}, \code{Bnwiscda_sqmi}), the date of the peaks (\code{Apeak_dt}, \code{Bpeak_dt}) (as character to thus support incomplete dates but still retained to maximize pair-wise peak counts), the peak values (\code{Apeak_va}, \code{Bpeak_va}), absolute number of days in the water year between the peaks (\code{absdays_between_pks}), nonexceedance probabilities by plotting position between the peaks (these are useful for bivariate copula analyses) (\code{U} [site A], \code{V} [site B]), and a \code{cex} column of a scaled symbol size based on the \code{absdays_between_pks}. The \code{cex} have a minimum of 1 and maximum of 2 and if the days between peak is \code{NA}, the \code{cex} is set to 0.9. This preprocessing to have the \code{cex} makes it easier for the user to produce diagnostic plots.}
}
\source{
Original \R by WHA for this package.
}
\author{W.H. Asquith}
\seealso{\code{\link{splitPeakCodes}}
}
\examples{
\dontrun{
AB <- jointPeaks("08167000", "08167500") # }
}
\keyword{Joint Peak Table}
\keyword{Joint Probability}

\encoding{utf8}
\name{CritK}
\alias{CritK}
\title{Compute Critical Value of Grubbs--Beck statistic (eta) Given Probability }
\description{
Compute critical value for the Grubbs--Beck statistic (\code{eta} = \eqn{GB_r(p)}) given a probability (p-value), which is the \dQuote{pseudo-studentized} magnitude of \eqn{r}th smallest observation. The \code{CritK} function is the same as the \eqn{GB_r(p)} quantile function.  In distribution notation, this is equivalent to saying \eqn{GB_r(F)} for nonexceedance probability \eqn{F \in (0,1)}, and cumulative distribution function \eqn{F(GB_r)} is the value that comes from \code{\link{RthOrderPValueOrthoT}}.
}
\usage{
CritK(n, r, p)
}
\arguments{
  \item{n}{The number of observations;}
  \item{r}{The number of truncated observations; and}
  \item{p}{The probability value (p-value).}
}
\value{
  The critical value of the Grubbs--Beck statistic (\code{eta} = \eqn{GB_r(p)}).
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, not \code{P3_089(R).txt}---Named: \code{CritK}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.
}
\author{ W.H. Asquith  consulting T.A. Cohn sources }
\seealso{\code{\link{critK10}}
}
\examples{
\donttest{
CritK(58, 2, .001) # CPU heavy: -3.561143}
}
\keyword{critical values}
\keyword{MGBT}

\encoding{utf8}
\name{CondMomsZ}
\alias{CondMomsZ}
\title{Conditional Moments: N.B. Moments employ only observations above Xsi }
\description{
Compute the \eqn{Z}-conditional moments (standard normal distributed moments) based on only those (\eqn{n} \eqn{-} \eqn{r}) observations above a threshold \eqn{X_{si}} for a sample size of \eqn{n} and \eqn{r} number of truncated observations. The first moment is \code{gtmoms(xsi,1)}, which is in the first returned column. The second moment is
\preformatted{
  (gtmoms(xsi,2) - gtmoms(xsi,1)^2)/(n-r)
} that is in the second returned column. Further mathematical details are available under \code{\link{gtmoms}}.
}
\usage{
CondMomsZ(n, r, xsi)
}
\arguments{
  \item{n}{The number of observations;}
  \item{r}{The number of truncated observations; and}
  \item{xsi}{The lower threshold (see \code{\link{gtmoms}}).}
}
\value{
  The value a two-column, one-row \R \code{matrix}.
}
\note{
The \code{CondMomsZ} function appears to not be used for any purpose. Only the \code{CondMomsChi2} function is needed for MGBT. The author WHA hypothesizes that TAC has the simple logic of this function constructed in long hand as needed within other functions---Rigorous inquiry of TAC's design purposes is not possible.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, \code{P3_089(R).txt}---Named \code{CondMomsZ}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.
}
\author{W.H. Asquith consulting T.A. Cohn sources}
\seealso{\code{\link{CondMomsChi2}}, \code{\link{gtmoms}}
}
\examples{
CondMomsZ(58, 2, -3.561143)
#             [,1]       [,2]
#[1,] 0.0007033727 0.01781241
}
\keyword{moments}
\keyword{moments (conditional)}
\keyword{utility functions}

\encoding{utf8}
\name{gtmoms}
\alias{gtmoms}
\title{Moments of Observations Above the Threshold }
\description{
Moments of observations above the threshold (\code{xsi}, \eqn{x_{si}}), which has been standardized to a zero mean and unit standard deviation. Define the standard normal hazard function as
\deqn{H(x) = \phi(x) / (1 - \Phi(x))\mbox{,}}
where \eqn{\phi(x)} is the standard normal density function and \eqn{\Phi(x)} is the standard normal distribution (cumulative) function. For a truncation index, \eqn{r}, define the recursion formula, \eqn{\Psi} for \code{gtmoms} as
\deqn{\Psi(x_{si}, r) = (r-1)\Psi(x_{si}, r-2) + x_{si}^{r-1}H(x_{si})\mbox{,}}
for which \eqn{\Psi(x_{si}, 0) = 1} and \eqn{\Psi(x_{si}, 1) = H(x_{si})}.
}
\usage{
gtmoms(xsi, r)
}
\arguments{
  \item{xsi}{The lower threshold; and}
  \item{r}{The number of truncated observations.}
}
\value{
  The moments.
}
\note{
\bold{AUTHOR TODO---}Note that is it not clear in TAC documentation that \eqn{X_{si}} is a scalar or vector quantity, and \code{gtmoms} is automatically vectored in the \R idioms if \eqn{X_{si}} is. Also it is not immediately clear \eqn{X_{si}} is or is not one of the order statistics. Based on MGBT operation in USGS-PeakFQ output (USGS, 2014), the threshold is \dQuote{known} no better in accuracy than one of the sample order statistics, so \eqn{X_{si}} might be written \eqn{x_{[r:n]}}. But this answer could be only restricted to a implementation in software and perhaps not theory. Finally, although the computations involve the standard normal distribution, the standardization form of \eqn{X_{si}} is not yet confirmed during the WHA porting process.
}
\source{
\code{LowOutliers_jfe(R).txt}, \code{LowOutliers_wha(R).txt}, \code{P3_089(R).txt}---Named \code{gtmoms}
}
\references{
Cohn, T.A., 2013--2016, Personal communication of original R source code: U.S. Geological Survey, Reston, Va.

U.S. Geological Survey (USGS), 2018, PeakFQ---Flood frequency analysis based on Bulletin 17B and recommendations of the Advisory Committee on Water Information (ACWI) Subcommittee on Hydrology (SOH) Hydrologic Frequency Analysis Work Group (HFAWG), version 7.2.
}
\author{ W.H. Asquith consulting T.A. Cohn sources }
\seealso{\code{\link{CondMomsChi2}}, \code{\link{gtmoms}}
}
\examples{
gtmoms(-3.561143, 2)  # Is this a meaningful example?
#[1] 0.9974952
}
\keyword{moments}
\keyword{moments (conditional)}
\keyword{utility functions}

\encoding{utf8}
\name{readNWISwatstore}
\alias{readNWISwatstore}
\title{Read NWIS WATSTORE-Formatted Period of Record Peak Streamflows and Other Germane Operations}
\description{
Read U.S. Geological Survey (USGS) National Water Information System (NWIS) (U.S. Geological Survey, 2019) peak streamflows. But with a major high-level twist what is retained or produced by the operation. This function retrieves the WATSTORE formatted version of the peak streamflow data on a streamgage by streamgage basis and this is the format especially suitable for the USGS PeakFQ software (U.S. Geological Survey, 2020). That format will reflect the period of record. This function uses direct URL pathing to NWIS to retrieve this format and write the results to the user's file system. The function does not use the \pkg{dataRetrieval} package for the WATSTORE format. Then optionally, this function uses the \pkg{dataRetrieval} package to retrieve a tab-delimited version of the peak streamflows and write those to the user's file system. Peak streamflow visualization with attention to the peak discharge qualification codes and other features is optionally made here by the \code{\link{plotPeaks}} function and optionally a portable document formatted graphics file can be written as well. This function is explicitly design around a single directory for each streamgage to store the retrieved and plotted data.
}
\usage{
readNWISwatstore(siteNumbers, path=".", dirend="d", return.progress=FALSE,
                              tabpk=TRUE, vispk=TRUE, vispdf=TRUE,
                              unlinkpath=FALSE, citeNWISdoi=TRUE,
                              all_peaks_na_okay=TRUE, ...)
}
\arguments{
  \item{siteNumbers}{USGS site number(or multiple sites) as string(s). This is usually an 8 digit number but not exclusively so and is the \code{site_no} slot in the NWIS database. The WATSTORE formatted data will be written into  file name being set to \code{site_no".pkf"} (the extension \code{".pkf"} is to be read as \dQuote{peak flows});}
  \item{path}{A directory path with default to current working directory;}
  \item{dirend}{Optional character content appended to the site number as part of directory creation. For example, if the \code{site_no} is \code{08167000} and the default \code{dirend="d"}, then if the \code{path} is the current working directory, then the full path to the directory staged to contain results of this function is \code{./08167000d};}
  \item{return.progress}{A logical to trigger the return of a two-column data frame of the \code{siteNumbers} and a logical vector of whether peaks appear to have been successfully returned or not by the loop within the function. This table could be useful in identifying which streamgages need to be removed from some parent list during initial data-mining efforts;}
  \item{tabpk}{A logical to trigger writing of the period of record of the peak streamflows in a tab-delimited format by the \code{write.table()} function with the file name being set to \code{site_no".txt"};}
  \item{vispk}{A logical to trigger \code{\link{plotPeaks}} for the peak streamflow visualization;}
  \item{vispdf}{A logical to trigger \code{pdf()} and \code{dev.off()} to create a portable document format of the peak streamflow visualization by \code{\link{plotPeaks}} file name being set to \code{site_no".pdf"};}
  \item{unlinkpath}{A logical to trigger unlinking of the full path ahead of its recreation and then operation of the side effects of this function. The unlinking is recursive, so attention to settings of \code{path} and \code{dirend} are self evident;}
  \item{citeNWISdoi}{A logical to trigger the writing of a \code{CITATION.md} file to the NWIS DOI on the date of the retrieval. It is USGS policy to use the citation to NWIS by its digital object identifier (DOI) and provide an accessed date. The \code{CITATION.md} has been written in a simple markdown format so that software repository infrastructure (such as \code{code.usgs.gov}) would markup this file to a simple web page when selected by the user in an internet browser;}
  \item{all_peaks_na_okay}{A logical controlling how \code{\link{splitPeakCodes}} will behave if there are no non-missing peaks. This is the same argument so named in that function, but has a different default for \code{readNWISwatstore} because the anticipating is that someone using this function is likely doing large batch processing and greater risk of a gage-height only streamgage being in the list; and}
  \item{...}{Additional arguments to pass to \code{\link{plotPeaks}}.}
}
\value{
  No values are returned; this function is used for its file system side effects. Those effects will include \code{".pkf"} (WATSTORE formatted peak streamflow data) and potentially include the following additional file by extension: \code{".pdf"}, \code{".txt"} (tab-delimited peak streamflow data), and \code{CITATION.md} file recording a correct policy-compliant citation to the USGS NWIS DOI number with access date as per additional arguments to this function. This function does not recognize the starting and ending period options that the \pkg{dataRetrieval} package provides because the WATSTORE format is ensured to be period of record. Therefore, the disabling of optional period retrievals ensures that the \code{".pkf"} and \code{".[txt|pdf]"} files have the same data.
}
\author{W.H. Asquith}
\references{
U.S. Geological Survey, 2019, USGS water data for the Nation: U.S. Geological Survey National Water Information System database, accessed October 11, 2019, at \doi{10.5066/F7P55KJN}.

U.S. Geological Survey, 2020, PeakFQ---Flood frequency analysis based on Bulletin 17C and recommendations of the Advisory Committee on Water Information (ACWI) Subcommittee on Hydrology (SOH) Hydrologic Frequency Analysis Work Group (HFAWG), version 7.3.
}
\seealso{\code{\link{plotPeaks}}
}
\examples{
\dontrun{
  # The dataRetrieval package provides a readNWISpeak() function but that works on
  # the tab-delimited like format structure and not the WATSTORE structure. The
  # WATSTORE structure is used by the USGS PeakFQ software and direct URL is needed
  # to acquire such data
  path <- tempdir()
  readNWISwatstore("08167000", path=path)
  # path/08167000d/08167000.[pdf|pkf|txt]
  #                CITATION.md
  # files will be created in the temporary directory }

\dontrun{
  # a gage-height only site, table created, codes split, and then the plot so
  # produced is produced with a message in the middle about all peaks missing
  readNWISwatstore("02146285", path=tempdir(), dirend="d",
            tabpk=TRUE, vispk=TRUE, vispdf=TRUE, unlinkpath=FALSE,
            citeNWISdoi=TRUE, all_peaks_na_okay=TRUE) # }

\dontrun{
  sites <- c("08171000", "02231100", "02317600")
  gotEM <- readNWISwatstore(sites, path=tempdir(),
                            return.progress=TRUE, unlinkpath=FALSE)
  # WHA On January 23, 2023, we see first this message from dataRetrieval:
  # No sites/data found using the selection criteria specified
  # WHA then readNWISwatstore() then adds clarity on which streamgage failed
  #   for streamgage '02231100'
  # WHA because this streamgage does not have any peaks entries and appears to be a
  # WHA measurement only site. The readNWISwatstore() continues and the following
  # WHA warnings occur
  # Warning messages:
  # 1: In makeWaterYear(x) : water year 2019 occurs more than once (x2)
  # 2: In plotPeaks2(pkall, codes = TRUE, showDubNA = TRUE, site = site,  :
  # site '02317600', water year 2019 occurs more than once (x2) (drawing them all)
  # WHA because streamgage 02317600 has two records that process into the 2019
  # WHA water years, which technically is not possible with annual maxima series.
  head(gotEM)
  #      site gotThePeaks
  # 1 08171000        TRUE # successful retrieval w/o identified problems
  # 2 02231100       FALSE # no peak records to retrieve
  # 3 02317600        TRUE # records retrieved but the 2019 water year is duplicated }
}
\keyword{graphics}
